@@ -12,8 +12,6 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/.
 
-from __future__ import with_statement
-
 import os
 import sys
 from pkg_resources import resource_listdir, resource_string
@@ -51,6 +49,16 @@ class DefaultWikiChecker(Formatter):
                 self.__marks.append((fullmatch.start(0), fullmatch.end(0)))
         return rv
 
+    def handle_code_block(self, line, startmatch=None):
+        prev_processor = getattr(self, 'code_processor', None)
+        try:
+            return self.__super.handle_code_block(line, startmatch)
+        finally:
+            processor = self.code_processor
+            if startmatch and processor and processor != prev_processor and \
+                    processor.error:
+                self.__marks.append((startmatch.start(0), startmatch.end(0)))
+
     def format(self, text, out=None):
         return self.__super.format(SourceWrapper(self, text), out)
 
@@ -119,11 +127,11 @@ def parse_args():
 
 
 def download_default_pages(names, prefix):
-    from httplib import HTTPConnection
+    from httplib import HTTPSConnection
     host = 'trac.edgewall.org'
     if prefix and not prefix.endswith('/'):
         prefix += '/'
-    conn = HTTPConnection(host)
+    conn = HTTPSConnection(host)
     for name in names:
         if name in ('WikiStart', 'SandBox'):
             continue
@@ -160,7 +168,7 @@ def main():
     if options.download:
         download_default_pages(args, options.prefix)
 
-    env = EnvironmentStub()
+    env = EnvironmentStub(disable=['trac.mimeview.pygments.*'])
     load_components(env)
     with env.db_transaction:
         for name in names:
