@@ -1,7 +1,7 @@
 #!/usr/bin/env python
 # -*- coding: utf-8 -*-
 #
-# Copyright (C)2006-2009 Edgewall Software
+# Copyright (C) 2006-2013 Edgewall Software
 # Copyright (C) 2006 Matthew Good <matt@matt-good.net>
 # All rights reserved.
 #
@@ -15,40 +15,45 @@
 #
 # Author: Matthew Good <matt@matt-good.net>
 
+from __future__ import print_function
+
 import errno
 import fileinput
 import sys
-from optparse import OptionParser
 from getpass import getpass
+from hashlib import md5
+from optparse import OptionParser
 
-# The md5 module is deprecated in Python 2.5
-try:
-    from hashlib import md5
-except ImportError:
-    from md5 import md5
 
 def ask_pass():
     pass1 = getpass('New password: ')
     pass2 = getpass('Re-type new password: ')
     if pass1 != pass2:
-        print >>sys.stderr, "They don't match, sorry"
+        print("They don't match, sorry", file=sys.stderr)
         sys.exit(1)
     return pass1
 
+
 def get_digest(userprefix, password=None):
-    if password == None:
+    if password is None:
         password = ask_pass()
     return make_digest(userprefix, password)
 
+
 def make_digest(userprefix, password):
     return userprefix + md5(userprefix + password).hexdigest()
 
+
 usage = "%prog [-c] [-b] passwordfile realm username"
 parser = OptionParser(usage=usage)
 parser.add_option('-c', action='store_true', dest='create', default=False,
-                  help='Create a new file')
+                  help="Create a new file")
 parser.add_option('-b', action='store_true', dest='batch', default=False,
-                  help='Batch mode, password on the commandline.')
+                  help="Batch mode, password on the commandline.")
+
+if len(sys.argv) <= 1:
+    parser.print_help()
+    sys.exit(1)
 
 opts, args = parser.parse_args()
 
@@ -59,21 +64,21 @@ try:
         filename, realm, username = args
         password = None
 except ValueError:
-    parser.error('Wrong number of arguments')
+    parser.error("Wrong number of arguments")
 
 prefix = '%s:%s:' % (username, realm)
 
 if opts.create:
     try:
         f = open(filename, 'w')
-    except EnvironmentError, e:
+    except EnvironmentError as e:
         if e.errno == errno.EACCES:
-            print >>sys.stderr, 'Unable to update file', filename
+            print("Unable to update file", filename, file=sys.stderr)
             sys.exit(1)
         else:
             raise
     try:
-        print >>f, get_digest(prefix, password)
+        print(get_digest(prefix, password), file=f)
     finally:
         f.close()
 else:
@@ -82,24 +87,21 @@ else:
         for line in fileinput.input(filename, inplace=True):
             if line.startswith(prefix):
                 if not matched:
-                    print get_digest(prefix, password)
+                    print(get_digest(prefix, password))
                 matched = True
             else:
-                print line,
+                print(line.rstrip())
         if not matched:
-            f = open(filename, 'a')
-            try:
-                print >>f, get_digest(prefix, password)
-            finally:
-                f.close()
-    except EnvironmentError, e:
+            with open(filename, 'a') as f:
+                print(get_digest(prefix, password), file=f)
+    except EnvironmentError as e:
         if e.errno == errno.ENOENT:
-            print >>sys.stderr, 'Could not open passwd file %s for reading.' \
-                                % filename
-            print >>sys.stderr, 'Use -c option to create a new one.'
+            print("Could not open passwd file %s for reading." % filename,
+                  file=sys.stderr)
+            print("Use -c option to create a new one.", file=sys.stderr)
             sys.exit(1)
         elif e.errno == errno.EACCES:
-            print >>sys.stderr, 'Unable to update file', filename
+            print("Unable to update file", filename, file=sys.stderr)
             sys.exit(1)
         else:
             raise
