@@ -1,5 +1,18 @@
 #!/usr/bin/env python
+# -*- coding: utf-8 -*-
 #
+# Copyright (C) 2005-2013 Edgewall Software
+# Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
+
 # This script completely migrates a <= 0.8.x Trac environment to use the new
 # default ticket model introduced in Trac 0.9.
 #
@@ -9,7 +22,7 @@
 #
 # Make sure to make a backup of the Trac environment before running this!
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import os
 import sys
@@ -27,8 +40,8 @@ priority_mapping = {
 
 def main():
     if len(sys.argv) < 2:
-        print >> sys.stderr, 'usage: %s /path/to/projenv' \
-                             % os.path.basename(sys.argv[0])
+        print("usage: %s /path/to/projenv" % os.path.basename(sys.argv[0]),
+              file=sys.stderr)
         sys.exit(2)
 
     env = open_environment(sys.argv[1])
