@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2007-2013 Edgewall Software
+# Copyright (C) 2007-2014 Edgewall Software
 # Copyright (C) 2007 Christian Boos <cboos@edgewall.org>
 # All rights reserved.
 #
@@ -14,43 +14,13 @@
 
 """Example macro."""
 
-revision = "$Rev: 12412 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/HelloWorld.py $"
+revision = "$Rev: 12605 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/HelloWorld.py $"
 
-#
-# The following shows the code for macro, old-style.
-#
-# The `execute` function serves no purpose other than to illustrate
-# the example, it will not be used anymore.
-#
-# ---- (ignore in your own macro) ----
-# --
-from trac.util import escape
 from trac.util.translation import cleandoc_
-
-def execute(hdf, txt, env):
-    # Currently hdf is set only when the macro is called
-    # From a wiki page
-    if hdf:
-        hdf['wiki.macro.greeting'] = 'Hello World'
-
-    # args will be `None` if the macro is called without parenthesis.
-    args = txt or 'No arguments'
-
-    # then, as `txt` comes from the user, it's important to guard against
-    # the possibility to inject malicious HTML/Javascript, by using `escape()`:
-    return 'Hello World, args = ' + escape(args)
-# --
-# ---- (ignore in your own macro) ----
-
-
-#
-# The following is the converted new-style macro
-#
-# ---- (reuse for your own macro) ----
-# --
 from trac.wiki.macros import WikiMacroBase
 
+
 class HelloWorldMacro(WikiMacroBase):
     _description = cleandoc_(
     """Simple HelloWorld macro.
@@ -61,10 +31,10 @@ class HelloWorldMacro(WikiMacroBase):
 
     The documentation of the class (i.e. what you're reading)
     will become the documentation of the macro, as shown by
-    the !MacroList macro (usually used in the TracWikiMacros page).
+    the !MacroList macro (usually used in the WikiMacros page).
     """)
 
-    def expand_macro(self, formatter, name, content):
+    def expand_macro(self, formatter, name, content, args=None):
         """Return some output that will be displayed in the Wiki content.
 
         `name` is the actual name of the macro (no surprise, here it'll be
@@ -72,10 +42,11 @@ class HelloWorldMacro(WikiMacroBase):
         `content` is the text enclosed in parenthesis at the call of the
           macro. Note that if there are ''no'' parenthesis (like in, e.g.
           [[HelloWorld]]), then `content` is `None`.
+        `args` will contain a dictionary of arguments when called using the
+          Wiki processor syntax and will be `None` if called using the
+          macro syntax.
         """
         return 'Hello World, content = ' + unicode(content)
 
     # Note that there's no need to HTML escape the returned data,
     # as the template engine (Genshi) will do it for us.
-# --
-# ---- (reuse for your own macro) ----
