@@ -16,8 +16,8 @@ from trac.core import implements,Component
 from trac.ticket.api import ITicketActionController
 from trac.perm import IPermissionRequestor
 
-revision = "$Rev: 12731 $"
-url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.0-stable/sample-plugins/workflow/DeleteTicket.py $"
+revision = "$Rev: 13452 $"
+url = "$URL: https://svn.edgewall.org/repos/trac/branches/1.2-stable/sample-plugins/workflow/DeleteTicket.py $"
 
 class DeleteTicketActionController(Component):
     """Provides the admin with a way to delete a ticket.
@@ -43,7 +43,8 @@ class DeleteTicketActionController(Component):
 
     def get_ticket_actions(self, req, ticket):
         actions = []
-        if 'TICKET_DELETE' in req.perm(ticket.resource):
+        if 'TICKET_DELETE' in req.perm(ticket.resource) \
+                and ticket.exists:
             actions.append((0,'delete'))
         return actions
 
