@@ -12,37 +12,63 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
-from ConfigParser import ConfigParser
-from copy import deepcopy
+import copy
 import os.path
+import re
+from ConfigParser import ConfigParser, ParsingError
 
 from genshi.builder import tag
+
 from trac.admin import AdminCommandError, IAdminCommandProvider
-from trac.core import *
+from trac.core import Component, ExtensionPoint, TracError, implements
 from trac.util import AtomicFile, as_bool
-from trac.util.compat import cleandoc, wait_for_file_mtime_change
-from trac.util.text import printout, to_unicode, CRLF
-from trac.util.translation import _, N_, tag_
+from trac.util.compat import OrderedDict, wait_for_file_mtime_change
+from trac.util.text import cleandoc, printout, to_unicode, to_utf8
+from trac.util.translation import N_, _, dgettext, tag_
 
 __all__ = ['Configuration', 'ConfigSection', 'Option', 'BoolOption',
            'IntOption', 'FloatOption', 'ListOption', 'ChoiceOption',
            'PathOption', 'ExtensionOption', 'OrderedExtensionsOption',
            'ConfigurationError']
 
-# Retained for backward-compatibility, use as_bool() instead
-_TRUE_VALUES = ('yes', 'true', 'enabled', 'on', 'aye', '1', 1, True)
-
 _use_default = object()
 
-def _to_utf8(basestr):
-    return to_unicode(basestr).encode('utf-8')
+
+def _getint(value):
+    return int(value or 0)
+
+
+def _getfloat(value):
+    return float(value or 0.0)
+
+
+def _getlist(value, sep, keep_empty):
+    if not value:
+        return []
+    if isinstance(value, basestring):
+        if isinstance(sep, (list, tuple)):
+            splitted = re.split('|'.join(map(re.escape, sep)), value)
+        else:
+            splitted = value.split(sep)
+        items = [item.strip() for item in splitted]
+    else:
+        items = list(value)
+    if not keep_empty:
+        items = [item for item in items if item not in (None, '')]
+    return items
+
+
+def _getdoc(option_or_section):
+    doc = to_unicode(option_or_section.__doc__)
+    if doc:
+        doc = dgettext(option_or_section.doc_domain, doc,
+                       **(option_or_section.doc_args or {}))
+    return doc
 
 
 class ConfigurationError(TracError):
     """Exception raised when a value in the configuration file is not valid."""
-    title = N_('Configuration Error')
+    title = N_("Configuration Error")
 
     def __init__(self, message=None, title=None, show_traceback=False):
         if message is None:
@@ -51,6 +77,79 @@ class ConfigurationError(TracError):
                                                  show_traceback)
 
 
+class UnicodeConfigParser(ConfigParser):
+    """A Unicode-aware version of ConfigParser. Arguments are encoded to
+    UTF-8 and return values are decoded from UTF-8.
+    """
+
+    # All of the methods of ConfigParser are overridden except
+    # `getboolean`, `getint`, `getfloat`, `defaults`, `read`, `readfp`,
+    # `optionxform` and `write`. `getboolean`, `getint` and `getfloat`
+    # call `get`, so it isn't necessary to reimplement them.
+    # The base class `RawConfigParser` doesn't inherit from `object`
+    # so we can't use `super`.
+
+    def __init__(self, **kwargs):
+        dict_type = kwargs.pop('dict_type', None) or OrderedDict
+        ConfigParser.__init__(self, dict_type=dict_type, **kwargs)
+
+    def sections(self):
+        return map(to_unicode, ConfigParser.sections(self))
+
+    def add_section(self, section):
+        section_str = to_utf8(section)
+        ConfigParser.add_section(self, section_str)
+
+    def has_section(self, section):
+        section_str = to_utf8(section)
+        return ConfigParser.has_section(self, section_str)
+
+    def options(self, section):
+        section_str = to_utf8(section)
+        return map(to_unicode, ConfigParser.options(self, section_str))
+
+    def get(self, section, option, raw=False, vars=None):
+        section_str = to_utf8(section)
+        option_str = to_utf8(option)
+        return to_unicode(ConfigParser.get(self, section_str,
+                                           option_str, raw, vars))
+
+    def items(self, section, raw=False, vars=None):
+        section_str = to_utf8(section)
+        return [(to_unicode(k), to_unicode(v))
+                for k, v in ConfigParser.items(self, section_str, raw, vars)]
+
+    def has_option(self, section, option):
+        section_str = to_utf8(section)
+        option_str = to_utf8(option)
+        return ConfigParser.has_option(self, section_str, option_str)
+
+    def set(self, section, option, value=None):
+        section_str = to_utf8(section)
+        option_str = to_utf8(option)
+        value_str = to_utf8(value) if value is not None else ''
+        ConfigParser.set(self, section_str, option_str, value_str)
+
+    def remove_option(self, section, option):
+        section_str = to_utf8(section)
+        option_str = to_utf8(option)
+        ConfigParser.remove_option(self, section_str, option_str)
+
+    def remove_section(self, section):
+        section_str = to_utf8(section)
+        ConfigParser.remove_section(self, section_str)
+
+    def __copy__(self):
+        parser = self.__class__()
+        parser._sections = copy.copy(self._sections)
+        return parser
+
+    def __deepcopy__(self, memo):
+        parser = self.__class__()
+        parser._sections = copy.deepcopy(self._sections)
+        return parser
+
+
 class Configuration(object):
     """Thin layer over `ConfigParser` from the Python standard library.
 
@@ -60,13 +159,16 @@ class Configuration(object):
     """
     def __init__(self, filename, params={}):
         self.filename = filename
-        self.parser = ConfigParser()
-        self._old_sections = {}
+        self.parser = UnicodeConfigParser()
+        self._pristine_parser = None
         self.parents = []
         self._lastmtime = 0
         self._sections = {}
         self.parse_if_needed(force=True)
 
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.filename)
+
     def __contains__(self, name):
         """Return whether the configuration contains a section of the given
         name.
@@ -79,9 +181,6 @@ class Configuration(object):
             self._sections[name] = Section(self, name)
         return self._sections[name]
 
-    def __repr__(self):
-        return '<%s %r>' % (self.__class__.__name__, self.filename)
-
     @property
     def exists(self):
         """Return boolean indicating configuration file existence.
@@ -104,8 +203,6 @@ class Configuration(object):
         or "1", this method wll return `True`, otherwise `False`.
 
         Valid default input is a string or a bool. Returns a bool.
-
-        (since Trac 0.9.3, "enabled" added in 0.11)
         """
         return self[section].getbool(key, default)
 
@@ -116,8 +213,6 @@ class Configuration(object):
         `ConfigurationError` exception is raised.
 
         Valid default input is a string or an int. Returns an int.
-
-        (since Trac 0.10)
         """
         return self[section].getint(key, default)
 
@@ -128,8 +223,6 @@ class Configuration(object):
         `ConfigurationError` exception is raised.
 
         Valid default input is a string, float or int. Returns a float.
-
-        (since Trac 0.12)
         """
         return self[section].getfloat(key, default)
 
@@ -137,13 +230,12 @@ class Configuration(object):
         """Return a list of values that have been specified as a single
         comma-separated option.
 
-        A different separator can be specified using the `sep` parameter. If
-        the `keep_empty` parameter is set to `True`, empty elements are
+        A different separator can be specified using the `sep` parameter. The
+        `sep` parameter can specify multiple values using a list or a tuple.
+        If the `keep_empty` parameter is set to `True`, empty elements are
         included in the list.
 
         Valid default input is a string or a list. Returns a string.
-
-        (since Trac 0.10)
         """
         return self[section].getlist(key, default, sep, keep_empty)
 
@@ -154,8 +246,6 @@ class Configuration(object):
         configuration file.
 
         Valid default input is a string. Returns a normalized path.
-
-        (enabled since Trac 0.11.5)
         """
         return self[section].getpath(key, default)
 
@@ -167,15 +257,16 @@ class Configuration(object):
         self[section].set(key, value)
 
     def defaults(self, compmgr=None):
-        """Returns a dictionary of the default configuration values
-        (''since 0.10'').
+        """Returns a dictionary of the default configuration values.
 
         If `compmgr` is specified, return only options declared in components
         that are enabled in the given `ComponentManager`.
         """
         defaults = {}
-        for (section, key), option in Option.get_registry(compmgr).items():
-            defaults.setdefault(section, {})[key] = option.default
+        for (section, key), option in \
+                Option.get_registry(compmgr).iteritems():
+            defaults.setdefault(section, {})[key] = \
+                option.dumps(option.default)
         return defaults
 
     def options(self, section, compmgr=None):
@@ -184,7 +275,8 @@ class Configuration(object):
 
         This includes options that have default values that haven't been
         overridden. If `compmgr` is specified, only return default option
-        values for components that are enabled in the given `ComponentManager`.
+        values for components that are enabled in the given
+        `ComponentManager`.
         """
         return self[section].options(compmgr)
 
@@ -199,7 +291,7 @@ class Configuration(object):
         options declared in components that are enabled in the given
         `ComponentManager` are returned.
         """
-        sections = set([to_unicode(s) for s in self.parser.sections()])
+        sections = set(self.parser.sections())
         for parent in self.parents:
             sections.update(parent.sections(compmgr, defaults=False))
         if defaults:
@@ -210,64 +302,53 @@ class Configuration(object):
         """Returns True if option exists in section in either the project
         trac.ini or one of the parents, or is available through the Option
         registry.
-
-        (since Trac 0.11)
         """
-        section_str = _to_utf8(section)
-        if self.parser.has_section(section_str):
-            if _to_utf8(option) in self.parser.options(section_str):
-                return True
-        for parent in self.parents:
-            if parent.has_option(section, option, defaults=False):
-                return True
-        return defaults and (section, option) in Option.registry
+        return self[section].contains(option, defaults)
 
     def save(self):
         """Write the configuration options to the primary file."""
-        if not self.filename:
-            return
 
-        # Only save options that differ from the defaults
+        all_options = {}
+        for (section, name), option in Option.get_registry().iteritems():
+            all_options.setdefault(section, {})[name] = option
+
+        def normalize(section, name, value):
+            option = all_options.get(section, {}).get(name)
+            return option.normalize(value) if option else value
+
         sections = []
         for section in self.sections():
-            section_str = _to_utf8(section)
             options = []
             for option in self[section]:
-                default_str = None
+                default = None
                 for parent in self.parents:
                     if parent.has_option(section, option, defaults=False):
-                        default_str = _to_utf8(parent.get(section, option))
+                        default = normalize(section, option,
+                                            parent.get(section, option))
                         break
-                option_str = _to_utf8(option)
-                current_str = False
-                if self.parser.has_option(section_str, option_str):
-                    current_str = self.parser.get(section_str, option_str)
-                if current_str is not False and current_str != default_str:
-                    options.append((option_str, current_str))
+                if self.parser.has_option(section, option):
+                    current = normalize(section, option,
+                                        self.parser.get(section, option))
+                    if current != default:
+                        options.append((option, current))
             if options:
-                sections.append((section_str, sorted(options)))
+                sections.append((section, sorted(options)))
+
+        # Prepare new file contents to write to disk.
+        parser = UnicodeConfigParser()
+        for section, options in sections:
+            parser.add_section(section)
+            for key, val in options:
+                parser.set(section, key, val)
 
-        # At this point, all the strings in `sections` are UTF-8 encoded `str`
         try:
-            wait_for_file_mtime_change(self.filename)
-            with AtomicFile(self.filename, 'w') as fileobj:
-                fileobj.write('# -*- coding: utf-8 -*-\n\n')
-                for section_str, options in sections:
-                    fileobj.write('[%s]\n' % section_str)
-                    section = to_unicode(section_str)
-                    for key_str, val_str in options:
-                        if to_unicode(key_str) in self[section].overridden:
-                            fileobj.write('# %s = <inherited>\n' % key_str)
-                        else:
-                            val_str = val_str.replace(CRLF, '\n') \
-                                             .replace('\n', '\n ')
-                            fileobj.write('%s = %s\n' % (key_str, val_str))
-                    fileobj.write('\n')
-            self._old_sections = deepcopy(self.parser._sections)
+            self._write(parser)
         except Exception:
             # Revert all changes to avoid inconsistencies
-            self.parser._sections = deepcopy(self._old_sections)
+            self.parser = copy.deepcopy(self._pristine_parser)
             raise
+        else:
+            self._pristine_parser = copy.deepcopy(self.parser)
 
     def parse_if_needed(self, force=False):
         if not self.filename or not self.exists:
@@ -276,23 +357,19 @@ class Configuration(object):
         changed = False
         modtime = os.path.getmtime(self.filename)
         if force or modtime != self._lastmtime:
-            self.parser._sections = {}
-            if not self.parser.read(self.filename):
-                raise TracError(_("Error reading '%(file)s', make sure it is "
-                                  "readable.", file=self.filename))
+            self.parser = UnicodeConfigParser()
+            try:
+                if not self.parser.read(self.filename):
+                    raise TracError(_("Error reading '%(file)s', make sure "
+                                      "it is readable.", file=self.filename))
+            except ParsingError as e:
+                raise TracError(e)
             self._lastmtime = modtime
-            self._old_sections = deepcopy(self.parser._sections)
+            self._pristine_parser = copy.deepcopy(self.parser)
             changed = True
 
         if changed:
-            self.parents = []
-            if self.parser.has_option('inherit', 'file'):
-                for filename in self.parser.get('inherit', 'file').split(','):
-                    filename = to_unicode(filename.strip())
-                    if not os.path.isabs(filename):
-                        filename = os.path.join(os.path.dirname(self.filename),
-                                                filename)
-                    self.parents.append(Configuration(filename))
+            self.parents = self._get_parents()
         else:
             for parent in self.parents:
                 changed |= parent.parse_if_needed(force=force)
@@ -306,38 +383,72 @@ class Configuration(object):
                 and os.access(self.filename, os.W_OK):
             wait_for_file_mtime_change(self.filename)
 
-    def set_defaults(self, compmgr=None):
+    def set_defaults(self, compmgr=None, component=None):
         """Retrieve all default values and store them explicitly in the
         configuration, so that they can be saved to file.
 
-        Values already set in the configuration are not overridden.
+        Values already set in the configuration are not overwritten.
         """
-        for (section, name), option in Option.get_registry(compmgr).items():
-            if not self.parser.has_option(_to_utf8(section), _to_utf8(name)):
-                value = option.default
-                if any(parent[section].contains(name, defaults=False)
-                       for parent in self.parents):
-                    value = None
-                if value is not None:
-                    value = option.dumps(value)
+        def set_option_default(option):
+            section = option.section
+            name = option.name
+            if not self.has_option(section, name, defaults=False):
+                value = option.dumps(option.default)
                 self.set(section, name, value)
 
+        if component:
+            if component.endswith('.*'):
+                component = component[:-2]
+            component = component.lower().split('.')
+            from trac.core import ComponentMeta
+            for cls in ComponentMeta._components:
+                clsname = (cls.__module__ + '.' + cls.__name__).lower() \
+                                                               .split('.')
+                if clsname[:len(component)] == component:
+                    for option in cls.__dict__.itervalues():
+                        if isinstance(option, Option):
+                            set_option_default(option)
+        else:
+            for option in Option.get_registry(compmgr).itervalues():
+                set_option_default(option)
+
+    def _get_parents(self):
+        _parents = []
+        if self.parser.has_option('inherit', 'file'):
+            for filename in self.parser.get('inherit', 'file').split(','):
+                filename = filename.strip()
+                if not os.path.isabs(filename):
+                    filename = os.path.join(os.path.dirname(self.filename),
+                                            filename)
+                _parents.append(Configuration(filename))
+        return _parents
+
+    def _write(self, parser):
+        if not self.filename:
+            return
+        wait_for_file_mtime_change(self.filename)
+        with AtomicFile(self.filename, 'w') as fd:
+            fd.writelines(['# -*- coding: utf-8 -*-\n', '\n'])
+            parser.write(fd)
+
 
 class Section(object):
     """Proxy for a specific configuration section.
 
     Objects of this class should not be instantiated directly.
     """
-    __slots__ = ['config', 'name', 'overridden', '_cache']
+    __slots__ = ['config', 'name', '_cache']
 
     def __init__(self, config, name):
         self.config = config
         self.name = name
-        self.overridden = {}
         self._cache = {}
 
+    def __repr__(self):
+        return '<%s [%s]>' % (self.__class__.__name__, self.name)
+
     def contains(self, key, defaults=True):
-        if self.config.parser.has_option(_to_utf8(self.name), _to_utf8(key)):
+        if self.config.parser.has_option(self.name, key):
             return True
         for parent in self.config.parents:
             if parent[self.name].contains(key, defaults=False):
@@ -353,10 +464,8 @@ class Section(object):
         components that are enabled in the given `ComponentManager`.
         """
         options = set()
-        name_str = _to_utf8(self.name)
-        if self.config.parser.has_section(name_str):
-            for option_str in self.config.parser.options(name_str):
-                option = to_unicode(option_str)
+        if self.config.parser.has_section(self.name):
+            for option in self.config.parser.options(self.name):
                 options.add(option.lower())
                 yield option
         for parent in self.config.parents:
@@ -366,15 +475,12 @@ class Section(object):
                     options.add(loption)
                     yield option
         if defaults:
-            for section, option in Option.get_registry(compmgr).keys():
+            for section, option in Option.get_registry(compmgr).iterkeys():
                 if section == self.name and option.lower() not in options:
                     yield option
 
     __iter__ = iterate
 
-    def __repr__(self):
-        return '<%s [%s]>' % (self.__class__.__name__, self.name)
-
     def get(self, key, default=''):
         """Return the value of the specified option.
 
@@ -383,10 +489,8 @@ class Section(object):
         cached = self._cache.get(key, _use_default)
         if cached is not _use_default:
             return cached
-        name_str = _to_utf8(self.name)
-        key_str = _to_utf8(key)
-        if self.config.parser.has_option(name_str, key_str):
-            value = self.config.parser.get(name_str, key_str)
+        if self.config.parser.has_option(self.name, key):
+            value = self.config.parser.get(self.name, key)
         else:
             for parent in self.config.parents:
                 value = parent[self.name].get(key, _use_default)
@@ -395,24 +499,21 @@ class Section(object):
             else:
                 if default is not _use_default:
                     option = Option.registry.get((self.name, key))
-                    value = option.default if option else _use_default
+                    value = option.dumps(option.default) if option \
+                                                         else _use_default
                 else:
                     value = _use_default
         if value is _use_default:
             return default
-        if not value:
-            value = u''
-        elif isinstance(value, basestring):
-            value = to_unicode(value)
         self._cache[key] = value
         return value
 
     def getbool(self, key, default=''):
         """Return the value of the specified option as boolean.
 
-        This method returns `True` if the option value is one of "yes", "true",
-        "enabled", "on", or non-zero numbers, ignoring case. Otherwise `False`
-        is returned.
+        This method returns `True` if the option value is one of "yes",
+        "true", "enabled", "on", or non-zero numbers, ignoring case.
+        Otherwise `False` is returned.
 
         Valid default input is a string or a bool. Returns a bool.
         """
@@ -427,14 +528,13 @@ class Section(object):
         Valid default input is a string or an int. Returns an int.
         """
         value = self.get(key, default)
-        if not value:
-            return 0
         try:
-            return int(value)
+            return _getint(value)
         except ValueError:
             raise ConfigurationError(
-                    _('[%(section)s] %(entry)s: expected integer, got %(value)s',
-                      section=self.name, entry=key, value=repr(value)))
+                    _('[%(section)s] %(entry)s: expected integer,'
+                      ' got %(value)s', section=self.name, entry=key,
+                      value=repr(value)))
 
     def getfloat(self, key, default=''):
         """Return the value of the specified option as float.
@@ -445,35 +545,26 @@ class Section(object):
         Valid default input is a string, float or int. Returns a float.
         """
         value = self.get(key, default)
-        if not value:
-            return 0.0
         try:
-            return float(value)
+            return _getfloat(value)
         except ValueError:
             raise ConfigurationError(
-                    _('[%(section)s] %(entry)s: expected float, got %(value)s',
-                      section=self.name, entry=key, value=repr(value)))
+                    _('[%(section)s] %(entry)s: expected float,'
+                      ' got %(value)s', section=self.name, entry=key,
+                      value=repr(value)))
 
     def getlist(self, key, default='', sep=',', keep_empty=True):
         """Return a list of values that have been specified as a single
         comma-separated option.
 
-        A different separator can be specified using the `sep` parameter. If
-        the `keep_empty` parameter is set to `False`, empty elements are omitted
-        from the list.
+        A different separator can be specified using the `sep` parameter. The
+        `sep` parameter can specify multiple values using a list or a tuple.
+        If the `keep_empty` parameter is set to `True`, empty elements are
+        included in the list.
 
         Valid default input is a string or a list. Returns a list.
         """
-        value = self.get(key, default)
-        if not value:
-            return []
-        if isinstance(value, basestring):
-            items = [item.strip() for item in value.split(sep)]
-        else:
-            items = list(value)
-        if not keep_empty:
-            items = [item for item in items if item not in (None, '')]
-        return items
+        return _getlist(self.get(key, default), sep, keep_empty)
 
     def getpath(self, key, default=''):
         """Return the value of the specified option as a path, relative to
@@ -504,26 +595,19 @@ class Section(object):
         These changes are not persistent unless saved with `save()`.
         """
         self._cache.pop(key, None)
-        name_str = _to_utf8(self.name)
-        key_str = _to_utf8(key)
-        if not self.config.parser.has_section(name_str):
-            self.config.parser.add_section(name_str)
-        if value is None:
-            self.overridden[key] = True
-            value_str = ''
-        else:
-            value_str = _to_utf8(value)
-        return self.config.parser.set(name_str, key_str, value_str)
+        if not self.config.parser.has_section(self.name):
+            self.config.parser.add_section(self.name)
+        return self.config.parser.set(self.name, key, value)
 
     def remove(self, key):
         """Delete a key from this section.
 
-        Like for `set()`, the changes won't persist until `save()` gets called.
+        Like for `set()`, the changes won't persist until `save()` gets
+        called.
         """
-        name_str = _to_utf8(self.name)
-        if self.config.parser.has_section(name_str):
+        if self.config.parser.has_section(self.name):
             self._cache.pop(key, None)
-            self.config.parser.remove_option(_to_utf8(self.name), _to_utf8(key))
+            self.config.parser.remove_option(self.name, key)
 
 
 def _get_registry(cls, compmgr=None):
@@ -557,17 +641,18 @@ class ConfigSection(object):
         """Return the section registry, as a `dict` mapping section names to
         `ConfigSection` objects.
 
-        If `compmgr` is specified, only return sections for components that are
-        enabled in the given `ComponentManager`.
+        If `compmgr` is specified, only return sections for components that
+        are enabled in the given `ComponentManager`.
         """
         return _get_registry(ConfigSection, compmgr)
 
-    def __init__(self, name, doc, doc_domain='tracini'):
+    def __init__(self, name, doc, doc_domain='tracini', doc_args=None):
         """Create the configuration section."""
         self.name = name
         self.registry[self.name] = self
         self.__doc__ = cleandoc(doc)
         self.doc_domain = doc_domain
+        self.doc_args = doc_args
 
     def __get__(self, instance, owner):
         if instance is None:
@@ -579,6 +664,11 @@ class ConfigSection(object):
     def __repr__(self):
         return '<%s [%s]>' % (self.__class__.__name__, self.name)
 
+    @property
+    def doc(self):
+        """Return localized document of the section"""
+        return _getdoc(self)
+
 
 class Option(object):
     """Descriptor for configuration options."""
@@ -599,7 +689,7 @@ class Option(object):
         return _get_registry(Option, compmgr)
 
     def __init__(self, section, name, default=None, doc='',
-                 doc_domain='tracini'):
+                 doc_domain='tracini', doc_args=None):
         """Create the configuration option.
 
         @param section: the name of the configuration section this option
@@ -610,10 +700,11 @@ class Option(object):
         """
         self.section = section
         self.name = name
-        self.default = default
+        self.default = self.normalize(default)
         self.registry[(self.section, self.name)] = self
         self.__doc__ = cleandoc(doc)
         self.doc_domain = doc_domain
+        self.doc_args = doc_args
 
     def __get__(self, instance, owner):
         if instance is None:
@@ -628,8 +719,13 @@ class Option(object):
         raise AttributeError(_("Setting attribute is not allowed."))
 
     def __repr__(self):
-        return '<%s [%s] "%s">' % (self.__class__.__name__, self.section,
-                                   self.name)
+        return '<%s [%s] %r>' % (self.__class__.__name__, self.section,
+                                 self.name)
+
+    @property
+    def doc(self):
+        """Return localized document of the option"""
+        return _getdoc(self)
 
     def dumps(self, value):
         """Return the value as a string to write to a trac.ini file"""
@@ -643,6 +739,10 @@ class Option(object):
             return value
         return to_unicode(value)
 
+    def normalize(self, value):
+        """Normalize the given value to write to a trac.ini file"""
+        return self.dumps(value)
+
 
 class BoolOption(Option):
     """Descriptor for boolean configuration options."""
@@ -650,6 +750,11 @@ class BoolOption(Option):
     def accessor(self, section, name, default):
         return section.getbool(name, default)
 
+    def normalize(self, value):
+        if value not in (True, False):
+            value = as_bool(value)
+        return self.dumps(value)
+
 
 class IntOption(Option):
     """Descriptor for integer configuration options."""
@@ -657,6 +762,13 @@ class IntOption(Option):
     def accessor(self, section, name, default):
         return section.getint(name, default)
 
+    def normalize(self, value):
+        try:
+            value = _getint(value)
+        except ValueError:
+            pass
+        return self.dumps(value)
+
 
 class FloatOption(Option):
     """Descriptor for float configuration options."""
@@ -664,6 +776,13 @@ class FloatOption(Option):
     def accessor(self, section, name, default):
         return section.getfloat(name, default)
 
+    def normalize(self, value):
+        try:
+            value = _getfloat(value)
+        except ValueError:
+            pass
+        return self.dumps(value)
+
 
 class ListOption(Option):
     """Descriptor for configuration options that contain multiple values
@@ -671,19 +790,26 @@ class ListOption(Option):
     """
 
     def __init__(self, section, name, default=None, sep=',', keep_empty=False,
-                 doc='', doc_domain='tracini'):
-        Option.__init__(self, section, name, default, doc, doc_domain)
+                 doc='', doc_domain='tracini', doc_args=None):
         self.sep = sep
         self.keep_empty = keep_empty
+        Option.__init__(self, section, name, default, doc, doc_domain,
+                        doc_args)
 
     def accessor(self, section, name, default):
         return section.getlist(name, default, self.sep, self.keep_empty)
 
     def dumps(self, value):
         if isinstance(value, (list, tuple)):
-            return self.sep.join(Option.dumps(self, v) or '' for v in value)
+            sep = self.sep
+            if isinstance(sep, (list, tuple)):
+                sep = sep[0]
+            return sep.join(Option.dumps(self, v) or '' for v in value)
         return Option.dumps(self, value)
 
+    def normalize(self, value):
+        return self.dumps(_getlist(value, self.sep, self.keep_empty))
+
 
 class ChoiceOption(Option):
     """Descriptor for configuration options providing a choice among a list
@@ -692,10 +818,11 @@ class ChoiceOption(Option):
     The default value is the first choice in the list.
     """
 
-    def __init__(self, section, name, choices, doc='', doc_domain='tracini'):
-        Option.__init__(self, section, name, _to_utf8(choices[0]), doc,
-                        doc_domain)
-        self.choices = set(_to_utf8(choice).strip() for choice in choices)
+    def __init__(self, section, name, choices, doc='', doc_domain='tracini',
+                 doc_args=None):
+        Option.__init__(self, section, name, to_unicode(choices[0]), doc,
+                        doc_domain, doc_args)
+        self.choices = set(to_unicode(c).strip() for c in choices)
 
     def accessor(self, section, name, default):
         value = section.get(name, default)
@@ -726,8 +853,9 @@ class ExtensionOption(Option):
     active components implementing the interface."""
 
     def __init__(self, section, name, interface, default=None, doc='',
-                 doc_domain='tracini'):
-        Option.__init__(self, section, name, default, doc, doc_domain)
+                 doc_domain='tracini', doc_args=None):
+        Option.__init__(self, section, name, default, doc, doc_domain,
+                        doc_args)
         self.xtnpt = ExtensionPoint(interface)
 
     def __get__(self, instance, owner):
@@ -742,22 +870,24 @@ class ExtensionOption(Option):
                  "interface named %(implementation)s. Please check "
                  "that the Component is enabled or update the option "
                  "%(option)s in trac.ini.",
-                 interface=tag.tt(self.xtnpt.interface.__name__),
-                 implementation=tag.tt(value),
-                 option=tag.tt("[%s] %s" % (self.section, self.name))))
+                 interface=tag.code(self.xtnpt.interface.__name__),
+                 implementation=tag.code(value),
+                 option=tag.code("[%s] %s" % (self.section, self.name))))
 
 
 class OrderedExtensionsOption(ListOption):
-    """A comma separated, ordered, list of components implementing `interface`.
-    Can be empty.
+    """A comma separated, ordered, list of components implementing
+    `interface`. Can be empty.
 
     If `include_missing` is true (the default) all components implementing the
-    interface are returned, with those specified by the option ordered first."""
+    interface are returned, with those specified by the option ordered first.
+    """
 
     def __init__(self, section, name, interface, default=None,
-                 include_missing=True, doc='', doc_domain='tracini'):
+                 include_missing=True, doc='', doc_domain='tracini',
+                 doc_args=None):
         ListOption.__init__(self, section, name, default, doc=doc,
-                            doc_domain=doc_domain)
+                            doc_domain=doc_domain, doc_args=doc_args)
         self.xtnpt = ExtensionPoint(interface)
         self.include_missing = include_missing
 
@@ -778,11 +908,11 @@ class OrderedExtensionsOption(ListOption):
                      "interface named %(implementation)s. Please check "
                      "that the Component is enabled or update the option "
                      "%(option)s in trac.ini.",
-                     interface=tag.tt(self.xtnpt.interface.__name__),
+                     interface=tag.code(self.xtnpt.interface.__name__),
                      implementation=tag(
-                         (', ' if idx != 0 else None, tag.tt(impl))
+                         (', ' if idx != 0 else None, tag.code(impl))
                          for idx, impl in enumerate(not_found)),
-                     option=tag.tt("[%s] %s" % (self.section, self.name))))
+                     option=tag.code("[%s] %s" % (self.section, self.name))))
 
         def compare(x, y):
             x, y = x.__class__.__name__, y.__class__.__name__
@@ -822,22 +952,56 @@ class ConfigurationAdmin(Component):
     def _do_get(self, section, option):
         if not self.config.has_option(section, option):
             raise AdminCommandError(
-                _("Option '%(option)s' doesn't exist in section '%(section)s'",
-                  option=option, section=section))
+                _("Option '%(option)s' doesn't exist in section"
+                  " '%(section)s'", option=option, section=section))
         printout(self.config.get(section, option))
 
     def _do_set(self, section, option, value):
         self.config.set(section, option, value)
+        if section == 'components' and as_bool(value):
+            self.config.set_defaults(component=option)
         self.config.save()
         if section == 'inherit' and option == 'file':
-            self.config.parse_if_needed(force=True) # Full reload
+            self.config.parse_if_needed(force=True)  # Full reload
 
     def _do_remove(self, section, option):
         if not self.config.has_option(section, option):
             raise AdminCommandError(
-                _("Option '%(option)s' doesn't exist in section '%(section)s'",
-                  option=option, section=section))
+                _("Option '%(option)s' doesn't exist in section"
+                  " '%(section)s'", option=option, section=section))
         self.config.remove(section, option)
         self.config.save()
         if section == 'inherit' and option == 'file':
-            self.config.parse_if_needed(force=True) # Full reload
+            self.config.parse_if_needed(force=True)  # Full reload
+
+
+def get_configinfo(env):
+    """Returns a list of dictionaries containing the `name` and `options`
+    of each configuration section. The value of `options` is a list of
+    dictionaries containing the `name`, `value` and `modified` state of
+    each configuration option. The `modified` value is True if the value
+    differs from its default.
+
+    :since: version 1.1.2
+    """
+    all_options = {}
+    for (section, name), option in \
+            Option.get_registry(env.compmgr).iteritems():
+        all_options.setdefault(section, {})[name] = option
+    sections = []
+    for section in env.config.sections(env.compmgr):
+        options = []
+        for name, value in env.config.options(section, env.compmgr):
+            registered = all_options.get(section, {}).get(name)
+            if registered:
+                default = registered.default
+                normalized = registered.normalize(value)
+            else:
+                default = u''
+                normalized = unicode(value)
+            options.append({'name': name, 'value': value,
+                            'modified': normalized != default})
+        options.sort(key=lambda o: o['name'])
+        sections.append({'name': section, 'options': options})
+    sections.sort(key=lambda s: s['name'])
+    return sections
