@@ -19,8 +19,6 @@ we also modify the standard `distutils.command.build` and
 for compiling catalogs are issued upon install.
 """
 
-from __future__ import with_statement
-
 from StringIO import StringIO
 from itertools import izip
 import os
@@ -73,7 +71,7 @@ try:
         `cleandoc_keywords` option: a list of keywords to clean up the
         extracted messages with `cleandoc`.
         """
-        from trac.util.compat import cleandoc
+        from trac.util.text import cleandoc
 
         funcname = lineno = message_lineno = None
         kwargs_maps = func_kwargs_map = None
@@ -85,8 +83,8 @@ try:
         in_def = in_translator_comments = False
         comment_tag = None
 
-        encoding = parse_encoding(fileobj) \
-                   or options.get('encoding', 'iso-8859-1')
+        encoding = str(parse_encoding(fileobj) or
+                       options.get('encoding', 'iso-8859-1'))
         kwargs_maps = _DEFAULT_KWARGS_MAPS.copy()
         if 'kwargs_maps' in options:
             kwargs_maps.update(options['kwargs_maps'])
@@ -373,7 +371,7 @@ try:
             errors = [e for e in message.check(catalog)]
             try:
                 check_genshi_markup(catalog, message)
-            except TranslationError, e:
+            except TranslationError as e:
                 errors.append(e)
             return errors
 
@@ -411,7 +409,7 @@ try:
         try:
             return set([idx for idx, text in parse_msg(message)
                             if idx > 0])
-        except Exception, e:
+        except Exception as e:
             raise TranslationError('cannot parse message (%s: %s)' % \
                                    (e.__class__.__name__, unicode(e)))
 
@@ -466,6 +464,17 @@ try:
                 self.run_command('compile_catalog')
             def run(self):
                 self.l10n_run()
+                # When bdist_egg is called on distribute 0.6.29 and later, the
+                # egg file includes no *.mo and *.js files which are generated
+                # in l10n_run() method.
+                # We remove build_py.data_files property to re-compute in order
+                # to avoid the issue (#11640).
+                build_py = self.get_finalized_command('build_py')
+                if 'data_files' in build_py.__dict__ and \
+                   not any(any(name.endswith('.mo') for name in filenames)
+                           for pkg, src_dir, build_dir, filenames
+                           in build_py.data_files):
+                    del build_py.__dict__['data_files']
                 _install_lib.run(self)
         return build, install_lib
 
