@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2004-2010 Edgewall Software
+# Copyright (C) 2004-2014 Edgewall Software
 # Copyright (C) 2004 Daniel Lundin <daniel@edgewall.com>
 # Copyright (C) 2005-2006 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006-2007 Christian Boos <cboos@edgewall.org>
@@ -61,6 +61,7 @@ that can be `read()`.
 
 import re
 from StringIO import StringIO
+from collections import namedtuple
 
 from genshi import Markup, Stream
 from genshi.core import TEXT, START, END, START_NS, END_NS
@@ -68,16 +69,18 @@ from genshi.builder import Fragment, tag
 from genshi.input import HTMLParser
 
 from trac.config import IntOption, ListOption, Option
-from trac.core import *
+from trac.core import Component, ExtensionPoint, Interface, TracError, \
+                      implements
 from trac.resource import Resource
 from trac.util import Ranges, content_disposition
-from trac.util.text import exception_to_unicode, to_utf8, to_unicode
+from trac.util.text import exception_to_unicode, to_unicode
 from trac.util.translation import _, tag_
 
 
 __all__ = ['Context', 'Mimeview', 'RenderingContext', 'get_mimetype',
            'is_binary', 'detect_unicode', 'content_to_unicode', 'ct_mimetype']
 
+
 class RenderingContext(object):
     """
     A rendering context specifies ''how'' the content should be rendered.
@@ -107,7 +110,7 @@ class RenderingContext(object):
     set up nested contexts for each matching ticket that will be used for
     rendering the ticket descriptions.
 
-    :since: version 0.11
+    :since: version 1.0
     """
 
     def __init__(self, resource, href=None, perm=None):
@@ -123,7 +126,7 @@ class RenderingContext(object):
         to the given `resource` so that fine-grained permission checks will
         apply to that.
         """
-        self.parent = None #: The parent context, if any
+        self.parent = None  #: The parent context, if any
         self.resource = resource
         self.href = href
         self.perm = perm(resource) if perm and resource else perm
@@ -131,7 +134,9 @@ class RenderingContext(object):
 
     @staticmethod
     def from_request(*args, **kwargs):
-        """:deprecated: since 1.0, use `web_context` instead."""
+        """:deprecated: since 1.0, use `web_context` instead. Will be removed
+                        in release 1.3.1.
+        """
         from trac.web.chrome import web_context
         return web_context(*args, **kwargs)
 
@@ -139,7 +144,7 @@ class RenderingContext(object):
         path = []
         context = self
         while context:
-            if context.resource.realm: # skip toplevel resource
+            if context.resource.realm:  # skip toplevel resource
                 path.append(repr(context.resource))
             context = context.parent
         return '<%s %s>' % (type(self).__name__, ' - '.join(reversed(path)))
@@ -211,7 +216,7 @@ class RenderingContext(object):
     # The keys are strings, but the values could be anything.
     #
     # In nested contexts, the hints are inherited from their parent context,
-    # unless overriden locally.
+    # unless overridden locally.
 
     def set_hints(self, **keyvalues):
         """Set rendering hints for this rendering context.
@@ -279,63 +284,71 @@ class RenderingContext(object):
 
 
 class Context(RenderingContext):
-    """:deprecated: old name kept for compatibility, use `RenderingContext`."""
+    """
+    :deprecated: since 1.0, use `RenderingContext` instead. `Context` is
+                 kept for compatibility and will be removed release 1.3.1.
+    """
 
 
 # Some common MIME types and their associated keywords and/or file extensions
 
 KNOWN_MIME_TYPES = {
-    'application/javascript': 'js',
-    'application/msword':     'doc dot',
-    'application/pdf':        'pdf',
-    'application/postscript': 'ps',
-    'application/rtf':        'rtf',
-    'application/x-sh':       'sh',
-    'application/x-csh':      'csh',
-    'application/x-troff':    'nroff roff troff',
-    'application/x-yaml':     'yml yaml',
+    'application/javascript':  'js',
+    'application/msword':      'doc dot',
+    'application/pdf':         'pdf',
+    'application/postscript':  'ps',
+    'application/rtf':         'rtf',
+    'application/x-dos-batch': 'bat batch cmd dos',
+    'application/x-sh':        'sh',
+    'application/x-csh':       'csh',
+    'application/x-genshi':    'genshi',
+    'application/x-troff':     'nroff roff troff',
+    'application/x-yaml':      'yml yaml',
 
-    'application/rss+xml':    'rss',
-    'application/xsl+xml':    'xsl',
-    'application/xslt+xml':   'xslt',
+    'application/rss+xml':     'rss',
+    'application/xsl+xml':     'xsl',
+    'application/xslt+xml':    'xslt',
 
-    'image/x-icon':           'ico',
-    'image/svg+xml':          'svg',
+    'image/x-icon':            'ico',
+    'image/svg+xml':           'svg',
 
-    'model/vrml':             'vrml wrl',
+    'model/vrml':              'vrml wrl',
 
-    'text/css':               'css',
-    'text/html':              'html htm',
-    'text/plain':             'txt TXT text README INSTALL '
-                              'AUTHORS COPYING ChangeLog RELEASE',
-    'text/xml':               'xml',
+    'text/css':                'css',
+    'text/html':               'html htm',
+    'text/plain':              'txt TXT text README INSTALL '
+                               'AUTHORS COPYING ChangeLog RELEASE',
+    'text/xml':                'xml',
 
     # see also TEXT_X_TYPES below
-    'text/x-csrc':            'c xs',
-    'text/x-chdr':            'h',
-    'text/x-c++src':          'cc CC cpp C c++ C++',
-    'text/x-c++hdr':          'hh HH hpp H',
-    'text/x-csharp':          'cs c# C#',
-    'text/x-diff':            'patch',
-    'text/x-eiffel':          'e',
-    'text/x-elisp':           'el',
-    'text/x-fortran':         'f',
-    'text/x-haskell':         'hs',
-    'text/x-ini':             'ini cfg',
-    'text/x-objc':            'm mm',
-    'text/x-ocaml':           'ml mli',
-    'text/x-makefile':        'make mk Makefile GNUMakefile',
-    'text/x-pascal':          'pas',
-    'text/x-perl':            'pl pm PL',
-    'text/x-php':             'php3 php4',
-    'text/x-python':          'py',
-    'text/x-pyrex':           'pyx',
-    'text/x-ruby':            'rb',
-    'text/x-scheme':          'scm',
-    'text/x-textile':         'txtl',
-    'text/x-vba':             'vb vba bas',
-    'text/x-verilog':         'v',
-    'text/x-vhdl':            'vhd',
+    'text/x-apacheconf':       'apache',
+    'text/x-csrc':             'c xs',
+    'text/x-chdr':             'h',
+    'text/x-c++src':           'cc CC cpp C c++ C++',
+    'text/x-c++hdr':           'hh HH hpp H',
+    'text/x-csharp':           'cs c# C#',
+    'text/x-diff':             'patch',
+    'text/x-eiffel':           'e',
+    'text/x-elisp':            'el',
+    'text/x-fortran':          'f',
+    'text/x-haskell':          'hs',
+    'text/x-ini':              'ini cfg',
+    'text/x-nginx-conf':       'nginx',
+    'text/x-objc':             'm mm',
+    'text/x-ocaml':            'ml mli',
+    'text/x-makefile':         'make mk Makefile GNUMakefile',
+    'text/x-pascal':           'pas',
+    'text/x-perl':             'pl pm PL',
+    'text/x-php':              'php3 php4',
+    'text/x-python':           'py',
+    'text/x-python-doctest':   'pycon',
+    'text/x-pyrex':            'pyx',
+    'text/x-ruby':             'rb',
+    'text/x-scheme':           'scm',
+    'text/x-textile':          'txtl',
+    'text/x-vba':              'vb vba bas',
+    'text/x-verilog':          'v',
+    'text/x-vhdl':             'vhd',
 }
 for t in KNOWN_MIME_TYPES.keys():
     types = KNOWN_MIME_TYPES[t].split()
@@ -368,6 +381,7 @@ MODE_RE = re.compile(r"""
     | vim:.*?(?:syntax|filetype|ft)=(\w+)   # 4. look for VIM's syntax=<n>
     """, re.VERBOSE)
 
+
 def get_mimetype(filename, content=None, mime_map=MIME_MAP,
                  mime_map_patterns={}):
     """Guess the most probable MIME type of a file with the given name.
@@ -407,10 +421,12 @@ def get_mimetype(filename, content=None, mime_map=MIME_MAP,
                     return 'application/octet-stream'
         return mimetype
 
+
 def ct_mimetype(content_type):
     """Return the mimetype part of a content type."""
     return (content_type or '').split(';')[0].strip()
 
+
 def is_binary(data):
     """Detect binary content by checking the first thousand bytes for zeroes.
 
@@ -420,6 +436,7 @@ def is_binary(data):
         return False
     return '\0' in data[:1000]
 
+
 def detect_unicode(data):
     """Detect different unicode charsets by looking for BOMs (Byte Order Mark).
 
@@ -434,6 +451,7 @@ def detect_unicode(data):
     else:
         return None
 
+
 def content_to_unicode(env, content, mimetype):
     """Retrieve an `unicode` object from a `content` to be previewed.
 
@@ -579,7 +597,10 @@ class IContentConverter(Interface):
     def convert_content(req, mimetype, content, key):
         """Convert the given content from mimetype to the output MIME type
         represented by key. Returns a tuple in the form (content,
-        output_mime_type) or None if conversion is not possible."""
+        output_mime_type) or None if conversion is not possible.
+
+        content must be a `str` instance or an iterable instance which
+        iterates `str` instances."""
 
 
 class Content(object):
@@ -614,30 +635,33 @@ class Mimeview(Component):
         """Charset to be used when in doubt.""")
 
     tab_width = IntOption('mimeviewer', 'tab_width', 8,
-        """Displayed tab width in file preview. (''since 0.9'')""")
+        """Displayed tab width in file preview.""")
 
     max_preview_size = IntOption('mimeviewer', 'max_preview_size', 262144,
-        """Maximum file size for HTML preview. (''since 0.9'')""")
+        """Maximum file size for HTML preview.""")
 
     mime_map = ListOption('mimeviewer', 'mime_map',
         'text/x-dylan:dylan, text/x-idl:ice, text/x-ada:ads:adb',
         doc="""List of additional MIME types and keyword mappings.
         Mappings are comma-separated, and for each MIME type,
         there's a colon (":") separated list of associated keywords
-        or file extensions. (''since 0.10'')""")
+        or file extensions.
+        """)
 
     mime_map_patterns = ListOption('mimeviewer', 'mime_map_patterns',
-        'text/plain:README|INSTALL|COPYING.*',
+        'text/plain:README(?!\.rst)|INSTALL(?!\.rst)|COPYING.*',
         doc="""List of additional MIME types associated to filename patterns.
         Mappings are comma-separated, and each mapping consists of a MIME type
         and a Python regexp used for matching filenames, separated by a colon
-        (":"). (''since 1.0'')""")
+        (":"). (''since 1.0'')
+        """)
 
     treat_as_binary = ListOption('mimeviewer', 'treat_as_binary',
         'application/octet-stream, application/pdf, application/postscript, '
-        'application/msword,application/rtf,',
+        'application/msword, application/rtf',
         doc="""Comma-separated list of MIME types that should be treated as
-        binary data. (''since 0.11.5'')""")
+        binary data.
+        """)
 
     def __init__(self):
         self._mime_map = None
@@ -646,25 +670,30 @@ class Mimeview(Component):
     # Public API
 
     def get_supported_conversions(self, mimetype):
-        """Return a list of target MIME types in same form as
-        `IContentConverter.get_supported_conversions()`, but with the converter
-        component appended. Output is ordered from best to worst quality."""
+        """Return a list of target MIME types as instances of the `namedtuple`
+        `MimeConversion`. Output is ordered from best to worst quality.
+
+        The `MimeConversion` `namedtuple` has fields: key, name, extension,
+        in_mimetype, out_mimetype, quality, converter.
+        """
+        fields = ('key', 'name', 'extension', 'in_mimetype',
+                  'out_mimetype', 'quality', 'converter')
+        _MimeConversion = namedtuple('MimeConversion', fields)
         converters = []
-        for converter in self.converters:
-            conversions = converter.get_supported_conversions() or []
-            for k, n, e, im, om, q in conversions:
+        for c in self.converters:
+            for k, n, e, im, om, q in c.get_supported_conversions() or []:
                 if im == mimetype and q > 0:
-                    converters.append((k, n, e, im, om, q, converter))
-        converters = sorted(converters, key=lambda i: i[-2], reverse=True)
+                    converters.append(_MimeConversion(k, n, e, im, om, q, c))
+        converters = sorted(converters, key=lambda i: i.quality, reverse=True)
         return converters
 
     def convert_content(self, req, mimetype, content, key, filename=None,
-                        url=None):
+                        url=None, iterable=False):
         """Convert the given content to the target MIME type represented by
         `key`, which can be either a MIME type or a key. Returns a tuple of
         (content, output_mime_type, extension)."""
         if not content:
-            return ('', 'text/plain;charset=utf-8', '.txt')
+            return '', 'text/plain;charset=utf-8', '.txt'
 
         # Ensure we have a MIME type for this content
         full_mimetype = mimetype
@@ -673,24 +702,32 @@ class Mimeview(Component):
                 content = content.read(self.max_preview_size)
             full_mimetype = self.get_mimetype(filename, content)
         if full_mimetype:
-            mimetype = ct_mimetype(full_mimetype)   # split off charset
+            mimetype = ct_mimetype(full_mimetype)  # split off charset
         else:
-            mimetype = full_mimetype = 'text/plain' # fallback if not binary
+            mimetype = full_mimetype = 'text/plain'  # fallback if not binary
 
         # Choose best converter
-        candidates = list(self.get_supported_conversions(mimetype) or [])
-        candidates = [c for c in candidates if key in (c[0], c[4])]
+        candidates = [c for c in self.get_supported_conversions(mimetype)
+                        if key in (c.key, c.out_mimetype)]
         if not candidates:
             raise TracError(
                 _("No available MIME conversions from %(old)s to %(new)s",
                   old=mimetype, new=key))
 
         # First successful conversion wins
-        for ck, name, ext, input_mimettype, output_mimetype, quality, \
-                converter in candidates:
-            output = converter.convert_content(req, mimetype, content, ck)
+        for conversion in candidates:
+            output = conversion.converter.convert_content(req, mimetype,
+                                                          content,
+                                                          conversion.key)
             if output:
-                return (output[0], output[1], ext)
+                content, content_type = output
+                if iterable:
+                    if isinstance(content, basestring):
+                        content = (content,)
+                else:
+                    if not isinstance(content, basestring):
+                        content = ''.join(content)
+                return content, content_type, conversion.extension
         raise TracError(
             _("No available MIME conversions from %(old)s to %(new)s",
               old=mimetype, new=key))
@@ -751,7 +788,7 @@ class Mimeview(Component):
         expanded_content = None
         for qr, renderer in candidates:
             if force_source and not getattr(renderer, 'returns_source', False):
-                continue # skip non-source renderers in force_source mode
+                continue  # skip non-source renderers in force_source mode
             if isinstance(content, Content):
                 content.reset()
             try:
@@ -788,17 +825,19 @@ class Mimeview(Component):
 
                 # Render content as source code
                 if annotations:
-                    m = context.req.args.get('marks') if context.req else None
-                    return self._render_source(context, result, annotations,
-                                               m and Ranges(m))
+                    marks = context.req.args.get('marks') if context.req \
+                            else None
+                    if marks:
+                        context.set_hints(marks=marks)
+                    return self._render_source(context, result, annotations)
                 else:
                     if isinstance(result, list):
                         result = Markup('\n').join(result)
                     return tag.div(class_='code')(tag.pre(result)).generate()
 
-            except Exception, e:
-                self.log.warning('HTML preview using %s failed: %s',
-                                 renderer.__class__.__name__,
+            except Exception as e:
+                self.log.warning('HTML preview using %s with %r failed: %s',
+                                 renderer.__class__.__name__, context,
                                  exception_to_unicode(e, traceback=True))
                 if context.req and not context.get_hint('disable_warnings'):
                     from trac.web.chrome import add_warning
@@ -807,7 +846,7 @@ class Mimeview(Component):
                           renderer=renderer.__class__.__name__,
                           err=exception_to_unicode(e)))
 
-    def _render_source(self, context, stream, annotations, marks=None):
+    def _render_source(self, context, stream, annotations):
         from trac.web.chrome import add_warning
         annotators, labels, titles = {}, {}, {}
         for annotator in self.annotators:
@@ -832,12 +871,12 @@ class Mimeview(Component):
             annotator = annotators[a]
             try:
                 data = (annotator, annotator.get_annotation_data(context))
-            except TracError, e:
-                self.log.warning("Can't use annotator '%s': %s", a, e.message)
+            except TracError as e:
+                self.log.warning("Can't use annotator '%s': %s", a, e)
                 add_warning(context.req, tag.strong(
                     tag_("Can't use %(annotator)s annotator: %(error)s",
-                         annotator=tag.em(a), error=tag.pre(e.message))))
-                data = (None, None)
+                         annotator=tag.em(a), error=tag.pre(e))))
+                data = None, None
             annotator_datas.append(data)
 
         def _head_row():
@@ -850,8 +889,6 @@ class Mimeview(Component):
         def _body_rows():
             for idx, line in enumerate(_group_lines(stream)):
                 row = tag.tr()
-                if marks and idx + 1 in marks:
-                    row(class_='hilite')
                 for annotator, data in annotator_datas:
                     if annotator:
                         annotator.annotate_row(context, row, idx+1, line, data)
@@ -865,10 +902,6 @@ class Mimeview(Component):
             tag.tbody(_body_rows())
         )
 
-    def get_max_preview_size(self):
-        """:deprecated: use `max_preview_size` attribute directly."""
-        return self.max_preview_size
-
     def get_charset(self, content='', mimetype=None):
         """Infer the character encoding from the `content` or the `mimetype`.
 
@@ -938,7 +971,7 @@ class Mimeview(Component):
                     mimetype, regexp = mapping.split(':', 1)
                 try:
                     self._mime_map_patterns[mimetype] = re.compile(regexp)
-                except re.error, e:
+                except re.error as e:
                     self.log.warning("mime_map_patterns contains invalid "
                                      "regexp '%s' for mimetype '%s' (%s)",
                                      regexp, mimetype, exception_to_unicode(e))
@@ -956,13 +989,6 @@ class Mimeview(Component):
             return True
         return False
 
-    def to_utf8(self, content, mimetype=None):
-        """Convert an encoded `content` to utf-8.
-
-        :deprecated: since 0.10, you should use `unicode` strings only.
-        """
-        return to_utf8(content, self.get_charset(content, mimetype))
-
     def to_unicode(self, content, mimetype=None, charset=None):
         """Convert `content` (an encoded `str` object) to an `unicode` object.
 
@@ -1010,14 +1036,28 @@ class Mimeview(Component):
         """Helper method for converting `content` and sending it directly.
 
         `selector` can be either a key or a MIME Type."""
+        from trac.web.chrome import Chrome
         from trac.web.api import RequestDone
-        content, output_type, ext = self.convert_content(req, in_type,
-                                                         content, selector)
-        if isinstance(content, unicode):
-            content = content.encode('utf-8')
+        iterable = Chrome(self.env).use_chunked_encoding
+        content, output_type, ext = self.convert_content(req, in_type, content,
+                                                         selector,
+                                                         iterable=iterable)
+        if iterable:
+            def encoder(content):
+                for chunk in content:
+                    if isinstance(chunk, unicode):
+                        chunk = chunk.encode('utf-8')
+                    yield chunk
+            content = encoder(content)
+            length = None
+        else:
+            if isinstance(content, unicode):
+                content = content.encode('utf-8')
+            length = len(content)
         req.send_response(200)
         req.send_header('Content-Type', output_type)
-        req.send_header('Content-Length', len(content))
+        if length is not None:
+            req.send_header('Content-Length', length)
         if filename:
             req.send_header('Content-Disposition',
                             content_disposition('attachment',
@@ -1029,6 +1069,7 @@ class Mimeview(Component):
 
 def _group_lines(stream):
     space_re = re.compile('(?P<spaces> (?: +))|^(?P<tag><\w+.*?>)?( )')
+
     def pad_spaces(match):
         m = match.group('spaces')
         if m:
@@ -1108,18 +1149,28 @@ class LineNumberAnnotator(Component):
     """Text annotator that adds a column with line numbers."""
     implements(IHTMLPreviewAnnotator)
 
-    # ITextAnnotator methods
+    # IHTMLPreviewAnnotator methods
 
     def get_annotation_type(self):
         return 'lineno', _('Line'), _('Line numbers')
 
     def get_annotation_data(self, context):
-        return None
+        try:
+            marks = Ranges(context.get_hint('marks'))
+        except ValueError:
+            marks = None
+        return {
+            'id': context.get_hint('id', '') + 'L%s',
+            'marks': marks,
+            'offset': context.get_hint('lineno', 1) - 1
+        }
 
     def annotate_row(self, context, row, lineno, line, data):
-        row.append(tag.th(id='L%s' % lineno)(
-            tag.a(lineno, href='#L%s' % lineno)
-        ))
+        lineno += data['offset']
+        id = data['id'] % lineno
+        if data['marks'] and lineno in data['marks']:
+            row(class_='hilite')
+        row.append(tag.th(id=id)(tag.a(lineno, href='#' + id)))
 
 
 # -- Default renderers
