@@ -10,34 +10,31 @@
 #
 # Author: Matthew Good <matt@matt-good.net>
 
-from datetime import datetime
+from __future__ import absolute_import
+
 import os
-from pkg_resources import resource_filename
+import pygments
 import re
+from datetime import datetime
+from pkg_resources import resource_filename
+from pygments.formatters.html import HtmlFormatter
+from pygments.lexers import get_all_lexers, get_lexer_by_name
+from pygments.styles import get_all_styles, get_style_by_name
 
 from trac.core import *
-from trac.config import ListOption, Option
+from trac.config import ConfigSection, ListOption, Option
 from trac.env import ISystemInfoProvider
 from trac.mimeview.api import IHTMLPreviewRenderer, Mimeview
 from trac.prefs import IPreferencePanelProvider
-from trac.util import get_pkginfo
+from trac.util import get_pkginfo, lazy
 from trac.util.datefmt import http_date, localtz
 from trac.util.translation import _
 from trac.web.api import IRequestHandler, HTTPNotFound
-from trac.web.chrome import add_notice, add_stylesheet
+from trac.web.chrome import ITemplateProvider, add_notice, add_stylesheet
 
 from genshi import QName, Stream
 from genshi.core import Attrs, START, END, TEXT
 
-# Kludge to workaround the lack of absolute imports in Python version prior to
-# 2.5
-pygments = __import__('pygments', {}, {}, ['lexers', 'styles', 'formatters'])
-get_all_lexers = pygments.lexers.get_all_lexers
-get_lexer_by_name = pygments.lexers.get_lexer_by_name
-HtmlFormatter = pygments.formatters.html.HtmlFormatter
-get_all_styles = pygments.styles.get_all_styles
-get_style_by_name = pygments.styles.get_style_by_name
-
 __all__ = ['PygmentsRenderer']
 
 
@@ -45,7 +42,27 @@ class PygmentsRenderer(Component):
     """HTML renderer for syntax highlighting based on Pygments."""
 
     implements(ISystemInfoProvider, IHTMLPreviewRenderer,
-               IPreferencePanelProvider, IRequestHandler)
+               IPreferencePanelProvider, IRequestHandler,
+               ITemplateProvider)
+
+    is_valid_default_handler = False
+
+    pygments_lexer_options = ConfigSection('pygments-lexer',
+        """Configure Pygments [%(url)s lexer] options.
+
+        For example, to set the
+        [%(url)s#lexers-for-php-and-related-languages PhpLexer] options
+        `startinline` and `funcnamehighlighting`:
+        {{{#!ini
+        [pygments-lexer]
+        php.startinline = True
+        php.funcnamehighlighting = True
+        }}}
+
+        The lexer name is derived from the class name, with `Lexer` stripped
+        from the end. The lexer //short names// can also be used in place
+        of the lexer name.
+        """, doc_args={'url': 'http://pygments.org/docs/lexers/'})
 
     default_style = Option('mimeviewer', 'pygments_default_style', 'trac',
         """The default style to use for Pygments syntax highlighting.""")
@@ -82,9 +99,6 @@ class PygmentsRenderer(Component):
   </body>
 </html>"""
 
-    def __init__(self):
-        self._types = None
-
     # ISystemInfoProvider methods
 
     def get_system_info(self):
@@ -97,15 +111,12 @@ class PygmentsRenderer(Component):
     # IHTMLPreviewRenderer methods
 
     def get_extra_mimetypes(self):
-        for lexname, aliases, _, mimetypes in get_all_lexers():
-            name = aliases[0] if aliases else lexname
+        for _, aliases, _, mimetypes in get_all_lexers():
             for mimetype in mimetypes:
                 yield mimetype, aliases
 
     def get_quality_ratio(self, mimetype):
         # Extend default MIME type to mode mappings with configured ones
-        if self._types is None:
-            self._init_types()
         try:
             return self._types[mimetype][1]
         except KeyError:
@@ -113,15 +124,13 @@ class PygmentsRenderer(Component):
 
     def render(self, context, mimetype, content, filename=None, rev=None):
         req = context.req
-        if self._types is None:
-            self._init_types()
-        add_stylesheet(req, '/pygments/%s.css' %
-                       req.session.get('pygments_style', self.default_style))
+        style = req.session.get('pygments_style', self.default_style)
+        add_stylesheet(req, '/pygments/%s.css' % style)
         try:
             if len(content) > 0:
                 mimetype = mimetype.split(';', 1)[0]
                 language = self._types[mimetype][0]
-                return self._generate(language, content)
+                return self._generate(language, content, context)
         except (KeyError, ValueError):
             raise Exception("No Pygments lexer found for mime-type '%s'."
                             % mimetype)
@@ -129,7 +138,7 @@ class PygmentsRenderer(Component):
     # IPreferencePanelProvider methods
 
     def get_preference_panels(self, req):
-        yield ('pygments', _('Syntax Highlighting'))
+        yield 'pygments', _('Syntax Highlighting')
 
     def render_preference_panel(self, req, panel):
         styles = list(get_all_styles())
@@ -138,9 +147,11 @@ class PygmentsRenderer(Component):
             style = req.args.get('style')
             if style and style in styles:
                 req.session['pygments_style'] = style
-                add_notice(req, _('Your preferences have been saved.'))
+                add_notice(req, _("Your preferences have been saved."))
             req.redirect(req.href.prefs(panel or None))
 
+        for style in sorted(styles):
+            add_stylesheet(req, '/pygments/%s.css' % style, title=style.title())
         output = self._generate('html', self.EXAMPLE)
         return 'prefs_pygments.html', {
             'output': output,
@@ -151,7 +162,7 @@ class PygmentsRenderer(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        match = re.match(r'/pygments/(\w+)\.css', req.path_info)
+        match = re.match(r'/pygments/([-\w]+)\.css', req.path_info)
         if match:
             req.args['style'] = match.group(1)
             return True
@@ -160,7 +171,7 @@ class PygmentsRenderer(Component):
         style = req.args['style']
         try:
             style_cls = get_style_by_name(style)
-        except ValueError, e:
+        except ValueError as e:
             raise HTTPNotFound(e)
 
         parts = style_cls.__module__.split('.')
@@ -184,29 +195,68 @@ class PygmentsRenderer(Component):
         req.send_header('Content-Length', len(content))
         req.write(content)
 
+    # ITemplateProvider methods
+
+    def get_htdocs_dirs(self):
+        return []
+
+    def get_templates_dirs(self):
+        return [resource_filename('trac.mimeview', 'templates')]
+
     # Internal methods
 
-    def _init_types(self):
-        self._types = {}
-        for lexname, aliases, _, mimetypes in get_all_lexers():
-            name = aliases[0] if aliases else lexname
-            for mimetype in mimetypes:
-                self._types[mimetype] = (name, self.QUALITY_RATIO)
+    @lazy
+    def _lexer_alias_name_map(self):
+        lexer_alias_name_map = {}
+        for lexer_name, aliases, _, _ in get_all_lexers():
+            name = aliases[0] if aliases else lexer_name
+            for alias in aliases:
+                lexer_alias_name_map[alias] = name
+        return lexer_alias_name_map
 
-        # Pygments currently doesn't know application/javascript
-        if 'application/javascript' not in self._types:
+    @lazy
+    def _lexer_options(self):
+        lexer_options = {}
+        for key, lexer_option_value in self.pygments_lexer_options.options():
+            try:
+                lexer_name_or_alias, lexer_option_name = key.split('.')
+            except ValueError:
+                pass
+            else:
+                lexer_name = self._lexer_alias_to_name(lexer_name_or_alias)
+                lexer_option = {lexer_option_name: lexer_option_value}
+                lexer_options.setdefault(lexer_name, {}).update(lexer_option)
+        return lexer_options
+
+    @lazy
+    def _types(self):
+        types = {}
+        for lexer_name, aliases, _, mimetypes in get_all_lexers():
+            name = aliases[0] if aliases else lexer_name
+            for mimetype in mimetypes:
+                types[mimetype] = (name, self.QUALITY_RATIO)
+
+        # Pygments < 1.4 doesn't know application/javascript
+        if 'application/javascript' not in types:
             js_entry = self._types.get('text/javascript')
             if js_entry:
-                self._types['application/javascript'] = js_entry
+                types['application/javascript'] = js_entry
 
-        self._types.update(
-            Mimeview(self.env).configured_modes_mapping('pygments')
-        )
+        types.update(Mimeview(self.env).configured_modes_mapping('pygments'))
+        return types
 
-    def _generate(self, language, content):
-        lexer = get_lexer_by_name(language, stripnl=False)
+    def _generate(self, language, content, context=None):
+        lexer_name = self._lexer_alias_to_name(language)
+        lexer_options = {'stripnl': False}
+        lexer_options.update(self._lexer_options.get(lexer_name, {}))
+        if context:
+            lexer_options.update(context.get_hint('lexer_options', {}))
+        lexer = get_lexer_by_name(lexer_name, **lexer_options)
         return GenshiHtmlFormatter().generate(lexer.get_tokens(content))
 
+    def _lexer_alias_to_name(self, alias):
+        return self._lexer_alias_name_map.get(alias, alias)
+
 
 class GenshiHtmlFormatter(HtmlFormatter):
     """A Pygments formatter subclass that generates a Python stream instead
@@ -238,7 +288,7 @@ class GenshiHtmlFormatter(HtmlFormatter):
             yield last_class, u''.join(text)
 
     def generate(self, tokens):
-        pos = (None, -1, -1)
+        pos = None, -1, -1
         span = QName('span')
         class_ = QName('class')
 
