@@ -100,9 +100,9 @@ class MimeviewTestCase(unittest.TestCase):
 
         mimeview = Mimeview(self.env)
         conversions = mimeview.get_supported_conversions('text/x-sample')
-        self.assertEqual(Converter0(self.env), conversions[0][-1])
-        self.assertEqual(Converter1(self.env), conversions[1][-1])
-        self.assertEqual(Converter2(self.env), conversions[2][-1])
+        self.assertEqual(Converter0(self.env), conversions[0].converter)
+        self.assertEqual(Converter1(self.env), conversions[1].converter)
+        self.assertEqual(Converter2(self.env), conversions[2].converter)
 
 class GroupLinesTestCase(unittest.TestCase):
 
@@ -328,7 +328,7 @@ class MimeviewConverterTestCase(unittest.TestCase):
         self._test_send_converted('Ü' * 0x10000, 'unicode', True)
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(doctest.DocTestSuite(api))
     suite.addTest(unittest.makeSuite(GetMimeTypeTestCase))
@@ -338,4 +338,4 @@ def suite():
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
