@@ -16,11 +16,11 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
+from __future__ import print_function
 
 import csv
 import os
-from time import time
+from itertools import groupby
 
 from trac.admin import AdminCommandError, IAdminCommandProvider, get_dir_list
 from trac.cache import cached
@@ -28,40 +28,47 @@ from trac.config import ExtensionOption, OrderedExtensionsOption
 from trac.core import *
 from trac.resource import Resource, get_resource_name
 from trac.util import file_or_std
+from trac.util.datefmt import time_now
 from trac.util.text import path_to_unicode, print_table, printout, \
                            stream_encoding, to_unicode, wrap
-from trac.util.translation import _
+from trac.util.translation import _, N_
 
 __all__ = ['IPermissionRequestor', 'IPermissionStore', 'IPermissionPolicy',
            'IPermissionGroupProvider', 'PermissionError', 'PermissionSystem']
 
 
-class PermissionError(StandardError):
-    """Insufficient permissions to complete the operation"""
+class PermissionError(StandardError, TracBaseError):
+    """Insufficient permissions to perform the operation.
 
-    def __init__ (self, action=None, resource=None, env=None, msg=None):
-        StandardError.__init__(self)
+    :since 1.0.5: the `msg` attribute is deprecated and will be removed in
+                  1.3.1. Use the `message` property instead.
+    """
+
+    title = N_("Forbidden")
+
+    def __init__(self, action=None, resource=None, env=None, msg=None):
         self.action = action
         self.resource = resource
         self.env = env
-        self.msg = msg
-
-    def __unicode__ (self):
         if self.action:
             if self.resource:
-                return _('%(perm)s privileges are required to perform '
-                         'this operation on %(resource)s. You don\'t have the '
-                         'required permissions.',
-                         perm=self.action,
-                         resource=get_resource_name(self.env, self.resource))
+                msg = _("%(perm)s privileges are required to perform "
+                        "this operation on %(resource)s. You don't have the "
+                        "required permissions.",
+                        perm=self.action,
+                        resource=get_resource_name(self.env, self.resource))
             else:
-                return _('%(perm)s privileges are required to perform this '
-                         'operation. You don\'t have the required '
-                         'permissions.', perm=self.action)
-        elif self.msg:
-            return self.msg
-        else:
-            return _('Insufficient privileges to perform this operation.')
+                msg = _("%(perm)s privileges are required to perform this "
+                        "operation. You don't have the required "
+                        "permissions.", perm=self.action)
+        elif msg is None:
+            msg = _("Insufficient privileges to perform this operation.")
+        self.msg = msg
+        super(PermissionError, self).__init__(msg)
+
+    @property
+    def message(self):
+        return self.args[0]
 
 
 class IPermissionRequestor(Interface):
@@ -223,7 +230,6 @@ class DefaultPermissionStore(Component):
         formatted tuples."""
         return self._all_permissions
 
-
     @cached
     def _all_permissions(self):
         return [(username, action) for username, action in
@@ -276,16 +282,16 @@ class DefaultPermissionPolicy(Component):
 
     def __init__(self):
         self.permission_cache = {}
-        self.last_reap = time()
+        self.last_reap = time_now()
 
     # IPermissionPolicy methods
 
     def check_permission(self, action, username, resource, perm):
-        now = time()
+        now = time_now()
 
         if now - self.last_reap > self.CACHE_REAP_TIME:
             self.permission_cache = {}
-            self.last_reap = time()
+            self.last_reap = time_now()
 
         timestamp, permissions = self.permission_cache.get(username, (0, None))
 
@@ -299,7 +305,6 @@ class DefaultPermissionPolicy(Component):
         return action in permissions or None
 
 
-
 class PermissionSystem(Component):
     """Permission management sub-system."""
 
@@ -316,14 +321,11 @@ class PermissionSystem(Component):
 
     policies = OrderedExtensionsOption('trac', 'permission_policies',
         IPermissionPolicy,
-        'DefaultPermissionPolicy, LegacyAttachmentPolicy',
+        'ReadonlyWikiPolicy, DefaultPermissionPolicy, LegacyAttachmentPolicy',
         False,
-        """List of components implementing `IPermissionPolicy`, in the order in
-        which they will be applied. These components manage fine-grained access
-        control to Trac resources.
-        Defaults to the DefaultPermissionPolicy (pre-0.11 behavior) and
-        LegacyAttachmentPolicy (map ATTACHMENT_* permissions to realm specific
-        ones)""")
+        """List of components implementing `IPermissionPolicy`, in the order
+        in which they will be applied. These components manage fine-grained
+        access control to Trac resources.""")
 
     # Number of seconds a cached user permission set is valid for.
     CACHE_EXPIRY = 5
@@ -332,20 +334,25 @@ class PermissionSystem(Component):
 
     def __init__(self):
         self.permission_cache = {}
-        self.last_reap = time()
+        self.last_reap = time_now()
 
     # Public API
 
     def grant_permission(self, username, action):
-        """Grant the user with the given name permission to perform to specified
-        action."""
+        """Grant the user with the given name permission to perform to
+        specified action."""
         if action.isupper() and action not in self.get_actions():
             raise TracError(_('%(name)s is not a valid action.', name=action))
+        elif not action.isupper() and action.upper() in self.get_actions():
+            raise TracError(_("Permission %(name)s differs from a defined "
+                              "action by casing only, which is not allowed.",
+                              name=action))
 
         self.store.grant_permission(username, action)
 
     def revoke_permission(self, username, action):
-        """Revokes the permission of the specified user to perform an action."""
+        """Revokes the permission of the specified user to perform an
+        action."""
         self.store.revoke_permission(username, action)
 
     def get_actions_dict(self):
@@ -377,6 +384,33 @@ class PermissionSystem(Component):
                     actions.add(action)
         return list(actions)
 
+    def get_groups_dict(self):
+        """Get all groups as a `dict`.
+
+        The keys are the group names. The values are the group members.
+
+        :since: 1.1.3
+        """
+        groups = sorted((p for p in self.get_all_permissions()
+                           if not p[1].isupper()), key=lambda p: p[1])
+
+        return dict((k, sorted(i[0] for i in list(g)))
+                    for k, g in groupby(groups, key=lambda p: p[1]))
+
+    def get_users_dict(self):
+        """Get all users as a `dict`.
+
+        The keys are the user names. The values are the actions possessed
+        by the user.
+
+        :since: 1.1.3
+        """
+        perms = sorted((p for p in self.get_all_permissions()
+                          if p[1].isupper()), key=lambda p: p[0])
+
+        return dict((k, sorted(i[1] for i in list(g)))
+                    for k, g in groupby(perms, key=lambda p: p[0]))
+
     def get_user_permissions(self, username=None):
         """Return the permissions of the specified user.
 
@@ -411,7 +445,7 @@ class PermissionSystem(Component):
 
         Users are returned as a list of user names.
         """
-        now = time()
+        now = time_now()
         if now - self.last_reap > self.CACHE_REAP_TIME:
             self.permission_cache = {}
             self.last_reap = now
@@ -450,7 +484,8 @@ class PermissionSystem(Component):
             expand_action(a)
         return expanded_actions
 
-    def check_permission(self, action, username=None, resource=None, perm=None):
+    def check_permission(self, action, username=None, resource=None,
+                         perm=None):
         """Return True if permission to perform action for the given resource
         is allowed."""
         if username is None:
@@ -461,7 +496,7 @@ class PermissionSystem(Component):
             decision = policy.check_permission(action, username, resource,
                                                perm)
             if decision is not None:
-                if not decision:
+                if decision is False:
                     self.log.debug("%s denies %s performing %s on %r",
                                    policy.__class__.__name__, username,
                                    action, resource)
@@ -474,14 +509,9 @@ class PermissionSystem(Component):
 
     def get_permission_actions(self):
         """Implement the global `TRAC_ADMIN` meta permission.
-
-        Implements also the `EMAIL_VIEW` permission which allows for
-        showing email addresses even if `[trac] show_email_addresses`
-        is `false`.
         """
         actions = self.get_actions(skip=self)
-        actions.append('EMAIL_VIEW')
-        return [('TRAC_ADMIN', actions), 'EMAIL_VIEW']
+        return [('TRAC_ADMIN', actions)]
 
 
 class PermissionCache(object):
@@ -573,20 +603,16 @@ class PermissionCache(object):
 
     __contains__ = has_permission
 
-    def require(self, action, realm_or_resource=None, id=False, version=False):
+    def require(self, action, realm_or_resource=None, id=False, version=False,
+                message=None):
         resource = self._normalize_resource(realm_or_resource, id, version)
         if not self._has_permission(action, resource):
-            raise PermissionError(action, resource, self.env)
+            if message is None:
+                raise PermissionError(action, resource, self.env)
+            else:
+                raise PermissionError(msg=message)
     assert_permission = require
 
-    def permissions(self):
-        """Deprecated (but still used by the HDF compatibility layer)"""
-        self.env.log.warning("perm.permissions() is deprecated and "
-                             "is only present for HDF compatibility")
-        perm = PermissionSystem(self.env)
-        actions = perm.get_user_permissions(self.username)
-        return [action for action in actions if action in self]
-
 
 class PermissionAdmin(Component):
     """trac-admin command provider for permission system administration."""
@@ -654,14 +680,14 @@ class PermissionAdmin(Component):
             rows = permsys.get_all_permissions()
         rows.sort()
         print_table(rows, [_('User'), _('Action')])
-        print
+        print()
         printout(_("Available actions:"))
         actions = permsys.get_actions()
         actions.sort()
         text = ', '.join(actions)
         printout(wrap(text, initial_indent=' ', subsequent_indent=' ',
                       linesep='\n'))
-        print
+        print()
 
     def _do_add(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -674,6 +700,8 @@ class PermissionAdmin(Component):
             except self.env.db_exc.IntegrityError:
                 printout(_("The user %(user)s already has permission "
                            "%(action)s.", user=user, action=action))
+            except TracError as e:
+                raise AdminCommandError(e)
 
     def _do_remove(self, user, *actions):
         permsys = PermissionSystem(self.env)
@@ -685,9 +713,19 @@ class PermissionAdmin(Component):
                     permsys.revoke_permission(u, a)
                     found = True
             if not found:
-                raise AdminCommandError(
-                    _("Cannot remove permission %(action)s for user %(user)s.",
-                      action=action, user=user))
+                if user in self.get_user_list() and \
+                        action in permsys.get_user_permissions(user):
+                    msg = _("Cannot remove permission %(action)s for user "
+                            "%(user)s. The permission is granted through "
+                            "a meta-permission or group.", action=action,
+                            user=user)
+                elif action in permsys.get_actions():
+                    msg = _("Cannot remove permission %(action)s for user "
+                            "%(user)s. The user has not been granted the "
+                            "permission.", action=action, user=user)
+                else:
+                    msg = _("%(name)s is not a valid action.", name=action)
+                raise AdminCommandError(msg)
 
     def _do_export(self, filename=None):
         try:
@@ -700,7 +738,7 @@ class PermissionAdmin(Component):
                     actions = sorted(self.get_user_perms(user))
                     writer.writerow([s.encode(encoding, 'replace')
                                      for s in [user] + actions])
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot export to %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdout'),
@@ -730,12 +768,12 @@ class PermissionAdmin(Component):
                     old_actions = self.get_user_perms(user)
                     for action in set(actions) - set(old_actions):
                         permsys.grant_permission(user, action)
-        except csv.Error, e:
+        except csv.Error as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s line %(line)d: %(error)s ",
                   filename=path_to_unicode(filename or 'stdin'),
                   line=reader.line_num, error=e))
-        except IOError, e:
+        except IOError as e:
             raise AdminCommandError(
                 _("Cannot import from %(filename)s: %(error)s",
                   filename=path_to_unicode(filename or 'stdin'),
