@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2014 Edgewall Software
 # Copyright (C) 2003-2006 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2005 Christopher Lenz <cmlenz@gmx.de>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
@@ -17,20 +17,18 @@
 # Author: Jonas Borgström <jonas@edgewall.com>
 #         Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
 import re
 
-from trac.attachment import Attachment
 from trac import core
+from trac.attachment import Attachment
 from trac.cache import cached
 from trac.core import TracError
 from trac.resource import Resource, ResourceNotFound
 from trac.ticket.api import TicketSystem
-from trac.util import embedded_numbers, partition
+from trac.util import as_int, embedded_numbers
+from trac.util.datefmt import (datetime_now, from_utimestamp, parse_date,
+                               to_utimestamp, utc, utcmax)
 from trac.util.text import empty
-from trac.util.datefmt import datetime_now, from_utimestamp, to_utimestamp, \
-                              utc, utcmax
 from trac.util.translation import _
 
 __all__ = ['Ticket', 'Type', 'Status', 'Resolution', 'Priority', 'Severity',
@@ -46,28 +44,56 @@ def _fixup_cc_list(cc_value):
     return ', '.join(cclist)
 
 
+def _db_str_to_datetime(value):
+    if value is None:
+        return None
+    try:
+        return from_utimestamp(long(value))
+    except ValueError:
+        pass
+    try:
+        return parse_date(value.strip(), utc, 'datetime')
+    except Exception:
+        return None
+
+
+def _datetime_to_db_str(dt, is_custom_field):
+    if not dt:
+        return ''
+    ts = to_utimestamp(dt)
+    if is_custom_field:
+        # Padding with '0' would be easy to sort in report page for a user
+        fmt = '%018d' if ts >= 0 else '%+017d'
+        return fmt % ts
+    else:
+        return ts
+
+
 class Ticket(object):
 
+    realm = 'ticket'
+
     # Fields that must not be modified directly by the user
-    protected_fields = ('resolution', 'status', 'time', 'changetime')
+    # 'owner' should eventually be a protected field (#2045)
+    protected_fields = 'resolution', 'status', 'time', 'changetime'
 
     @staticmethod
     def id_is_valid(num):
-        return 0 < int(num) <= 1L << 31
+        try:
+            return 0 < int(num) <= 1L << 31
+        except (ValueError, TypeError):
+            return False
 
-    # 0.11 compatibility
+    @property
+    def resource(self):
+        return Resource(self.realm, self.id, self.version)
+
+    # 0.11 compatibility. Will be removed in 1.3.1.
     time_created = property(lambda self: self.values.get('time'))
     time_changed = property(lambda self: self.values.get('changetime'))
 
-    def __init__(self, env, tkt_id=None, db=None, version=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+    def __init__(self, env, tkt_id=None, version=None):
         self.env = env
-        if tkt_id is not None:
-            tkt_id = int(tkt_id)
-        self.resource = Resource('ticket', tkt_id, version)
         self.fields = TicketSystem(self.env).get_ticket_fields()
         self.editable_fields = \
             set(f['name'] for f in self.fields
@@ -81,12 +107,16 @@ class Ticket(object):
             if f['type'] == 'time':
                 self.time_fields.append(f['name'])
         self.values = {}
+        self._old = {}
         if tkt_id is not None:
             self._fetch_ticket(tkt_id)
         else:
             self._init_defaults()
             self.id = None
-        self._old = {}
+        self.version = version
+
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.id)
 
     exists = property(lambda self: self.id is not None)
 
@@ -100,22 +130,36 @@ class Ticket(object):
                 default = self.env.config.get('ticket',
                                               'default_' + field['name'])
             else:
-                default = field.get('value')
-                options = field.get('options')
-                if default and options and default not in options:
-                    try:
-                        default = options[int(default)]
-                    except (ValueError, IndexError):
-                        self.env.log.warning('Invalid default value "%s" '
-                                             'for custom field "%s"',
-                                             default, field['name'])
+                default = self._custom_field_default(field)
             if default:
                 self.values.setdefault(field['name'], default)
 
+    def _custom_field_default(self, field):
+        default = field.get('value')
+        options = field.get('options')
+        if default and options and default not in options:
+            try:
+                default = options[int(default)]
+            except (ValueError, IndexError):
+                self.env.log.warning('Invalid default value "%s" '
+                                     'for custom field "%s"',
+                                     default, field['name'])
+        if default and field.get('type') == 'time':
+            try:
+                default = parse_date(default,
+                                     hint=field.get('format'))
+            except TracError as e:
+                self.env.log.warning('Invalid default value "%s" '
+                                     'for custom field "%s": %s',
+                                     default, field['name'], e)
+                default = None
+        return default
+
     def _fetch_ticket(self, tkt_id):
         row = None
         if self.id_is_valid(tkt_id):
             # Fetch the standard ticket fields
+            tkt_id = int(tkt_id)
             for row in self.env.db_query("SELECT %s FROM ticket WHERE id=%%s" %
                                          ','.join(self.std_fields), (tkt_id,)):
                 break
@@ -138,31 +182,43 @@ class Ticket(object):
                 SELECT name, value FROM ticket_custom WHERE ticket=%s
                 """, (tkt_id,)):
             if name in self.custom_fields:
-                if value is None:
+                if name in self.time_fields:
+                    self.values[name] = _db_str_to_datetime(value)
+                elif value is None:
                     self.values[name] = empty
                 else:
                     self.values[name] = value
 
+        # Set defaults for custom fields that haven't been fetched.
+        for field in self.fields:
+            name = field['name']
+            if field.get('custom') and name not in self.values:
+                default = self._custom_field_default(field)
+                if default:
+                    self[name] = default
+
     def __getitem__(self, name):
         return self.values.get(name)
 
     def __setitem__(self, name, value):
         """Log ticket modifications so the table ticket_change can be updated
         """
-        if value:
+        if value and name not in self.time_fields:
             if isinstance(value, list):
                 raise TracError(_("Multi-values fields not supported yet"))
-            field = [field for field in self.fields if field['name'] == name]
-            if field and field[0].get('type') != 'textarea':
+            if self.fields.by_name(name, {}).get('type') != 'textarea':
                 value = value.strip()
         if name in self.values and self.values[name] == value:
             return
-        if name not in self._old: # Changed field
+        if name not in self._old:  # Changed field
             self._old[name] = self.values.get(name)
-        elif self._old[name] == value: # Change of field reverted
+        elif self._old[name] == value:  # Change of field reverted
             del self._old[name]
         self.values[name] = value
 
+    def __contains__(self, item):
+        return item in self.values
+
     def get_value_or_default(self, name):
         """Return the value of a field or the default value if it is undefined
         """
@@ -174,15 +230,13 @@ class Ticket(object):
 
     def get_default(self, name):
         """Return the default value of a field."""
-        field = [field for field in self.fields if field['name'] == name]
-        if field:
-            return field[0].get('value', '')
+        return self.fields.by_name(name, {}).get('value', '')
 
     def populate(self, values):
         """Populate the ticket with 'suitable' values from a dictionary"""
         field_names = [f['name'] for f in self.fields]
         for name in [name for name in values.keys() if name in field_names]:
-            self[name] = values.get(name, '')
+            self[name] = values[name]
 
         # We have to do an extra trick to catch unchecked checkboxes
         for name in [name for name in values.keys() if name[9:] in field_names
@@ -190,11 +244,8 @@ class Ticket(object):
             if name[9:] not in values:
                 self[name[9:]] = '0'
 
-    def insert(self, when=None, db=None):
+    def insert(self, when=None):
         """Add ticket to database.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert not self.exists, 'Cannot insert an existing ticket'
 
@@ -206,20 +257,6 @@ class Ticket(object):
             when = datetime_now(utc)
         self.values['time'] = self.values['changetime'] = when
 
-        # The owner field defaults to the component owner
-        if self.values.get('owner') == '< default >':
-            default_to_owner = ''
-            if self.values.get('component'):
-                try:
-                    component = Component(self.env, self['component'])
-                    default_to_owner = component.owner # even if it's empty
-                except ResourceNotFound:
-                    # No such component exists
-                    pass
-            # If the current owner is "< default >", we need to set it to
-            # _something_ else, even if that something else is blank.
-            self['owner'] = default_to_owner
-
         # Perform type conversions
         db_values = self._to_db_types(self.values)
 
@@ -246,10 +283,10 @@ class Ticket(object):
                 db.executemany(
                     """INSERT INTO ticket_custom (ticket, name, value)
                        VALUES (%s, %s, %s)
-                    """, [(tkt_id, c, db_values.get(c)) for c in custom_fields])
+                    """, [(tkt_id, c, db_values.get(c))
+                          for c in custom_fields])
 
-        self.id = tkt_id
-        self.resource = self.resource(id=tkt_id)
+        self.id = int(tkt_id)
         self._old = {}
 
         for listener in TicketSystem(self.env).change_listeners:
@@ -257,15 +294,25 @@ class Ticket(object):
 
         return self.id
 
-    def save_changes(self, author=None, comment=None, when=None, db=None,
-                     cnum='', replyto=None):
+    def get_comment_number(self, cdate):
+        """Return a comment number by its date."""
+        ts = to_utimestamp(cdate)
+        for cnum, in self.env.db_query("""\
+                SELECT oldvalue FROM ticket_change
+                WHERE ticket=%s AND time=%s AND field='comment'
+                """, (self.id, ts)):
+            try:
+                return int(cnum.rsplit('.', 1)[-1])
+            except ValueError:
+                break
+
+    def save_changes(self, author=None, comment=None, when=None, cnum='',
+                     replyto=None):
         """
         Store ticket changes in the database. The ticket must already exist in
         the database.  Returns False if there were no changes to save, True
         otherwise.
 
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         :since 1.0: the `cnum` parameter is deprecated, and threading should
         be controlled with the `replyto` argument
         """
@@ -277,31 +324,12 @@ class Ticket(object):
         props_unchanged = all(self.values.get(k) == v
                               for k, v in self._old.iteritems())
         if (not comment or not comment.strip()) and props_unchanged:
-            return False # Not modified
+            return False  # Not modified
 
         if when is None:
             when = datetime_now(utc)
         when_ts = to_utimestamp(when)
 
-        if 'component' in self.values:
-            # If the component is changed on a 'new' ticket
-            # then owner field is updated accordingly. (#623).
-            if self.values.get('status') == 'new' \
-                    and 'component' in self._old \
-                    and 'owner' not in self._old:
-                try:
-                    old_comp = Component(self.env, self._old['component'])
-                    old_owner = old_comp.owner or ''
-                    current_owner = self.values.get('owner') or ''
-                    if old_owner == current_owner:
-                        new_comp = Component(self.env, self['component'])
-                        if new_comp.owner:
-                            self['owner'] = new_comp.owner
-                except TracError:
-                    # If the old component has been removed from the database
-                    # we just leave the owner as is.
-                    pass
-
         # Perform type conversions
         db_values = self._to_db_types(self.values)
         old_db_values = self._to_db_types(self._old)
@@ -373,21 +401,19 @@ class Ticket(object):
         values = values.copy()
         for field, value in values.iteritems():
             if field in self.time_fields:
-                values[field] = to_utimestamp(values[field])
+                is_custom_field = field in self.custom_fields
+                values[field] = _datetime_to_db_str(value, is_custom_field)
             else:
                 values[field] = value if value else None
         return values
 
-    def get_changelog(self, when=None, db=None):
+    def get_changelog(self, when=None):
         """Return the changelog as a list of tuples of the form
         (time, author, field, oldvalue, newvalue, permanent).
 
         While the other tuple elements are quite self-explanatory,
         the `permanent` flag is used to distinguish collateral changes
         that are not yet immutable (like attachments, currently).
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         sid = str(self.id)
         when_ts = to_utimestamp(when)
@@ -421,19 +447,21 @@ class Ticket(object):
                 ORDER BY time,permanent,author
                 """
             args = (self.id, sid, sid)
-        return [(from_utimestamp(t), author, field, oldvalue or '',
-                 newvalue or '', permanent)
-                for t, author, field, oldvalue, newvalue, permanent in
-                self.env.db_query(sql, args)]
+        log = []
+        for t, author, field, oldvalue, newvalue, permanent \
+                in self.env.db_query(sql, args):
+            if field in self.time_fields:
+                oldvalue = _db_str_to_datetime(oldvalue)
+                newvalue = _db_str_to_datetime(newvalue)
+            log.append((from_utimestamp(t), author, field,
+                        oldvalue or '', newvalue or '', permanent))
+        return log
 
-    def delete(self, db=None):
+    def delete(self):
         """Delete the ticket.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         with self.env.db_transaction as db:
-            Attachment.delete_all(self.env, 'ticket', self.id, db)
+            Attachment.delete_all(self.env, self.realm, self.id)
             db("DELETE FROM ticket WHERE id=%s", (self.id,))
             db("DELETE FROM ticket_change WHERE ticket=%s", (self.id,))
             db("DELETE FROM ticket_custom WHERE ticket=%s", (self.id,))
@@ -441,11 +469,8 @@ class Ticket(object):
         for listener in TicketSystem(self.env).change_listeners:
             listener.ticket_deleted(self)
 
-    def get_change(self, cnum=None, cdate=None, db=None):
+    def get_change(self, cnum=None, cdate=None):
         """Return a ticket change by its number or date.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         if cdate is None:
             row = self._find_change(cnum)
@@ -561,7 +586,6 @@ class Ticket(object):
             if old_comment is False:
                 # There was no comment field, add one, find the
                 # original author in one of the other changed fields
-                old_author = None
                 for old_author, in db("""
                         SELECT author FROM ticket_change
                         WHERE ticket=%%s AND time=%%s AND NOT field %s LIMIT 1
@@ -588,12 +612,9 @@ class Ticket(object):
                 listener.ticket_comment_modified(self, cdate, author, comment,
                                                  old_comment)
 
-    def get_comment_history(self, cnum=None, cdate=None, db=None):
+    def get_comment_history(self, cnum=None, cdate=None):
         """Retrieve the edit history of a comment identified by its number or
         date.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         if cdate is None:
             row = self._find_change(cnum)
@@ -681,7 +702,7 @@ class Ticket(object):
                         """ % db.prefix_match(),
                         (self.id, ts, db.prefix_match_value('_'))):
                     break
-            return (ts, author, comment)
+            return ts, author, comment
 
 
 def simplify_whitespace(name):
@@ -695,7 +716,9 @@ class AbstractEnum(object):
     type = None
     ticket_col = None
 
-    def __init__(self, env, name=None, db=None):
+    exists = property(lambda self: self._old_value is not None)
+
+    def __init__(self, env, name=None):
         if not self.ticket_col:
             self.ticket_col = self.type
         self.env = env
@@ -713,13 +736,11 @@ class AbstractEnum(object):
             self.value = self._old_value = None
             self.name = self._old_name = None
 
-    exists = property(lambda self: self._old_value is not None)
+    def __repr__(self):
+        return '<%s %r %r>' % (self.__class__.__name__, self.name, self.value)
 
-    def delete(self, db=None):
+    def delete(self):
         """Delete the enum value.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot delete non-existent %s" % self.type
 
@@ -735,16 +756,13 @@ class AbstractEnum(object):
                         enum.value = unicode(int(enum.value) - 1)
                         enum.update()
                 except ValueError:
-                    pass # Ignore cast error for this non-essential operation
+                    pass  # Ignore cast error for this non-essential operation
             TicketSystem(self.env).reset_ticket_fields()
         self.value = self._old_value = None
         self.name = self._old_name = None
 
-    def insert(self, db=None):
+    def insert(self):
         """Add a new enum value.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert not self.exists, "Cannot insert existing %s" % self.type
         self.name = simplify_whitespace(self.name)
@@ -765,11 +783,8 @@ class AbstractEnum(object):
         self._old_name = self.name
         self._old_value = self.value
 
-    def update(self, db=None):
+    def update(self):
         """Update the enum value.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot update non-existent %s" % self.type
         self.name = simplify_whitespace(self.name)
@@ -785,17 +800,13 @@ class AbstractEnum(object):
                 db("UPDATE ticket SET %s=%%s WHERE %s=%%s"
                    % (self.ticket_col, self.ticket_col),
                    (self.name, self._old_name))
-            TicketSystem(self.env).reset_ticket_fields()
+                TicketSystem(self.env).reset_ticket_fields()
 
         self._old_name = self.name
         self._old_value = self.value
 
     @classmethod
-    def select(cls, env, db=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+    def select(cls, env):
         with env.db_query as db:
             for name, value in db("""
                     SELECT name, value FROM enum WHERE type=%s ORDER BY
@@ -817,12 +828,15 @@ class Status(object):
         self.env = env
 
     @classmethod
-    def select(cls, env, db=None):
+    def select(cls, env):
         for state in TicketSystem(env).get_all_status():
             status = cls(env)
             status.name = state
             yield status
 
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.name)
+
 
 class Resolution(AbstractEnum):
     type = 'resolution'
@@ -837,11 +851,10 @@ class Severity(AbstractEnum):
 
 
 class Component(object):
-    def __init__(self, env, name=None, db=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+
+    exists = property(lambda self: self._old_name is not None)
+
+    def __init__(self, env, name=None):
         self.env = env
         self.name = self._old_name = self.owner = self.description = None
         if name:
@@ -856,13 +869,11 @@ class Component(object):
                 raise ResourceNotFound(_("Component %(name)s does not exist.",
                                          name=name))
 
-    exists = property(lambda self: self._old_name is not None)
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.name)
 
-    def delete(self, db=None):
+    def delete(self):
         """Delete the component.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot delete non-existent component"
 
@@ -872,11 +883,8 @@ class Component(object):
             self.name = self._old_name = None
             TicketSystem(self.env).reset_ticket_fields()
 
-    def insert(self, db=None):
+    def insert(self):
         """Insert a new component.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert not self.exists, "Cannot insert existing component"
         self.name = simplify_whitespace(self.name)
@@ -891,11 +899,8 @@ class Component(object):
             self._old_name = self.name
             TicketSystem(self.env).reset_ticket_fields()
 
-    def update(self, db=None):
+    def update(self):
         """Update the component.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot update non-existent component"
         self.name = simplify_whitespace(self.name)
@@ -913,17 +918,13 @@ class Component(object):
                 db("UPDATE ticket SET component=%s WHERE component=%s",
                    (self.name, self._old_name))
                 self._old_name = self.name
-            TicketSystem(self.env).reset_ticket_fields()
+                TicketSystem(self.env).reset_ticket_fields()
 
     @classmethod
-    def select(cls, env, db=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+    def select(cls, env):
         for name, owner, description in env.db_query("""
                 SELECT name, owner, description FROM component ORDER BY name
-            """):
+                """):
             component = cls(env)
             component.name = component._old_name = name
             component.owner = owner or None
@@ -987,7 +988,14 @@ class MilestoneCache(core.Component):
 
 
 class Milestone(object):
-    def __init__(self, env, name=None, db=None):
+
+    realm = 'milestone'
+
+    @property
+    def resource(self):
+        return Resource(self.realm, self.name)  ### .version !!!
+
+    def __init__(self, env, name=None):
         """Create an undefined milestone or fetch one from the database,
         if `name` is given.
 
@@ -999,18 +1007,17 @@ class Milestone(object):
             if not self.cache.fetchone(name, self):
                 raise ResourceNotFound(
                     _("Milestone %(name)s does not exist.",
-                      name=name), _("Invalid milestone name"))
+                      name=name), _("Invalid milestone name."))
         else:
             self.cache.factory((None, None, None, ''), self)
 
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.name)
+
     @property
     def cache(self):
         return MilestoneCache(self.env)
 
-    @property
-    def resource(self):
-        return Resource('milestone', self.name) ### .version !!!
-
     exists = property(lambda self: self._old['name'] is not None)
     is_completed = property(lambda self: self.completed is not None)
     is_late = property(lambda self: self.due and
@@ -1023,23 +1030,18 @@ class Milestone(object):
         if invalidate:
             del self.cache.milestones
 
-    _to_old = checkin #: compatibility with hacks < 0.12.5 (remove in 1.1.1)
-
-    def delete(self, retarget_to=None, author=None, db=None):
+    def delete(self, retarget_to=None, author=None):
         """Delete the milestone.
 
         :since 1.0.2: the `retarget_to` and `author` parameters are
                       deprecated and will be removed in Trac 1.3.1. Tickets
                       should be moved to another milestone by calling
                       `move_tickets` before `delete`.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         with self.env.db_transaction as db:
             self.env.log.info("Deleting milestone %s", self.name)
             db("DELETE FROM milestone WHERE name=%s", (self.name,))
-            Attachment.delete_all(self.env, 'milestone', self.name)
+            Attachment.delete_all(self.env, self.realm, self.name)
             # Don't translate ticket comment (comment:40:ticket:5658)
             self.move_tickets(retarget_to, author, "Milestone deleted")
             self._old['name'] = None
@@ -1049,11 +1051,8 @@ class Milestone(object):
         for listener in TicketSystem(self.env).milestone_change_listeners:
             listener.milestone_deleted(self)
 
-    def insert(self, db=None):
+    def insert(self):
         """Insert a new milestone.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         self.name = simplify_whitespace(self.name)
         if not self.name:
@@ -1071,11 +1070,8 @@ class Milestone(object):
         for listener in TicketSystem(self.env).milestone_change_listeners:
             listener.milestone_created(self)
 
-    def update(self, db=None, author=None):
+    def update(self, author=None):
         """Update the milestone.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         self.name = simplify_whitespace(self.name)
         if not self.name:
@@ -1087,8 +1083,8 @@ class Milestone(object):
                 # Update milestone field in tickets
                 self.move_tickets(self.name, author, "Milestone renamed")
                 # Reparent attachments
-                Attachment.reparent_all(self.env, 'milestone', old['name'],
-                                        'milestone', self.name)
+                Attachment.reparent_all(self.env, self.realm, old['name'],
+                                        self.realm, self.name)
 
             self.env.log.info("Updating milestone '%s'", old['name'])
             db("""UPDATE milestone
@@ -1126,7 +1122,7 @@ class Milestone(object):
             if not self.cache.fetchone(new_milestone):
                 raise ResourceNotFound(
                     _("Milestone %(name)s does not exist.",
-                      name=new_milestone), _("Invalid milestone name"))
+                      name=new_milestone), _("Invalid milestone name."))
         now = datetime_now(utc)
         with self.env.db_transaction as db:
             sql = "SELECT id FROM ticket WHERE milestone=%s"
@@ -1144,11 +1140,7 @@ class Milestone(object):
         return tkt_ids
 
     @classmethod
-    def select(cls, env, include_completed=True, db=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+    def select(cls, env, include_completed=True):
         milestones = MilestoneCache(env).fetchall()
         if not include_completed:
             milestones = [m for m in milestones if m.completed is None]
@@ -1159,25 +1151,90 @@ class Milestone(object):
         return sorted(milestones, key=milestone_order)
 
 
+class Report(object):
+
+    realm = 'report'
+
+    @property
+    def exists(self):
+        return self.id is not None
+
+    def __init__(self, env, id=None):
+        self.env = env
+        self.id = self.title = self.query = self.description = None
+        if id is not None:
+            id_as_int = as_int(id, None)
+            if id_as_int is not None:
+                for title, description, query in self.env.db_query("""
+                        SELECT title, description, query FROM report
+                        WHERE id=%s
+                        """, (id_as_int,)):
+                    self.id = id_as_int
+                    self.title = title or ''
+                    self.description = description or ''
+                    self.query = query or ''
+                    return
+            raise ResourceNotFound(_("Report {%(num)s} does not exist.",
+                                     num=id), _("Invalid Report Number"))
+
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.id)
+
+    def delete(self):
+        """Delete the report."""
+        assert self.exists, "Cannot delete non-existent report"
+        self.env.db_transaction("DELETE FROM report WHERE id=%s", (self.id,))
+        self.id = None
+
+    def insert(self):
+        """Insert a new report."""
+        assert not self.exists, "Cannot insert existing report"
+
+        self.env.log.debug("Creating new report '%s'", self.id)
+        with self.env.db_transaction as db:
+            cursor = db.cursor()
+            cursor.execute("""
+                INSERT INTO report (title,query,description) VALUES (%s,%s,%s)
+                """, (self.title, self.query, self.description))
+            self.id = db.get_last_id(cursor, 'report')
+
+    def update(self):
+        self.env.db_transaction("""
+            UPDATE report SET title=%s, query=%s, description=%s
+            WHERE id=%s
+            """, (self.title, self.query, self.description, self.id))
+
+    @classmethod
+    def select(cls, env, sort='id', asc=True):
+        for id, title, description, query in env.db_query("""
+                SELECT id, title, description, query
+                FROM report ORDER BY %s %s
+                """ % ('title' if sort == 'title' else 'id',
+                       '' if asc else 'DESC')):
+            report = cls(env)
+            report.id = id
+            report.title = title
+            report.description = description
+            report.query = query
+            yield report
+
+
 def group_milestones(milestones, include_completed):
     """Group milestones into "open with due date", "open with no due date",
-    and possibly "completed". Return a list of (label, milestones) tuples."""
-    def category(m):
-        return 1 if m.is_completed else 2 if m.due else 3
-    open_due_milestones, open_not_due_milestones, \
-        closed_milestones = partition([(m, category(m))
-            for m in milestones], (2, 3, 1))
-    groups = [
-        (_('Open (by due date)'), open_due_milestones),
-        (_('Open (no due date)'), open_not_due_milestones),
-    ]
-    if include_completed:
-        groups.append((_('Closed'), closed_milestones))
-    return groups
+    and possibly "completed". Return a list of (label, milestones) tuples.
+
+    :since 1.1.3: the function has been moved to `trac.ticket.roadmap`. It
+                  will be removed from `trac.ticket.model` in 1.3.1.
+    """
+    from trac.ticket.roadmap import group_milestones
+    return group_milestones(milestones, include_completed)
 
 
 class Version(object):
-    def __init__(self, env, name=None, db=None):
+
+    exists = property(lambda self: self._old_name is not None)
+
+    def __init__(self, env, name=None):
         self.env = env
         self.name = self._old_name = self.time = self.description = None
         if name:
@@ -1192,13 +1249,11 @@ class Version(object):
                 raise ResourceNotFound(_("Version %(name)s does not exist.",
                                          name=name))
 
-    exists = property(lambda self: self._old_name is not None)
+    def __repr__(self):
+        return '<%s %r>' % (self.__class__.__name__, self.name)
 
-    def delete(self, db=None):
+    def delete(self):
         """Delete the version.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot delete non-existent version"
 
@@ -1208,11 +1263,8 @@ class Version(object):
             self.name = self._old_name = None
             TicketSystem(self.env).reset_ticket_fields()
 
-    def insert(self, db=None):
+    def insert(self):
         """Insert a new version.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert not self.exists, "Cannot insert existing version"
         self.name = simplify_whitespace(self.name)
@@ -1226,11 +1278,8 @@ class Version(object):
             self._old_name = self.name
             TicketSystem(self.env).reset_ticket_fields()
 
-    def update(self, db=None):
+    def update(self):
         """Update the version.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         assert self.exists, "Cannot update non-existent version"
         self.name = simplify_whitespace(self.name)
@@ -1252,11 +1301,7 @@ class Version(object):
         TicketSystem(self.env).reset_ticket_fields()
 
     @classmethod
-    def select(cls, env, db=None):
-        """
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
-        """
+    def select(cls, env):
         versions = []
         for name, time, description in env.db_query("""
                 SELECT name, time, description FROM version"""):
@@ -1266,5 +1311,5 @@ class Version(object):
             version.description = description or ''
             versions.append(version)
         def version_order(v):
-            return (v.time or utcmax, embedded_numbers(v.name))
+            return v.time or utcmax, embedded_numbers(v.name)
         return sorted(versions, key=version_order, reverse=True)
