@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2003-2009 Edgewall Software
+# Copyright (C) 2003-2013 Edgewall Software
 # Copyright (C) 2003-2004 Jonas Borgström <jonas@edgewall.com>
 # Copyright (C) 2006 Christian Boos <cboos@edgewall.org>
 # Copyright (C) 2006 Matthew Good <trac@matt-good.net>
@@ -16,8 +16,6 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import with_statement
-
 import csv
 import re
 from StringIO import StringIO
@@ -30,10 +28,12 @@ from trac.db import get_column_names
 from trac.perm import IPermissionRequestor
 from trac.resource import Resource, ResourceNotFound
 from trac.ticket.api import TicketSystem
-from trac.util import as_bool, as_int, content_disposition
+from trac.ticket.model import Report
+from trac.util import as_int, content_disposition
 from trac.util.datefmt import format_datetime, format_time, from_utimestamp
 from trac.util.presentation import Paginator
-from trac.util.text import exception_to_unicode, to_unicode, quote_query_string
+from trac.util.text import (exception_to_unicode, quote_query_string, sub_vars,
+                            sub_vars_re, to_unicode)
 from trac.util.translation import _, tag_
 from trac.web.api import HTTPBadRequest, IRequestHandler, RequestDone
 from trac.web.chrome import (INavigationContributor, Chrome,
@@ -104,9 +104,9 @@ def split_sql(sql, clause_re, skel=None):
         skel = sql_skeleton(sql)
     blocks = clause_re.split(skel.upper())
     if len(blocks) == 2:
-        return sql[:len(blocks[0])], sql[-len(blocks[1]):] # (before, after)
+        return sql[:len(blocks[0])], sql[-len(blocks[1]):]  # (before, after)
     else:
-        return sql, '' # no single clause separator
+        return sql, ''  # no single clause separator
 
 
 class ReportModule(Component):
@@ -114,13 +114,18 @@ class ReportModule(Component):
     implements(INavigationContributor, IPermissionRequestor, IRequestHandler,
                IWikiSyntaxProvider)
 
+    realm = Report.realm
+
     items_per_page = IntOption('report', 'items_per_page', 100,
         """Number of tickets displayed per page in ticket reports,
-        by default (''since 0.11'')""")
+        by default.
+        """)
 
     items_per_page_rss = IntOption('report', 'items_per_page_rss', 0,
-        """Number of tickets displayed in the rss feeds for reports
-        (''since 0.11'')""")
+        """Number of tickets displayed in the rss feeds for reports.
+        """)
+
+    REPORT_LIST_ID = -1  # Resource id of the report list page
 
     # INavigationContributor methods
 
@@ -128,7 +133,7 @@ class ReportModule(Component):
         return 'tickets'
 
     def get_navigation_items(self, req):
-        if 'REPORT_VIEW' in req.perm:
+        if 'REPORT_VIEW' in req.perm(self.realm, self.REPORT_LIST_ID):
             yield ('mainnav', 'tickets', tag.a(_('View Tickets'),
                                                href=req.href.report()))
 
@@ -142,7 +147,8 @@ class ReportModule(Component):
     # IRequestHandler methods
 
     def match_request(self, req):
-        match = re.match(r'/report(?:/(?:([0-9]+)|-1))?$', req.path_info)
+        match = re.match(r'/report(?:/(?:([0-9]+)|%s))?$'
+                         % self.REPORT_LIST_ID, req.path_info)
         if match:
             if match.group(1):
                 req.args['id'] = match.group(1)
@@ -150,14 +156,12 @@ class ReportModule(Component):
 
     def process_request(self, req):
         # did the user ask for any special report?
-        id = int(req.args.get('id', -1))
-        if id != -1:
-            req.perm('report', id).require('REPORT_VIEW')
-        else:
-            req.perm.require('REPORT_VIEW')
+        id = req.args.getint('id', self.REPORT_LIST_ID)
+        req.perm(self.realm, id).require('REPORT_VIEW')
 
         data = {}
         action = req.args.get('action', 'view')
+        template = None
         if req.method == 'POST':
             if action == 'new':
                 self._do_create(req)
@@ -174,9 +178,9 @@ class ReportModule(Component):
         elif action == 'delete':
             template = 'report_delete.html'
             data = self._render_confirm_delete(req, id)
-        elif id == -1:
+        elif id == self.REPORT_LIST_ID:
             template, data, content_type = self._render_list(req)
-            if content_type: # i.e. alternate format
+            if content_type:  # i.e. alternate format
                 return template, data, content_type
             if action == 'clear':
                 if 'query_href' in req.session:
@@ -185,14 +189,15 @@ class ReportModule(Component):
                     del req.session['query_tickets']
         else:
             template, data, content_type = self._render_view(req, id)
-            if content_type: # i.e. alternate format
+            if content_type:  # i.e. alternate format
                 return template, data, content_type
 
         from trac.ticket.query import QueryModule
-        show_query_link = 'TICKET_VIEW' in req.perm and \
+        show_query_link = 'TICKET_VIEW' in req.perm(TicketSystem.realm) and \
                           self.env.is_component_enabled(QueryModule)
 
-        if id != -1 or action == 'new':
+        if (id != self.REPORT_LIST_ID or action == 'new') and \
+                'REPORT_VIEW' in req.perm(self.realm, self.REPORT_LIST_ID):
             add_ctxtnav(req, _('Available Reports'), href=req.href.report())
             add_link(req, 'up', req.href.report(), _('Available Reports'))
         elif show_query_link:
@@ -201,7 +206,7 @@ class ReportModule(Component):
         # Kludge: only show link to custom query if the query module
         # is actually enabled
         if show_query_link:
-            add_ctxtnav(req, _('Custom Query'), href=req.href.query())
+            add_ctxtnav(req, _('New Custom Query'), href=req.href.query())
             data['query_href'] = req.href.query()
             data['saved_query_href'] = req.session.get('query_href')
         else:
@@ -213,63 +218,58 @@ class ReportModule(Component):
     # Internal methods
 
     def _do_create(self, req):
-        req.perm.require('REPORT_CREATE')
+        req.perm(self.realm).require('REPORT_CREATE')
 
         if 'cancel' in req.args:
             req.redirect(req.href.report())
 
-        title = req.args.get('title', '')
-        query = req.args.get('query', '')
-        description = req.args.get('description', '')
-        with self.env.db_transaction as db:
-            cursor = db.cursor()
-            cursor.execute("""
-                INSERT INTO report (title,query,description) VALUES (%s,%s,%s)
-                """, (title, query, description))
-            report_id = db.get_last_id(cursor, 'report')
+        report = Report(self.env)
+        report.title = req.args.get('title', '')
+        report.query = req.args.get('query', '')
+        report.description = req.args.get('description', '')
+        report.insert()
         add_notice(req, _("The report has been created."))
-        req.redirect(req.href.report(report_id))
+        req.redirect(req.href.report(report.id))
 
     def _do_delete(self, req, id):
-        req.perm('report', id).require('REPORT_DELETE')
+        req.perm(self.realm, id).require('REPORT_DELETE')
 
         if 'cancel' in req.args:
             req.redirect(req.href.report(id))
 
-        self.env.db_transaction("DELETE FROM report WHERE id=%s", (id,))
+        Report(self.env, id).delete()
         add_notice(req, _("The report {%(id)d} has been deleted.", id=id))
         req.redirect(req.href.report())
 
     def _do_save(self, req, id):
         """Save report changes to the database"""
-        req.perm('report', id).require('REPORT_MODIFY')
+        req.perm(self.realm, id).require('REPORT_MODIFY')
 
         if 'cancel' not in req.args:
-            title = req.args.get('title', '')
-            query = req.args.get('query', '')
-            description = req.args.get('description', '')
-            self.env.db_transaction("""
-                UPDATE report SET title=%s, query=%s, description=%s
-                WHERE id=%s
-                """, (title, query, description, id))
+            report = Report(self.env, id)
+            report.title = req.args.get('title', '')
+            report.query = req.args.get('query', '')
+            report.description = req.args.get('description', '')
+            report.update()
             add_notice(req, _("Your changes have been saved."))
         req.redirect(req.href.report(id))
 
     def _render_confirm_delete(self, req, id):
-        req.perm('report', id).require('REPORT_DELETE')
+        req.perm(self.realm, id).require('REPORT_DELETE')
 
-        title = self.get_report(id)[0]
+        title = Report(self.env, id).title
         return {'title': _("Delete Report {%(num)s} %(title)s", num=id,
                            title=title),
                 'action': 'delete',
                 'report': {'id': id, 'title': title}}
 
     def _render_editor(self, req, id, copy):
-        if id != -1:
-            req.perm('report', id).require('REPORT_MODIFY')
-            title, description, query = self.get_report(id)
+        if id != self.REPORT_LIST_ID:
+            req.perm(self.realm, id).require('REPORT_MODIFY')
+            r = Report(self.env, id)
+            title, description, query = r.title, r.description, r.query
         else:
-            req.perm.require('REPORT_CREATE')
+            req.perm(self.realm).require('REPORT_CREATE')
             title = description = query = ''
 
         # an explicitly given 'query' parameter will override the saved query
@@ -278,14 +278,11 @@ class ReportModule(Component):
         if copy:
             title += ' (copy)'
 
-        if copy or id == -1:
-            data = {'title': _('Create New Report'),
-                    'action': 'new',
+        if copy or id == self.REPORT_LIST_ID:
+            data = {'action': 'new',
                     'error': None}
         else:
-            data = {'title': _('Edit Report {%(num)d} %(title)s', num=id,
-                               title=title),
-                    'action': 'edit',
+            data = {'action': 'edit',
                     'error': req.args.get('error')}
 
         data['report'] = {'id': id, 'title': title,
@@ -295,15 +292,12 @@ class ReportModule(Component):
     def _render_list(self, req):
         """Render the list of available reports."""
         sort = req.args.get('sort', 'report')
-        asc = as_bool(req.args.get('asc', 1))
+        asc = req.args.getbool('asc', True)
         format = req.args.get('format')
 
-        rows = self.env.db_query("""
-                SELECT id, title, description FROM report ORDER BY %s %s
-                """ % ('title' if sort == 'title' else 'id',
-                       '' if asc else 'DESC'))
-        rows = [(id, title, description) for id, title, description in rows
-                if 'REPORT_VIEW' in req.perm('report', id)]
+        rows = [(report.id, report.title, report.description)
+                for report in Report.select(self.env, sort, asc)
+                if 'REPORT_VIEW' in req.perm(self.realm, report.id)]
 
         if format == 'rss':
             data = {'rows': rows}
@@ -319,7 +313,7 @@ class ReportModule(Component):
 
         def report_href(**kwargs):
             return req.href.report(sort=req.args.get('sort'),
-                                   asc='1' if asc else '0', **kwargs)
+                                   asc=1 if asc else 0, **kwargs)
 
         add_link(req, 'alternate',
                  auth_link(req, report_href(format='rss')),
@@ -330,10 +324,10 @@ class ReportModule(Component):
                  _('Tab-delimited Text'), 'text/plain')
 
         reports = [(id, title, description,
-                    'REPORT_MODIFY' in req.perm('report', id),
-                    'REPORT_DELETE' in req.perm('report', id))
+                    'REPORT_MODIFY' in req.perm(self.realm, id),
+                    'REPORT_DELETE' in req.perm(self.realm, id))
                    for id, title, description in rows]
-        data = {'reports': reports, 'sort': sort, 'asc': asc}
+        data = {'reports': reports, 'sort': sort, 'asc': 1 if asc else 0}
 
         return 'report_list.html', data, None
 
@@ -342,10 +336,11 @@ class ReportModule(Component):
 
     def _render_view(self, req, id):
         """Retrieve the report results and pre-process them for rendering."""
-        title, description, sql = self.get_report(id)
+        r = Report(self.env, id)
+        title, description, sql = r.title, r.description, r.query
         try:
             args = self.get_var_args(req)
-        except ValueError, e:
+        except ValueError as e:
             raise TracError(_("Report failed: %(error)s", error=e))
 
         # If this is a saved custom query, redirect to the query module
@@ -360,7 +355,7 @@ class ReportModule(Component):
             query = query if query[0] == '?' else query[6:]
             report_id = 'report=%s' % id
             if 'report=' in query:
-                if not report_id in query:
+                if report_id not in query:
                     err = _('When specified, the report number should be '
                             '"%(num)s".', num=id)
                     req.redirect(req.href.report(id, action='edit', error=err))
@@ -370,13 +365,14 @@ class ReportModule(Component):
                 query += report_id
             req.redirect(req.href.query() + quote_query_string(query))
         elif query.startswith('query:'):
+            from trac.ticket.query import Query, QuerySyntaxError
             try:
-                from trac.ticket.query import Query, QuerySyntaxError
                 query = Query.from_string(self.env, query[6:], report=id)
-                req.redirect(query.get_href(req))
-            except QuerySyntaxError, e:
+            except QuerySyntaxError as e:
                 req.redirect(req.href.report(id, action='edit',
                                              error=to_unicode(e)))
+            else:
+                req.redirect(query.get_href(req.href))
 
         format = req.args.get('format')
         if format == 'sql':
@@ -384,19 +380,19 @@ class ReportModule(Component):
 
         title = '{%i} %s' % (id, title)
 
-        report_resource = Resource('report', id)
+        report_resource = Resource(self.realm, id)
         req.perm(report_resource).require('REPORT_VIEW')
         context = web_context(req, report_resource)
 
-        page = as_int(req.args.get('page'), 1)
+        page = req.args.getint('page', 1)
         default_max = {'rss': self.items_per_page_rss,
                        'csv': 0, 'tab': 0}.get(format, self.items_per_page)
         max = req.args.get('max')
-        limit = as_int(max, default_max, min=0) # explict max takes precedence
+        limit = as_int(max, default_max, min=0)  # explict max takes precedence
         offset = (page - 1) * limit
 
         sort_col = req.args.get('sort', '')
-        asc = as_bool(req.args.get('asc', 1))
+        asc = req.args.getbool('asc', True)
 
         def report_href(**kwargs):
             """Generate links to this report preserving user variables,
@@ -409,22 +405,20 @@ class ReportModule(Component):
             if max:
                 params['max'] = max
             params.update(kwargs)
-            params['asc'] = '1' if params.get('asc', asc) else '0'
+            params['asc'] = as_int(params.get('asc'), asc, min=0, max=1)
             return req.href.report(id, params)
 
         data = {'action': 'view',
                 'report': {'id': id, 'resource': report_resource},
                 'context': context,
-                'title': title, 'description': description,
+                'title': sub_vars(title, args),
+                'description': sub_vars(description or '', args),
                 'max': limit, 'args': args, 'show_args_form': False,
                 'message': None, 'paginator': None,
                 'report_href': report_href,
                 }
 
-        res = None
-        with self.env.db_query as db:
-            res = self.execute_paginated_report(req, db, id, sql, args, limit,
-                                                offset)
+        res = self.execute_paginated_report(req, id, sql, args, limit, offset)
 
         if len(res) == 2:
             e, sql = res
@@ -513,15 +507,15 @@ class ReportModule(Component):
 
             header_group = header_groups[-1]
 
-            if col.startswith('__') and col.endswith('__'): # __col__
+            if col.startswith('__') and col.endswith('__'):  # __col__
                 header['hidden'] = True
-            elif col[0] == '_' and col[-1] == '_':          # _col_
+            elif col[0] == '_' and col[-1] == '_':           # _col_
                 header_group = []
                 header_groups.append(header_group)
                 header_groups.append([])
-            elif col[0] == '_':                             # _col
+            elif col[0] == '_':                              # _col
                 header['hidden'] = True
-            elif col[-1] == '_':                            # col_
+            elif col[-1] == '_':                             # col_
                 header_groups.append([])
             header_group.append(header)
 
@@ -536,7 +530,7 @@ class ReportModule(Component):
             col_idx = 0
             cell_groups = []
             row = {'cell_groups': cell_groups}
-            realm = 'ticket'
+            realm = TicketSystem.realm
             parent_realm = ''
             parent_id = ''
             email_cells = []
@@ -577,7 +571,7 @@ class ReportModule(Component):
             else:
                 resource = Resource(realm, row.get('id'))
             # FIXME: for now, we still need to hardcode the realm in the action
-            if resource.realm.upper()+'_VIEW' not in req.perm(resource):
+            if resource.realm.upper() + '_VIEW' not in req.perm(resource):
                 continue
             authorized_results.append(result)
             if email_cells:
@@ -598,7 +592,6 @@ class ReportModule(Component):
                      'numrows': numrows})
 
         if format == 'rss':
-            data['email_map'] = chrome.get_email_map()
             data['context'] = web_context(req, report_resource,
                                           absurls=True)
             return 'report.rss', data, 'application/rss+xml'
@@ -620,7 +613,7 @@ class ReportModule(Component):
                      _('Comma-delimited Text'), 'text/plain')
             add_link(req, 'alternate', report_href(format='tab', page=p),
                      _('Tab-delimited Text'), 'text/plain')
-            if 'REPORT_SQL_VIEW' in req.perm('report', id):
+            if 'REPORT_SQL_VIEW' in req.perm(self.realm, id):
                 add_link(req, 'alternate',
                          req.href.report(id=id, format='sql'),
                          _('SQL Query'), 'text/plain')
@@ -650,31 +643,42 @@ class ReportModule(Component):
                     args=", ".join(missing_args)))
             return 'report_view.html', data, None
 
-    def execute_report(self, req, db, id, sql, args):
-        """Execute given sql report (0.10 backward compatibility method)
-
-        :see: ``execute_paginated_report``
+    def execute_paginated_report(self, req, *largs, **kwargs):
         """
-        res = self.execute_paginated_report(req, db, id, sql, args)
-        if len(res) == 2:
-            raise res[0]
-        return res[:5]
+        :param req: `Request` object.
+        :param db: Database connection object (optional and deprecated).
+        :param id: Integer id of the report.
+        :param sql: SQL query that generates the report.
+        :param args: SQL query arguments.
+        :param limit: Maximum number of results to return (optional).
+        :param offset: Offset to start of results (optional).
 
-    def execute_paginated_report(self, req, db, id, sql, args,
-                                 limit=0, offset=0):
-        sql, args, missing_args = self.sql_sub_vars(sql, args, db)
+        :deprecated: since 1.1.2, the `db` positional argument is deprecated
+                     and will be removed in 1.3.1.
+        """
+        from trac.db.util import ConnectionWrapper
+        if isinstance(largs[0], ConnectionWrapper):
+            return self._execute_paginated_report(req, *largs, **kwargs)
+        with self.env.db_query as db:
+            return self._execute_paginated_report(req, db, *largs, **kwargs)
+
+    def _execute_paginated_report(self, req, db, id, sql, args,
+                                  limit=0, offset=0):
+        """Deprecated and will be removed in Trac 1.3.1. Call
+        `execute_paginated_report` instead."""
+        sql, args, missing_args = self.sql_sub_vars(sql, args)
         if not sql:
             raise TracError(_("Report {%(num)s} has no SQL query.", num=id))
         self.log.debug('Report {%d} with SQL "%s"', id, sql)
         self.log.debug('Request args: %r', req.args)
 
-        cursor = db.cursor()
-
         num_items = 0
         order_by = []
         limit_offset = None
         base_sql = sql.replace(SORT_COLUMN, '1').replace(LIMIT_OFFSET, '')
-        if id == -1 or limit == 0:
+
+        cursor = db.cursor()
+        if id == self.REPORT_LIST_ID or limit == 0:
             sql = base_sql
         else:
             # The number of tickets is obtained
@@ -682,7 +686,7 @@ class ReportModule(Component):
             self.log.debug("Report {%d} SQL (count): %s", id, count_sql)
             try:
                 cursor.execute(count_sql, args)
-            except Exception, e:
+            except Exception as e:
                 self.log.warn('Exception caught while executing Report {%d}: '
                               '%r, args %r%s', id, count_sql, args,
                               exception_to_unicode(e, traceback=True))
@@ -694,7 +698,7 @@ class ReportModule(Component):
             self.log.debug("Report {%d} SQL (col names): %s", id, colnames_sql)
             try:
                 cursor.execute(colnames_sql, args)
-            except Exception, e:
+            except Exception as e:
                 self.log.warn('Exception caught while executing Report {%d}: '
                               '%r, args %r%s', id, colnames_sql, args,
                               exception_to_unicode(e, traceback=True))
@@ -703,8 +707,8 @@ class ReportModule(Component):
 
             # The ORDER BY columns are inserted
             sort_col = req.args.get('sort', '')
-            asc = req.args.get('asc', '1')
-            self.log.debug("%r %s (%s)", cols, sort_col, asc and '^' or 'v')
+            asc = req.args.getbool('asc', True)
+            self.log.debug("%r %s (%s)", cols, sort_col, '^' if asc else 'v')
             order_cols = []
             if sort_col and sort_col not in cols:
                 raise TracError(_('Query parameter "sort=%(sort_col)s" '
@@ -714,7 +718,7 @@ class ReportModule(Component):
                 order_cols.append('__group__')
             if sort_col:
                 sort_col = '%s %s' % (db.quote(sort_col),
-                                      asc == '1' and 'ASC' or 'DESC')
+                                      'ASC' if asc else 'DESC')
 
             if SORT_COLUMN in sql:
                 # Method 1: insert sort_col at specified position
@@ -728,7 +732,7 @@ class ReportModule(Component):
                 # is there already an ORDER BY in the original sql?
                 skel = sql_skeleton(sql)
                 before, after = split_sql(sql, _order_by_re, skel)
-                if after: # there were some other criterions, keep them
+                if after:  # there were some other criterions, keep them
                     order_by.append(after)
                 sql = ' '.join([before, 'ORDER BY', ', '.join(order_by)])
 
@@ -748,7 +752,7 @@ class ReportModule(Component):
             self.log.debug("Report {%d} SQL (order + limit): %s", id, sql)
         try:
             cursor.execute(sql, args)
-        except Exception, e:
+        except Exception as e:
             self.log.warn('Exception caught while executing Report {%d}: '
                           '%r, args %r%s', id, sql, args,
                           exception_to_unicode(e, traceback=True))
@@ -767,18 +771,13 @@ class ReportModule(Component):
         return cols, rows, num_items, missing_args, limit_offset
 
     def get_report(self, id):
-        try:
-            number = int(id)
-        except (ValueError, TypeError):
-            pass
-        else:
-            for title, description, sql in self.env.db_query("""
-                    SELECT title, description, query from report WHERE id=%s
-                    """, (number,)):
-                return title, description, sql
+        """Returns the `title`, `description` and `sql` for a report.
 
-        raise ResourceNotFound(_("Report {%(num)s} does not exist.", num=id),
-                               _("Invalid Report Number"))
+        :since 1.2: Deprecated and will be removed in 1.3.1. Use the
+                    `Report` model class instead.
+        """
+        report = Report(self.env, id)
+        return report.title, report.description, report.query
 
     def get_var_args(self, req):
         # reuse somehow for #9574 (wiki vars)
@@ -794,15 +793,13 @@ class ReportModule(Component):
 
         return report_args
 
-    def sql_sub_vars(self, sql, args, db=None):
+    def sql_sub_vars(self, sql, args):
         """Extract $XYZ-style variables from the `sql` query.
-
-        :since 1.0: the `db` parameter is no longer needed and will be removed
-        in version 1.1.1
         """
         names = set()
         values = []
         missing_args = []
+
         def add_value(aname):
             names.add(aname)
             try:
@@ -812,16 +809,14 @@ class ReportModule(Component):
                 missing_args.append(aname)
             values.append(arg)
 
-        var_re = re.compile("[$]([A-Z_][A-Z0-9_]*)")
-
         # simple parameter substitution outside literal
         def repl(match):
             add_value(match.group(1))
             return '%s'
 
         # inside a literal break it and concatenate with the parameter
-        def repl_literal(expr):
-            parts = var_re.split(expr[1:-1])
+        def repl_literal(expr, db):
+            parts = sub_vars_re.split(expr[1:-1])
             if len(parts) == 1:
                 return expr
             params = parts[1::2]
@@ -829,17 +824,18 @@ class ReportModule(Component):
             parts[1::2] = ['%s'] * len(params)
             for param in params:
                 add_value(param)
-            return self.env.get_read_db().concat(*parts)
+            return db.concat(*parts)
 
         sql_io = StringIO()
 
         # break SQL into literals and non-literals to handle replacing
         # variables within them with query parameters
-        for expr in re.split("('(?:[^']|(?:''))*')", sql):
-            if expr.startswith("'"):
-                sql_io.write(repl_literal(expr))
-            else:
-                sql_io.write(var_re.sub(repl, expr))
+        with self.env.db_query as db:
+            for expr in re.split("('(?:[^']|(?:''))*')", sql):
+                if expr.startswith("'"):
+                    sql_io.write(repl_literal(expr, db))
+                else:
+                    sql_io.write(sub_vars_re.sub(repl, expr))
 
         # Remove arguments that don't appear in the SQL query
         for name in set(args) - names:
@@ -902,7 +898,7 @@ class ReportModule(Component):
         raise RequestDone
 
     def _send_sql(self, req, id, title, description, sql):
-        req.perm('report', id).require('REPORT_SQL_VIEW')
+        req.perm(self.realm, id).require('REPORT_SQL_VIEW')
 
         out = StringIO()
         out.write('-- ## %s: %s ## --\n\n' % (id, title.encode('utf-8')))
@@ -940,12 +936,12 @@ class ReportModule(Component):
             return intertrac
         id, args, fragment = formatter.split_link(target)
         try:
-            self.get_report(id)
+            Report(self.env, id)
         except ResourceNotFound:
             return tag.a(label, class_='missing report',
                          title=_("report does not exist"))
         else:
-            if 'REPORT_VIEW' in formatter.req.perm('report', id):
+            if 'REPORT_VIEW' in formatter.req.perm(self.realm, id):
                 return tag.a(label, href=formatter.href.report(id) + args,
                              class_='report')
             else:
