@@ -1,24 +1,35 @@
-from __future__ import with_statement
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2005-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 
 from datetime import datetime, timedelta
-import os.path
 from StringIO import StringIO
 import tempfile
-import shutil
 import unittest
 
+import trac.tests.compat
 from trac import core
 from trac.attachment import Attachment
 from trac.core import TracError, implements
-from trac.resource import ResourceNotFound
+from trac.resource import Resource, ResourceNotFound
+from trac.test import EnvironmentStub
 from trac.ticket.model import (
     Ticket, Component, Milestone, Priority, Type, Version
 )
+from trac.ticket.roadmap import MilestoneModule
 from trac.ticket.api import (
     IMilestoneChangeListener, ITicketChangeListener, TicketSystem
 )
-from trac.test import EnvironmentStub
-from trac.util.datefmt import from_utimestamp, to_utimestamp, utc
+from trac.util.datefmt import datetime_now, from_utimestamp, to_utimestamp, utc
 
 
 class TestTicketChangeListener(core.Component):
@@ -40,6 +51,36 @@ class TestTicketChangeListener(core.Component):
         self.action = 'deleted'
         self.ticket = ticket
 
+    # the listener has no ticket_comment_modified and ticket_change_deleted
+
+
+class TestTicketChangeListener_2(core.Component):
+    implements(ITicketChangeListener)
+
+    def ticket_created(self, ticket):
+        pass
+
+    def ticket_changed(self, ticket, comment, author, old_values):
+        pass
+
+    def ticket_deleted(self, ticket):
+        pass
+
+    def ticket_comment_modified(self, ticket, cdate, author, comment,
+                                old_comment):
+        self.action = 'comment_modified'
+        self.ticket = ticket
+        self.cdate = cdate
+        self.author = author
+        self.comment = comment
+        self.old_comment = old_comment
+
+    def ticket_change_deleted(self, ticket, cdate, changes):
+        self.action = 'change_deleted'
+        self.ticket = ticket
+        self.cdate = cdate
+        self.changes = changes
+
 
 class TicketTestCase(unittest.TestCase):
 
@@ -67,6 +108,15 @@ class TicketTestCase(unittest.TestCase):
         ticket['foo'] = 'This is a custom field'
         return ticket
 
+    def test_resource_id_is_none(self):
+        ticket = Ticket(self.env)
+        self.assertEqual(Resource('ticket'), ticket.resource)
+
+    def test_resource_exists(self):
+        ticket_id = self._insert_ticket('Foo')
+        ticket = Ticket(self.env, ticket_id)
+        self.assertEqual(Resource('ticket', 1), ticket.resource)
+
     def test_invalid_ticket_id(self):
         self.assertEqual(Ticket.id_is_valid(-1), False)
         self.assertEqual(Ticket.id_is_valid(0), False)
@@ -76,6 +126,12 @@ class TicketTestCase(unittest.TestCase):
         self.assertRaises(ResourceNotFound, Ticket, self.env, -1)
         self.assertRaises(ResourceNotFound, Ticket, self.env, 1L << 32)
 
+    def test_repr(self):
+        ticket = self._create_a_ticket()
+        self.assertEqual("<Ticket None>", repr(ticket))
+        ticket.insert()
+        self.assertEqual("<Ticket 1>", repr(ticket))
+
     def test_create_ticket_1(self):
         ticket = self._create_a_ticket()
         self.assertEqual('santa', ticket['reporter'])
@@ -117,9 +173,9 @@ class TicketTestCase(unittest.TestCase):
         log = ticket3.get_changelog()
         self.assertEqual(len(log), 3)
         ok_vals = ['foo', 'summary', 'comment']
-        self.failUnless(log[0][2] in ok_vals)
-        self.failUnless(log[1][2] in ok_vals)
-        self.failUnless(log[2][2] in ok_vals)
+        self.assertIn(log[0][2], ok_vals)
+        self.assertIn(log[1][2], ok_vals)
+        self.assertIn(log[2][2], ok_vals)
 
     def test_create_ticket_5(self):
         ticket3 = self._modify_a_ticket()
@@ -129,6 +185,71 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual(len(log), 0)
         self.assertRaises(TracError, Ticket, self.env, 1)
 
+    def _test_empty_strings_stored_as_null(self, ticket):
+        """Ticket fields that contain empty strings are stored as NULLs
+        in the database. NULLs are cast to empty strings on fetch.
+        """
+        std_fields = [name for name in ticket.std_fields
+                           if name not in ticket.protected_fields]
+        cst_fields = [name for name in ticket.custom_fields
+                           if name not in ticket.protected_fields]
+
+        # Values are stored as NULL in the database
+        self.assertEqual([(None,) * len(std_fields)],
+                         self.env.db_query("""
+                            SELECT %s FROM ticket WHERE id=%%s
+                            """ % ','.join(std_fields), (ticket.id,)))
+        self.assertEqual([(None,)] * len(cst_fields),
+                         self.env.db_query("""
+                            SELECT value FROM ticket_custom
+                            WHERE ticket=%%s AND name IN (%s)
+                            """ % ','.join(['%s'] * len(cst_fields)),
+                            [ticket.id] + cst_fields))
+        # Values are returned from the model as empty strings
+        for name in ticket.editable_fields:
+            self.assertEqual('', ticket[name], name)
+
+    def test_create_empty_strings_stored_as_null(self):
+        """Ticket fields with empty strings are NULL when creating ticket.
+        """
+        ticket = Ticket(self.env)
+        ticket.populate(dict((name, '') for name in ticket.editable_fields))
+        ticket.insert()
+
+        self._test_empty_strings_stored_as_null(ticket)
+
+    def test_change_empty_strings_stored_as_null(self):
+        """Ticket fields with empty strings are NULL when changing ticket.
+        """
+        ticket = Ticket(self.env)
+        ticket.insert()
+        ticket.populate(dict((name, '') for name in ticket.editable_fields))
+        ticket.save_changes()
+
+        self._test_empty_strings_stored_as_null(ticket)
+
+    def test_whitespace_stripped_from_text_field(self):
+        """Whitespace is stripped from text fields.
+        Test for regression of #11891.
+        """
+        ticket = Ticket(self.env)
+        ticket['keywords'] = 'kw1'
+        ticket['milestone'] = 'milestone1'
+        ticket.insert()
+
+        ticket['keywords'] = '  kw1'
+        ticket['milestone'] = 'milestone2'
+        ticket.save_changes()
+        changes = self.env.db_query("""
+            SELECT oldvalue, newvalue FROM ticket_change
+            """)
+
+        self.assertEqual('kw1', ticket['keywords'])
+        self.assertEqual('milestone2', ticket['milestone'])
+        self.assertEqual(2, len(changes))
+        self.assertIn(('milestone1', 'milestone2'), changes)
+        self.assertIn(('1', None), changes)
+
     def test_ticket_id_is_always_int(self):
         ticket_id = self._insert_ticket('Foo')
         self.assertEqual(ticket_id, int(ticket_id))
@@ -136,6 +257,18 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual(ticket_id, ticket.id)
         self.assertEqual(ticket.resource.id, ticket_id)
 
+    def test_resource_not_found_for_invalid_ticket_id(self):
+        try:
+            Ticket(self.env, '42')
+            self.fail('ResourceNotFound not raised')
+        except ResourceNotFound, e:
+            self.assertEqual(u'Ticket 42 does not exist.', unicode(e))
+        try:
+            Ticket(self.env, 'blah')
+            self.fail('ResourceNotFound not raised')
+        except ResourceNotFound, e:
+            self.assertEqual(u'Ticket blah does not exist.', unicode(e))
+
     def test_can_save_ticket_without_explicit_comment(self):
         ticket = Ticket(self.env)
         ticket.insert()
@@ -156,7 +289,7 @@ class TicketTestCase(unittest.TestCase):
         ticket.save_changes()
 
         for change in ticket.get_changelog():
-            self.assertEqual(None, change[1])
+            self.assertIsNone(change[1])
 
     def test_comment_with_whitespace_only_is_not_saved(self):
         ticket = Ticket(self.env)
@@ -206,6 +339,23 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('two', ticket['bar'])
         self.assertEqual('three', ticket['baz'])
 
+    def test_ticket_custom_field_default_values(self):
+        """Ticket created before a custom field is added will have the
+        default value for the custom field.
+        """
+        tid = self._insert_ticket("The summary")
+        self.env.config.set('ticket-custom', 'baz', 'text')
+        self.env.config.set('ticket-custom', 'baz.value', 'Something')
+        self.env.config.set('ticket-custom', 'bar', 'select')
+        self.env.config.set('ticket-custom', 'bar.options', 'one|two|three')
+        self.env.config.set('ticket-custom', 'bar.value', 'two')
+        TicketSystem(self.env).reset_ticket_fields()
+        del TicketSystem(self.env).custom_fields
+        ticket = Ticket(self.env, tid)
+
+        self.assertEqual('Something', ticket['baz'])
+        self.assertEqual('two', ticket['bar'])
+
     def test_set_field_stripped(self):
         """
         Verify that whitespace around ticket fields is stripped, except for
@@ -226,77 +376,6 @@ class TicketTestCase(unittest.TestCase):
             ticket['component'] = ['  foo  ',  '  bar  ']
         self.assertRaises(TracError, set_multi_valued)
 
-    def test_owner_from_component(self):
-        """
-        Verify that the owner of a new ticket is set to the owner of the
-        component.
-        """
-        component = Component(self.env)
-        component.name = 'test'
-        component.owner = 'joe'
-        component.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test'
-        ticket.insert()
-        self.assertEqual('joe', ticket['owner'])
-
-    def test_owner_from_changed_component(self):
-        """
-        Verify that the owner of a new ticket is updated when the component is
-        changed.
-        """
-        component1 = Component(self.env)
-        component1.name = 'test1'
-        component1.owner = 'joe'
-        component1.insert()
-
-        component2 = Component(self.env)
-        component2.name = 'test2'
-        component2.owner = 'kate'
-        component2.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test1'
-        ticket['status'] = 'new'
-        tktid = ticket.insert()
-
-        ticket = Ticket(self.env, tktid)
-        ticket['component'] = 'test2'
-        ticket.save_changes('jane', 'Testing')
-        self.assertEqual('kate', ticket['owner'])
-
-    def test_no_disown_from_changed_component(self):
-        """
-        Verify that a ticket is not disowned when the component is changed to
-        a non-assigned component.
-        """
-        component1 = Component(self.env)
-        component1.name = 'test1'
-        component1.owner = 'joe'
-        component1.insert()
-
-        component2 = Component(self.env)
-        component2.name = 'test2'
-        component2.owner = ''
-        component2.insert()
-
-        ticket = Ticket(self.env)
-        ticket['reporter'] = 'santa'
-        ticket['summary'] = 'Foo'
-        ticket['component'] = 'test1'
-        ticket['status'] = 'new'
-        tktid = ticket.insert()
-
-        ticket = Ticket(self.env, tktid)
-        ticket['component'] = 'test2'
-        ticket.save_changes('jane', 'Testing')
-        self.assertEqual('joe', ticket['owner'])
-
     def test_populate_ticket(self):
         data = {'summary': 'Hello world', 'reporter': 'john',
                 'foo': 'bar', 'checkbox_cbon': '', 'cbon': 'on',
@@ -309,7 +388,7 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('john', ticket['reporter'])
 
         # An unknown field
-        assert ticket['bar'] is None
+        self.assertIsNone(ticket['bar'])
 
         # Custom field
         self.assertEqual('bar', ticket['foo'])
@@ -318,6 +397,28 @@ class TicketTestCase(unittest.TestCase):
         self.assertEqual('on', ticket['cbon'])
         self.assertEqual('0', ticket['cboff'])
 
+    def test_custom_time(self):
+        # Add a custom field of type 'time'
+        self.env.config.set('ticket-custom', 'due', 'time')
+        ticket = Ticket(self.env)
+        self.assertFalse('due' in ticket.std_fields)
+        self.assertTrue('due' in ticket.time_fields)
+        ticket['reporter'] = 'john'
+        ticket['summary'] = 'Task1'
+        tktid = ticket.insert()
+        ticket = Ticket(self.env, tktid)
+        # Empty string is default value, but not a time stamp
+        self.assertEqual(None, ticket['due'])
+        ts = datetime(2011, 11, 11, 0, 0, 0, 0, utc)
+        ticket['due'] = ts
+        t1 = datetime(2001, 1, 1, 1, 1, 1, 0, utc)
+        ticket.save_changes('joe', when=t1)
+        self.assertEqual(ts, ticket['due'])
+        ticket['due'] = ''
+        t2 = datetime(2001, 1, 1, 1, 1, 2, 0, utc)
+        ticket.save_changes('joe', when=t2)
+        self.assertEqual('', ticket['due'])
+
     def test_changelog(self):
         tkt_id = self._insert_ticket('Test', reporter='joe', component='foo',
                                      milestone='bar')
@@ -598,6 +699,28 @@ class TicketCommentEditTestCase(TicketCommentTestCase):
             self.assertEqual((i, t[i], 'joe (%d)' % i,
                              'Comment 1 (%d)' % i), history[i])
 
+    def test_change_listener_comment_modified(self):
+        listener = TestTicketChangeListener_2(self.env)
+        ticket = Ticket(self.env, self.id)
+        ticket.modify_comment(cdate=self.t2, author='jack',
+                              comment='New Comment 2', when=datetime_now(utc))
+
+        self.assertEqual('comment_modified', listener.action)
+        self.assertEqual(ticket, listener.ticket)
+        self.assertEqual(self.t2, listener.cdate)
+        self.assertEqual('jack', listener.author)
+        self.assertEqual('New Comment 2', listener.comment)
+        self.assertEqual('Comment 2', listener.old_comment)
+
+    def test_get_comment_number(self):
+        ticket = Ticket(self.env, self.id)
+        self.assertEqual(1, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=1)))
+        self.assertEqual(2, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=2)))
+        self.assertEqual(3, ticket.get_comment_number(self.created +
+                                                      timedelta(seconds=3)))
+
 
 class TicketCommentDeleteTestCase(TicketCommentTestCase):
 
@@ -627,12 +750,12 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
         ticket = Ticket(self.env, self.id)
         self.assertEqual('a', ticket['keywords'])
         self.assertEqual('change4', ticket['foo'])
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(cnum=4, when=t)
         self.assertEqual('a, b', ticket['keywords'])
         self.assertEqual('change3', ticket['foo'])
-        self.assertEqual(None, ticket.get_change(cnum=4))
-        self.assertNotEqual(None, ticket.get_change(cnum=3))
+        self.assertIsNone(ticket.get_change(cnum=4))
+        self.assertIsNotNone(ticket.get_change(cnum=3))
         self.assertEqual(t, ticket.time_changed)
 
     def test_delete_last_comment_when_custom_field_gone(self):
@@ -646,29 +769,29 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
         del TicketSystem(self.env).custom_fields
         ticket = Ticket(self.env, self.id)
         #
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(cnum=4, when=t)
         self.assertEqual('a, b', ticket['keywords'])
         # 'foo' is no longer defined for the ticket
-        self.assertEqual(None, ticket['foo'])
+        self.assertIsNone(ticket['foo'])
         # however, 'foo=change3' is still in the database
         self.assertEqual([('change3',)], self.env.db_query("""
             SELECT value FROM ticket_custom WHERE ticket=%s AND name='foo'
             """, (self.id,)))
-        self.assertEqual(None, ticket.get_change(cnum=4))
-        self.assertNotEqual(None, ticket.get_change(cnum=3))
+        self.assertIsNone(ticket.get_change(cnum=4))
+        self.assertIsNotNone(ticket.get_change(cnum=3))
         self.assertEqual(t, ticket.time_changed)
 
     def test_delete_last_comment_by_date(self):
         ticket = Ticket(self.env, self.id)
         self.assertEqual('a', ticket['keywords'])
         self.assertEqual('change4', ticket['foo'])
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(cdate=self.t4, when=t)
         self.assertEqual('a, b', ticket['keywords'])
         self.assertEqual('change3', ticket['foo'])
-        self.assertEqual(None, ticket.get_change(cdate=self.t4))
-        self.assertNotEqual(None, ticket.get_change(cdate=self.t3))
+        self.assertIsNone(ticket.get_change(cdate=self.t4))
+        self.assertIsNotNone(ticket.get_change(cdate=self.t3))
         self.assertEqual(t, ticket.time_changed)
 
     def test_delete_mid_comment(self):
@@ -677,9 +800,9 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
             comment=dict(author='joe', old='4', new='Comment 4'),
             keywords=dict(author='joe', old='a, b', new='a'),
             foo=dict(author='joe', old='change3', new='change4'))
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(cnum=3, when=t)
-        self.assertEqual(None, ticket.get_change(cnum=3))
+        self.assertIsNone(ticket.get_change(cnum=3))
         self.assertEqual('a', ticket['keywords'])
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
@@ -693,9 +816,9 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
             comment=dict(author='joe', old='4', new='Comment 4'),
             keywords=dict(author='joe', old='a, b', new='a'),
             foo=dict(author='joe', old='change3', new='change4'))
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(cdate=self.t3, when=t)
-        self.assertEqual(None, ticket.get_change(cdate=self.t3))
+        self.assertIsNone(ticket.get_change(cdate=self.t3))
         self.assertEqual('a', ticket['keywords'])
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
@@ -717,7 +840,7 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
             keywords=dict(author='joe', old='1, 2', new='a'),
             foo=dict(author='joe', old='change3', new='change4'))
         ticket.delete_change(3)
-        self.assertEqual(None, ticket.get_change(3))
+        self.assertIsNone(ticket.get_change(3))
         self.assertEqual('a', ticket['keywords'])
         self.assertChange(ticket, 4, self.t4, 'joe',
             comment=dict(author='joe', old='4', new='Comment 4'),
@@ -729,10 +852,29 @@ class TicketCommentDeleteTestCase(TicketCommentTestCase):
         ticket.delete_change(4)
         ticket.delete_change(3)
         ticket.delete_change(2)
-        t = datetime.now(utc)
+        t = datetime_now(utc)
         ticket.delete_change(1, when=t)
         self.assertEqual(t, ticket.time_changed)
 
+    def test_ticket_change_deleted(self):
+        listener = TestTicketChangeListener_2(self.env)
+        ticket = Ticket(self.env, self.id)
+
+        ticket.delete_change(cdate=self.t3, when=datetime_now(utc))
+        self.assertEqual('change_deleted', listener.action)
+        self.assertEqual(ticket, listener.ticket)
+        self.assertEqual(self.t3, listener.cdate)
+        self.assertEqual(dict(keywords=('a, b, c', 'a, b'),
+                              foo=('change2', 'change3')),
+                         listener.changes)
+
+        ticket.delete_change(cnum=2, when=datetime_now(utc))
+        self.assertEqual('change_deleted', listener.action)
+        self.assertEqual(ticket, listener.ticket)
+        self.assertEqual(self.t2, listener.cdate)
+        self.assertEqual(dict(owner=('john', 'jack'),
+                              foo=('change 1', 'change2')),
+                         listener.changes)
 
 class EnumTestCase(unittest.TestCase):
 
@@ -742,6 +884,11 @@ class EnumTestCase(unittest.TestCase):
     def tearDown(self):
         self.env.reset_db()
 
+    def test_repr(self):
+        self.assertEqual("<Priority None None>", repr(Priority(self.env)))
+        self.assertEqual("<Priority 'major' u'3'>",
+                         repr(Priority(self.env, 'major')))
+
     def test_priority_fetch(self):
         prio = Priority(self.env, 'major')
         self.assertEqual(prio.name, 'major')
@@ -751,14 +898,14 @@ class EnumTestCase(unittest.TestCase):
         prio = Priority(self.env)
         prio.name = 'foo'
         prio.insert()
-        self.assertEqual(True, prio.exists)
+        self.assertTrue(prio.exists)
 
     def test_priority_insert_with_value(self):
         prio = Priority(self.env)
         prio.name = 'bar'
         prio.value = 100
         prio.insert()
-        self.assertEqual(True, prio.exists)
+        self.assertTrue(prio.exists)
 
     def test_priority_update(self):
         prio = Priority(self.env, 'major')
@@ -771,7 +918,7 @@ class EnumTestCase(unittest.TestCase):
         prio = Priority(self.env, 'major')
         self.assertEqual('3', prio.value)
         prio.delete()
-        self.assertEqual(False, prio.exists)
+        self.assertFalse(prio.exists)
         self.assertRaises(TracError, Priority, self.env, 'major')
         prio = Priority(self.env, 'minor')
         self.assertEqual('3', prio.value)
@@ -806,12 +953,12 @@ class MilestoneTestCase(unittest.TestCase):
 
     def setUp(self):
         self.env = EnvironmentStub(default_data=True)
-        self.env.path = os.path.join(tempfile.gettempdir(), 'trac-tempenv')
-        os.mkdir(self.env.path)
+        self.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
+        self.created_at = datetime(2001, 1, 1, tzinfo=utc)
+        self.updated_at = self.created_at + timedelta(seconds=1)
 
     def tearDown(self):
-        shutil.rmtree(self.env.path)
-        self.env.reset_db()
+        self.env.reset_db_and_disk()
 
     def _create_milestone(self, **values):
         milestone = Milestone(self.env)
@@ -819,13 +966,27 @@ class MilestoneTestCase(unittest.TestCase):
             setattr(milestone, k, v)
         return milestone
 
+    def _insert_ticket(self, when=None, **kwargs):
+        ticket = Ticket(self.env)
+        for name, value in kwargs.iteritems():
+            ticket[name] = value
+        ticket.insert(when or self.created_at)
+        return ticket
+
+    def _update_ticket(self, ticket, author=None, comment=None, when=None,
+                       **kwargs):
+        for name, value in kwargs.iteritems():
+            ticket[name] = value
+        ticket.save_changes(author, comment, when or self.updated_at)
+
     def test_new_milestone(self):
         milestone = Milestone(self.env)
-        self.assertEqual(False, milestone.exists)
-        self.assertEqual(None, milestone.name)
-        self.assertEqual(None, milestone.due)
-        self.assertEqual(None, milestone.completed)
+        self.assertFalse(milestone.exists)
+        self.assertIsNone(milestone.name)
+        self.assertIsNone(milestone.due)
+        self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone None>", repr(milestone))
 
     def test_new_milestone_empty_name(self):
         """
@@ -833,21 +994,23 @@ class MilestoneTestCase(unittest.TestCase):
         milestone being correctly detected as non-existent.
         """
         milestone = Milestone(self.env, '')
-        self.assertEqual(False, milestone.exists)
-        self.assertEqual(None, milestone.name)
-        self.assertEqual(None, milestone.due)
-        self.assertEqual(None, milestone.completed)
+        self.assertFalse(milestone.exists)
+        self.assertIsNone(milestone.name)
+        self.assertIsNone(milestone.due)
+        self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone None>", repr(milestone))
 
     def test_existing_milestone(self):
         self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
 
         milestone = Milestone(self.env, 'Test')
-        self.assertEqual(True, milestone.exists)
+        self.assertTrue(milestone.exists)
         self.assertEqual('Test', milestone.name)
-        self.assertEqual(None, milestone.due)
-        self.assertEqual(None, milestone.completed)
+        self.assertIsNone(milestone.due)
+        self.assertIsNone(milestone.completed)
         self.assertEqual('', milestone.description)
+        self.assertEqual("<Milestone u'Test'>", repr(milestone))
 
     def test_create_and_update_milestone(self):
         milestone = Milestone(self.env)
@@ -867,35 +1030,119 @@ class MilestoneTestCase(unittest.TestCase):
             WHERE name='Test'
             """))
 
+    def test_move_tickets(self):
+        self.env.db_transaction.executemany(
+            "INSERT INTO milestone (name) VALUES (%s)",
+            [('Test',), ('Testing',)])
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        self._update_ticket(tkt2, status='closed', resolution='fixed')
+        milestone = Milestone(self.env, 'Test')
+        milestone.move_tickets('Testing', 'anonymous', 'Move tickets')
+
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('Testing', tkt1['milestone'])
+        self.assertEqual('Testing', tkt2['milestone'])
+        self.assertEqual(tkt1['changetime'], tkt2['changetime'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
+
+    def test_move_tickets_exclude_closed(self):
+        self.env.db_transaction.executemany(
+            "INSERT INTO milestone (name) VALUES (%s)",
+            [('Test',), ('Testing',)])
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        self._update_ticket(tkt2, status='closed', resolution='fixed')
+        milestone = Milestone(self.env, 'Test')
+        milestone.move_tickets('Testing', 'anonymous', 'Move tickets',
+                               exclude_closed=True)
+
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('Testing', tkt1['milestone'])
+        self.assertEqual('Test', tkt2['milestone'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
+        self.assertEqual(self.updated_at, tkt2['changetime'])
+
+    def test_move_tickets_target_doesnt_exist(self):
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        milestone = Milestone(self.env, 'Test')
+        self.assertRaises(ResourceNotFound, milestone.move_tickets,
+                          'Testing', 'anonymous')
+
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('Test', tkt1['milestone'])
+        self.assertEqual('Test', tkt2['milestone'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
+        self.assertNotEqual(self.updated_at, tkt2['changetime'])
+
     def test_create_milestone_without_name(self):
         milestone = Milestone(self.env)
         self.assertRaises(TracError, milestone.insert)
 
     def test_delete_milestone(self):
         self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
-
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        self._update_ticket(tkt2, status='closed', resolution='fixed')
         milestone = Milestone(self.env, 'Test')
         milestone.delete()
-        self.assertEqual(False, milestone.exists)
+        self.assertFalse(milestone.exists)
         self.assertEqual([],
             self.env.db_query("SELECT * FROM milestone WHERE name='Test'"))
 
-    def test_delete_milestone_retarget_tickets(self):
-        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('', tkt1['milestone'])
+        self.assertEqual('', tkt2['milestone'])
+        self.assertEqual(tkt1['changetime'], tkt2['changetime'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
 
-        tkt1 = Ticket(self.env)
-        tkt1.populate({'summary': 'Foo', 'milestone': 'Test'})
-        tkt1.insert()
-        tkt2 = Ticket(self.env)
-        tkt2.populate({'summary': 'Bar', 'milestone': 'Test'})
-        tkt2.insert()
+    def test_delete_milestone_with_attachment(self):
+        milestone = Milestone(self.env)
+        milestone.name = 'MilestoneWithAttachment'
+        milestone.insert()
 
-        milestone = Milestone(self.env, 'Test')
-        milestone.delete(retarget_to='Other')
+        attachment = Attachment(self.env, 'milestone', milestone.name)
+        attachment.insert('foo.txt', StringIO(), 0, 1)
+
+        milestone.delete()
         self.assertEqual(False, milestone.exists)
 
-        self.assertEqual('Other', Ticket(self.env, tkt1.id)['milestone'])
-        self.assertEqual('Other', Ticket(self.env, tkt2.id)['milestone'])
+        attachments = Attachment.select(self.env, 'milestone', milestone.name)
+        self.assertRaises(StopIteration, attachments.next)
+
+    def test_delete_milestone_retarget_tickets(self):
+        self.env.db_transaction.executemany(
+            "INSERT INTO milestone (name) VALUES (%s)",
+            [('Test',), ('Other',)])
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        self._update_ticket(tkt2, status='closed', resolution='fixed')
+        milestone = Milestone(self.env, 'Test')
+        milestone.delete(retarget_to='Other')
+        self.assertFalse(milestone.exists)
+
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('Other', tkt1['milestone'])
+        self.assertEqual('Other', tkt2['milestone'])
+        self.assertEqual(tkt1['changetime'], tkt2['changetime'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
 
     def test_update_milestone(self):
         self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
@@ -919,23 +1166,6 @@ class MilestoneTestCase(unittest.TestCase):
         milestone.name = None
         self.assertRaises(TracError, milestone.update)
 
-    def test_update_milestone_update_tickets(self):
-        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
-
-        tkt1 = Ticket(self.env)
-        tkt1.populate({'summary': 'Foo', 'milestone': 'Test'})
-        tkt1.insert()
-        tkt2 = Ticket(self.env)
-        tkt2.populate({'summary': 'Bar', 'milestone': 'Test'})
-        tkt2.insert()
-
-        milestone = Milestone(self.env, 'Test')
-        milestone.name = 'Testing'
-        milestone.update()
-
-        self.assertEqual('Testing', Ticket(self.env, tkt1.id)['milestone'])
-        self.assertEqual('Testing', Ticket(self.env, tkt2.id)['milestone'])
-
     def test_rename_milestone(self):
         milestone = Milestone(self.env)
         milestone.name = 'OldName'
@@ -957,6 +1187,24 @@ class MilestoneTestCase(unittest.TestCase):
         self.assertEqual('foo.txt', attachments.next().filename)
         self.assertRaises(StopIteration, attachments.next)
 
+    def test_rename_milestone_retarget_tickets(self):
+        self.env.db_transaction("INSERT INTO milestone (name) VALUES ('Test')")
+        tkt1 = self._insert_ticket(status='new', summary='Foo',
+                                   milestone='Test')
+        tkt2 = self._insert_ticket(status='new', summary='Bar',
+                                   milestone='Test')
+        self._update_ticket(tkt2, status='closed', resolution='fixed')
+        milestone = Milestone(self.env, 'Test')
+        milestone.name = 'Testing'
+        milestone.update()
+
+        tkt1 = Ticket(self.env, tkt1.id)
+        tkt2 = Ticket(self.env, tkt2.id)
+        self.assertEqual('Testing', tkt1['milestone'])
+        self.assertEqual('Testing', tkt2['milestone'])
+        self.assertEqual(tkt1['changetime'], tkt2['changetime'])
+        self.assertNotEqual(self.updated_at, tkt1['changetime'])
+
     def test_select_milestones(self):
         self.env.db_transaction.executemany(
             "INSERT INTO milestone (name) VALUES (%s)",
@@ -964,9 +1212,9 @@ class MilestoneTestCase(unittest.TestCase):
 
         milestones = list(Milestone.select(self.env))
         self.assertEqual('1.0', milestones[0].name)
-        assert milestones[0].exists
+        self.assertTrue(milestones[0].exists)
         self.assertEqual('2.0', milestones[1].name)
-        assert milestones[1].exists
+        self.assertTrue(milestones[1].exists)
 
     def test_change_listener_created(self):
         listener = TestMilestoneChangeListener(self.env)
@@ -998,10 +1246,10 @@ class MilestoneTestCase(unittest.TestCase):
         listener = TestMilestoneChangeListener(self.env)
         milestone = self._create_milestone(name='Milestone 1')
         milestone.insert()
-        self.assertEqual(True, milestone.exists)
+        self.assertTrue(milestone.exists)
         milestone.delete()
         self.assertEqual('Milestone 1', milestone.name)
-        self.assertEqual(False, milestone.exists)
+        self.assertFalse(milestone.exists)
         self.assertEqual('deleted', listener.action)
         self.assertEqual(milestone, listener.milestone)
 
@@ -1026,6 +1274,11 @@ class ComponentTestCase(unittest.TestCase):
         for c in Component.select(self.env):
             self.assertEqual(c.exists, True)
 
+    def test_repr(self):
+        self.assertEqual('<Component None>', repr(Component(self.env)))
+        self.assertEqual("<Component 'component1'>",
+                         repr(Component(self.env, 'component1')))
+
     def test_create_and_update(self):
         component = Component(self.env)
         component.name = 'Test'
@@ -1061,6 +1314,10 @@ class VersionTestCase(unittest.TestCase):
         for v in Version.select(self.env):
             self.assertEqual(v.exists, True)
 
+    def test_repr(self):
+        self.assertEqual('<Version None>', repr(Version(self.env)))
+        self.assertEqual("<Version '1.0'>", repr(Version(self.env, '1.0')))
+
     def test_create_and_update(self):
         version = Version(self.env)
         version.name = 'Test'
@@ -1076,16 +1333,16 @@ class VersionTestCase(unittest.TestCase):
             "SELECT name, time, description FROM version WHERE name='Test'"))
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(TicketTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(TicketCommentEditTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(TicketCommentDeleteTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(EnumTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(MilestoneTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(ComponentTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(VersionTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(TicketTestCase))
+    suite.addTest(unittest.makeSuite(TicketCommentEditTestCase))
+    suite.addTest(unittest.makeSuite(TicketCommentDeleteTestCase))
+    suite.addTest(unittest.makeSuite(EnumTestCase))
+    suite.addTest(unittest.makeSuite(MilestoneTestCase))
+    suite.addTest(unittest.makeSuite(ComponentTestCase))
+    suite.addTest(unittest.makeSuite(VersionTestCase))
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
