@@ -1,5 +1,22 @@
-#!/usr/bin/python
-from trac.tests.functional import *
+#!/usr/bin/env python
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2008-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
+
+import unittest
+
+from trac.tests.contentgen import random_page, random_sentence, \
+                                  random_unique_camel
+from trac.tests.functional import FunctionalTwillTestCaseSetup, tc
 
 
 class RegressionTestRev5883(FunctionalTwillTestCaseSetup):
@@ -29,11 +46,14 @@ class RegressionTestRev5883(FunctionalTwillTestCaseSetup):
 
 def functionalSuite(suite=None):
     if not suite:
-        import trac.tests.functional.testcases
-        suite = trac.tests.functional.testcases.functionalSuite()
+        import trac.tests.functional
+        suite = trac.tests.functional.functionalSuite()
     suite.addTest(RegressionTestRev5883())
     return suite
 
 
+test_suite = functionalSuite
+
+
 if __name__ == '__main__':
-    unittest.main(defaultTest='functionalSuite')
+    unittest.main(defaultTest='test_suite')
