@@ -1,7 +1,21 @@
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2007-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
+
 
 def do_upgrade(env, ver, cursor):
     """Upgrade the reports to better handle the new workflow capabilities"""
-    owner = env.get_read_db().concat('owner', "' *'")
+    with env.db_query as db:
+        owner = db.concat('owner', "' *'")
     cursor.execute('SELECT id, query, description FROM report')
     reports = cursor.fetchall()
     for report, query, description in reports:
