@@ -50,7 +50,7 @@ def _reloader_thread(modification_callback, loop_callback):
             if filename not in mtimes:
                 mtimes[filename] = mtime
                 continue
-            if mtime > mtimes[filename]:
+            if mtime != mtimes[filename]:
                 modification_callback(filename)
                 sys.exit(3)
         loop_callback()
@@ -79,7 +79,7 @@ def main(func, modification_callback, *args, **kwargs):
             try:
                 func(*args, **kwargs)
                 exit_code.append(None)
-            except SystemExit, e:
+            except SystemExit as e:
                 exit_code.append(e.code)
             except:
                 traceback.print_exception(*sys.exc_info())
