@@ -434,9 +434,9 @@ def get_date_format_jquery_ui(locale):
 def get_time_format_jquery_ui(locale):
     """Get the time format for the jQuery UI timepicker addon."""
     if locale == 'iso8601':
-        return 'hh:mm:ssz'  # XXX timepicker doesn't support 'ISO_8601'
+        return 'HH:mm:ssZ'
     if babel and locale:
-        values = {'h': 'h', 'hh': 'hh', 'H': 'h', 'HH': 'hh',
+        values = {'h': 'h', 'hh': 'hh', 'H': 'H', 'HH': 'HH',
                   'm': 'm', 'mm': 'mm', 's': 's', 'ss': 'ss'}
         f = get_time_format('medium', locale=locale).format
         if '%(a)s' in f:
@@ -450,17 +450,21 @@ def get_time_format_jquery_ui(locale):
     ampm = format_time(t, '%p', tzinfo=utc)
     if ampm:
         tmpl = tmpl.replace(ampm, 'TT' if ampm[0].isupper() else 'tt', 1)
-    return tmpl.replace('23', 'hh', 1).replace('11', 'hh', 1) \
+    return tmpl.replace('23', 'HH', 1).replace('11', 'hh', 1) \
                .replace('59', 'mm', 1).replace('58', 'ss', 1)
 
 def get_timezone_list_jquery_ui(t=None):
     """Get timezone list for jQuery timepicker addon"""
+    def utcoffset(tz, t):  # in minutes
+        offset = t.astimezone(get_timezone(tz)).utcoffset()
+        return offset.days * 24 * 60 + offset.seconds // 60
+    def label(offset):
+        sign = '-' if offset < 0 else '+'
+        return '%s%02d:%02d' % (sign, abs(offset // 60), offset % 60)
     t = to_datetime(t, utc)
-    zones = set(t.astimezone(get_timezone(tz)).strftime('%z')
-                for tz in all_timezones)
-    return [{'value': 'Z', 'label': '+00:00'} \
-            if zone == '+0000' else zone[:-2] + ':' + zone[-2:]
-            for zone in sorted(zones, key=lambda tz: int(tz))]
+    offsets = set(utcoffset(tz, t) for tz in all_timezones)
+    return [{'value': offset, 'label': label(offset)}
+            for offset in sorted(offsets)]
 
 def get_first_week_day_jquery_ui(req):
     """Get first week day for jQuery date picker"""
@@ -612,6 +616,7 @@ def parse_date(text, tzinfo=None, locale=None, hint='date'):
         formatted_hint = {
             'datetime': get_datetime_format_hint,
             'date': get_date_format_hint,
+            'relative': get_datetime_format_hint,
             'iso8601': lambda l: get_datetime_format_hint('iso8601'),
         }.get(hint, lambda(l): hint)(locale)
         if locale == 'iso8601' and hint in ('date', 'datetime') or \
@@ -793,10 +798,12 @@ def _i18n_parse_date_0(text, order, regexp, period_names, month_names, tzinfo):
     t = tzinfo.localize(datetime(*(values[k] for k in 'yMdhms')))
     return tzinfo.normalize(t)
 
-_REL_TIME_RE = re.compile(
-    r'(\d+\.?\d*)\s*'
-    r'(second|minute|hour|day|week|month|year|[hdwmy])s?\s*'
-    r'(?:ago)?$')
+_REL_FUTURE_RE = re.compile(
+    r'(?:in|\+)\s*(\d+\.?\d*)\s*'
+    r'(second|minute|hour|day|week|month|year|[hdwmy])s?$')
+_REL_PAST_RE = re.compile(
+    r'(?:-\s*)?(\d+\.?\d*)\s*'
+    r'(second|minute|hour|day|week|month|year|[hdwmy])s?\s*(?:ago)?$')
 _time_intervals = dict(
     second=lambda v: timedelta(seconds=v),
     minute=lambda v: timedelta(minutes=v),
@@ -811,7 +818,7 @@ _time_intervals = dict(
     m=lambda v: timedelta(days=30 * v),
     y=lambda v: timedelta(days=365 * v),
 )
-_TIME_START_RE = re.compile(r'(this|last)\s*'
+_TIME_START_RE = re.compile(r'(this|last|next)\s*'
                             r'(second|minute|hour|day|week|month|year)$')
 _time_starts = dict(
     second=lambda now: datetime(now.year, now.month, now.day, now.hour,
@@ -837,8 +844,15 @@ def _parse_relative_time(text, tzinfo, now=None):
         dt = _time_starts['day'](now)
     elif text == 'yesterday':
         dt = _time_starts['day'](now) - timedelta(days=1)
+    elif text == 'tomorrow':
+        dt = _time_starts['day'](now) + timedelta(days=1)
     if dt is None:
-        match = _REL_TIME_RE.match(text)
+        match = _REL_FUTURE_RE.match(text)
+        if match:
+            (value, interval) = match.groups()
+            dt = now + _time_intervals[interval](float(value))
+    if dt is None:
+        match = _REL_PAST_RE.match(text)
         if match:
             (value, interval) = match.groups()
             dt = now - _time_intervals[interval](float(value))
@@ -857,6 +871,16 @@ def _parse_relative_time(text, tzinfo, now=None):
                     dt = dt.replace(year=dt.year - 1)
                 else:
                     dt -= _time_intervals[start](1)
+            elif which == 'next':
+                if start == 'month':
+                    if dt.month < 12:
+                        dt = dt.replace(month=dt.month + 1)
+                    else:
+                        dt = dt.replace(year=dt.year + 1, month=1)
+                elif start == 'year':
+                    dt = dt.replace(year=dt.year + 1)
+                else:
+                    dt += _time_intervals[start](1)
 
     if dt is None:
         return None
@@ -864,6 +888,7 @@ def _parse_relative_time(text, tzinfo, now=None):
         dt = tzinfo.localize(dt)
     return tzinfo.normalize(dt)
 
+
 # -- formatting/parsing helper functions
 
 def user_time(req, func, *args, **kwargs):
@@ -883,6 +908,12 @@ def user_time(req, func, *args, **kwargs):
         kwargs['locale'] = getattr(req, 'lc_time', None)
     return func(*args, **kwargs)
 
+def format_date_or_datetime(format, *args, **kwargs):
+    if format == 'date':
+        return format_date(*args, **kwargs)
+    else:
+        return format_datetime(*args, **kwargs)
+
 # -- timezone utilities
 
 class FixedOffset(tzinfo):
@@ -991,11 +1022,11 @@ class LocalTimezone(tzinfo):
             if not std_correct:
                 raise ValueError('Non existent time "%s"' % dt)
         tt = None
-        if std_correct is dst_correct is True:
+        if std_correct and dst_correct:
             tt = local_tt[bool(is_dst)]
-        elif std_correct is True:
+        elif std_correct:
             tt = local_tt[0]
-        elif dst_correct is True:
+        elif dst_correct:
             tt = local_tt[1]
         if tt:
             utc_ts = to_timestamp(datetime(tzinfo=utc, *tt[:6]))
