@@ -20,10 +20,10 @@ import trac.tests.compat
 from trac.util.text import (
     _get_default_ambiwidth, empty, expandtabs, fix_eol, javascript_quote,
     levenshtein_distance, normalize_whitespace, print_table,
-    quote_query_string, shorten_line, strip_line_ws, stripws, text_width,
-    to_js_string, to_unicode, unicode_from_base64, unicode_quote,
-    unicode_quote_plus, unicode_to_base64, unicode_unquote, unicode_urlencode,
-    wrap)
+    quote_query_string, shorten_line, strip_line_ws, stripws, sub_vars,
+    text_width, to_js_string, to_unicode, to_utf8, unicode_from_base64,
+    unicode_quote, unicode_quote_plus, unicode_to_base64, unicode_unquote,
+    unicode_urlencode, wrap)
 
 
 class ToUnicodeTestCase(unittest.TestCase):
@@ -46,20 +46,20 @@ class ToUnicodeTestCase(unittest.TestCase):
         u = u'\uB144'
         try:
             raise ValueError, '%s is not a number.' % u
-        except ValueError, e:
+        except ValueError as e:
             self.assertEqual(u'\uB144 is not a number.', to_unicode(e))
 
     def test_from_exception_using_str_args(self):
         u = u'Das Ger\xe4t oder die Ressource ist belegt'
         try:
             raise ValueError, u.encode('utf-8')
-        except ValueError, e:
+        except ValueError as e:
             self.assertEqual(u, to_unicode(e))
 
     def test_from_windows_error(self):
         try:
             os.stat('non/existent/file.txt')
-        except OSError, e:
+        except OSError as e:
             uc = to_unicode(e)
             self.assertIsInstance(uc, unicode, uc)
             self.assertTrue(uc.startswith('[Error '), uc)
@@ -72,7 +72,7 @@ class ToUnicodeTestCase(unittest.TestCase):
             s = socket.socket(af, socktype, proto)
             try:
                 s.connect(sa)
-            except socket.error, e:
+            except socket.error as e:
                 uc = to_unicode(e)
                 self.assertIsInstance(uc, unicode, uc)
                 if hasattr(e, 'strerror'):
@@ -160,6 +160,31 @@ class QuoteQueryStringTestCase(unittest.TestCase):
                          quote_query_string(text))
 
 
+class ToUtf8TestCase(unittest.TestCase):
+    def test_unicode(self):
+        self.assertEqual('à', to_utf8('à'))
+        self.assertEqual('ç', to_utf8('ç'))
+
+    def test_boolean(self):
+        self.assertEqual('True', to_utf8(True))
+        self.assertEqual('False', to_utf8(False))
+
+    def test_int(self):
+        self.assertEqual('-1', to_utf8(-1))
+        self.assertEqual('0', to_utf8(0))
+        self.assertEqual('1', to_utf8(1))
+
+    def test_utf8(self):
+        self.assertEqual('à', to_utf8(u'à'))
+        self.assertEqual('ç', to_utf8(u'ç'))
+
+    def test_exception_with_utf8_message(self):
+        self.assertEqual('thė mèssägē', to_utf8(Exception('thė mèssägē')))
+
+    def test_exception_with_unicode_message(self):
+        self.assertEqual('thė mèssägē', to_utf8(Exception(u'thė mèssägē')))
+
+
 class WhitespaceTestCase(unittest.TestCase):
     def test_default(self):
         self.assertEqual(u'This is text ',
@@ -388,7 +413,6 @@ class StripwsTestCase(unittest.TestCase):
                                  leading=False, trailing=False))
 
 
-
 class LevenshteinDistanceTestCase(unittest.TestCase):
     def test_distance(self):
         self.assertEqual(5, levenshtein_distance('kitten', 'sitting'))
@@ -398,6 +422,14 @@ class LevenshteinDistanceTestCase(unittest.TestCase):
         self.assertEqual(0, levenshtein_distance('milestone', 'milestone'))
 
 
+class SubVarsTestCase(unittest.TestCase):
+    def test_sub_vars(self):
+        subtext = sub_vars("$USER's tickets for '$COMPONENT', $MILESTONE",
+                           {'USER': 'user1', 'COMPONENT': 'component1'})
+        self.assertEqual("user1's tickets for 'component1', $MILESTONE",
+                         subtext)
+
+
 class ShortenLineTestCase(unittest.TestCase):
 
     def test_less_than_maxlen(self):
@@ -480,7 +512,7 @@ class DefaultAmbiwidthTestCase(unittest.TestCase):
         self.assertEqual(1, _get_default_ambiwidth())
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(ToUnicodeTestCase))
     suite.addTest(unittest.makeSuite(ExpandtabsTestCase))
@@ -488,6 +520,7 @@ def suite():
     suite.addTest(unittest.makeSuite(JavascriptQuoteTestCase))
     suite.addTest(unittest.makeSuite(ToJsStringTestCase))
     suite.addTest(unittest.makeSuite(QuoteQueryStringTestCase))
+    suite.addTest(unittest.makeSuite(ToUtf8TestCase))
     suite.addTest(unittest.makeSuite(WhitespaceTestCase))
     suite.addTest(unittest.makeSuite(TextWidthTestCase))
     suite.addTest(unittest.makeSuite(PrintTableTestCase))
@@ -496,6 +529,7 @@ def suite():
     suite.addTest(unittest.makeSuite(UnicodeBase64TestCase))
     suite.addTest(unittest.makeSuite(StripwsTestCase))
     suite.addTest(unittest.makeSuite(LevenshteinDistanceTestCase))
+    suite.addTest(unittest.makeSuite(SubVarsTestCase))
     suite.addTest(unittest.makeSuite(ShortenLineTestCase))
     if os.name != 'nt':
         suite.addTest(unittest.makeSuite(DefaultAmbiwidthTestCase))
@@ -503,4 +537,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
