@@ -43,7 +43,7 @@ def join(*args):
     return '/'.join(arg for arg in args if arg)
 
 
-class ParseError(Exception):
+class ParseError(TracBaseError):
     """Exception thrown for parse errors in authz files"""
 
 
@@ -121,14 +121,16 @@ class AuthzSourcePolicy(Component):
 
     implements(IPermissionPolicy)
 
-    authz_file = PathOption('trac', 'authz_file', '',
+    authz_file = PathOption('svn', 'authz_file', '',
         """The path to the Subversion
-        [http://svnbook.red-bean.com/en/1.5/svn.serverconfig.pathbasedauthz.html authorization (authz) file].
+        [%(svnbook)s authorization (authz) file].
         To enable authz permission checking, the `AuthzSourcePolicy` permission
-        policy must be added to `[trac] permission_policies`.
-        """)
+        policy must be added to `[trac] permission_policies`. Non-absolute
+        paths are relative to the Environment `conf` directory.
+        """,
+        doc_args={'svnbook': 'http://svnbook.red-bean.com/en/1.7/svn.serverconfig.pathbasedauthz.html'})
 
-    authz_module_name = Option('trac', 'authz_module_name', '',
+    authz_module_name = Option('svn', 'authz_module_name', '',
         """The module prefix used in the `authz_file` for the default
         repository. If left empty, the global section is used.
         """)
@@ -208,14 +210,14 @@ class AuthzSourcePolicy(Component):
     def _get_authz_info(self):
         try:
             mtime = os.path.getmtime(self.authz_file)
-        except OSError, e:
+        except OSError as e:
             if self._authz is not None:
                 self.log.error('Error accessing authz file: %s',
                                exception_to_unicode(e))
             self._mtime = mtime = 0
             self._authz = None
             self._users = set()
-        if mtime > self._mtime:
+        if mtime != self._mtime:
             self._mtime = mtime
             rm = RepositoryManager(self.env)
             modules = set(repos.reponame
@@ -223,14 +225,14 @@ class AuthzSourcePolicy(Component):
             if '' in modules and self.authz_module_name:
                 modules.add(self.authz_module_name)
             modules.add('')
-            self.log.info('Parsing authz file: %s' % self.authz_file)
+            self.log.info('Parsing authz file: %s', self.authz_file)
             try:
                 self._authz = parse(read_file(self.authz_file), modules)
                 self._users = set(user for paths in self._authz.itervalues()
                                   for path in paths.itervalues()
                                   for user, result in path.iteritems()
                                   if result)
-            except Exception, e:
+            except Exception as e:
                 self._authz = None
                 self._users = set()
                 self.log.error('Error parsing authz file: %s',
