@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2010 Edgewall Software
+# Copyright (C) 2005-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -198,7 +198,7 @@ joe = r
 jane = r
 """)
         self.env = EnvironmentStub(enable=[AuthzSourcePolicy])
-        self.env.config.set('trac', 'authz_file', self.authz)
+        self.env.config.set('svn', 'authz_file', self.authz)
         self.policy = AuthzSourcePolicy(self.env)
 
         # Monkey-subclass RepositoryManager to serve mock repositories
@@ -388,14 +388,12 @@ unknown = r
         self.assertRevPerm(True, 'user', 'scoped', 789)
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(unittest.makeSuite(AuthzParserTestCase, 'test'))
-    suite.addTest(unittest.makeSuite(AuthzSourcePolicyTestCase, 'test'))
+    suite.addTest(unittest.makeSuite(AuthzParserTestCase))
+    suite.addTest(unittest.makeSuite(AuthzSourcePolicyTestCase))
     return suite
 
 
 if __name__ == '__main__':
-    runner = unittest.TextTestRunner()
-    runner.run(suite())
-
+    unittest.main(defaultTest='test_suite')
