@@ -15,7 +15,7 @@ from datetime import datetime, timedelta
 import unittest
 
 from trac.core import Component, TracError, implements
-from trac.perm import IPermissionPolicy, PermissionCache
+from trac.perm import IPermissionPolicy
 from trac.resource import Resource
 from trac.test import MockRequest
 from trac.util.datefmt import utc
@@ -81,9 +81,9 @@ class MockRepository(Repository):
 
     def get_changeset(self, rev):
         assert rev % 3 == 1  # allow only 3n + 1
-        return Changeset(self, rev, 'message-%d' % rev, 'author-%d' % rev,
-                         datetime(2001, 1, 1, tzinfo=utc) +
-                         timedelta(seconds=rev))
+        return MockChangeset(self, rev, 'message-%d' % rev, 'author-%d' % rev,
+                             datetime(2001, 1, 1, tzinfo=utc) +
+                             timedelta(seconds=rev))
 
     def previous_rev(self, rev, path=''):
         assert rev % 3 == 1  # allow only 3n + 1
@@ -104,6 +104,22 @@ class MockRepository(Repository):
     def rev_older_than(self, rev1, rev2):
         return self.normalize_rev(rev1) < self.normalize_rev(rev2)
 
+    def close(self):
+        pass
+
+    def _not_implemented(self, *args, **kwargs):
+        raise NotImplementedError
+
+    get_changes = _not_implemented
+    get_oldest_rev = _not_implemented
+    next_rev = _not_implemented
+
+
+class MockChangeset(Changeset):
+
+    def get_changes(self):
+        raise StopIteration
+
 
 class MockNode(Node):
 
@@ -134,6 +150,17 @@ class MockNode(Node):
                 path += '-old'
             rev -= 3
 
+    def _not_implemented(self, *args, **kwargs):
+        raise NotImplementedError
+
+    get_annotations = _not_implemented
+    get_content = _not_implemented
+    get_content_length = _not_implemented
+    get_content_type = _not_implemented
+    get_entries = _not_implemented
+    get_last_modified = _not_implemented
+    get_properties = _not_implemented
+
 
 class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
 
@@ -235,7 +262,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             req = MockRequest(self.env, path_info='/log/mock/file', **kwargs)
             try:
                 self.process_request(req)
-            except NoSuchChangeset, e:
+            except NoSuchChangeset as e:
                 self.assertEqual(message, unicode(e))
 
         fn('No changeset 101 in the repository', args={'rev': '101'})
@@ -422,7 +449,7 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
             req = MockRequest(self.env, path_info='/log/mock/file', **kwargs)
             try:
                 self.process_request(req)
-            except NoSuchChangeset, e:
+            except NoSuchChangeset as e:
                 self.assertEqual(message, unicode(e))
 
         fn('No changeset 101 in the repository', args={'revs': '101'})
@@ -483,11 +510,11 @@ class LogModuleTestCase(RequestHandlerPermissionsTestCaseBase):
         self.assertIn(' href="/trac.cgi/log/mock/?revs=42-43"', rendered)
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(LogModuleTestCase))
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
