@@ -1,28 +1,45 @@
-# -*- encoding: utf-8 -*-
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2006-2014 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.com/license.html.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/.
 
 import unittest
 
 from trac.test import Mock
-from trac.versioncontrol import NoSuchChangeset, NoSuchNode
 from trac.versioncontrol.api import *
 from trac.versioncontrol.web_ui import *
 from trac.wiki.tests import formatter
 
 
+YOUNGEST_REV = 200
+
+
 def _get_changeset(rev):
     if rev == '1':
         return Mock(message="start", is_viewable=lambda perm: True)
     else:
         raise NoSuchChangeset(rev)
 
+
 def _normalize_rev(rev):
+    if rev is None or rev in ('', 'head'):
+        return YOUNGEST_REV
     try:
-        return int(rev)
+        nrev = int(rev)
+        if nrev <= YOUNGEST_REV:
+            return nrev
     except ValueError:
-        if rev == 'head':
-            return '200'
-        else:
-            raise NoSuchChangeset(rev)
+        pass
+    raise NoSuchChangeset(rev)
+
 
 def _get_node(path, rev=None):
     if path == 'foo':
@@ -34,15 +51,75 @@ def _get_node(path, rev=None):
         return Mock(path=path, rev=rev, isfile=True,
                     is_viewable=lambda resource: True)
 
+
+class GitRepositoryStub(object):
+
+    has_linear_changesets = False
+
+    _revs = [
+        ('ffffffffffffffffffffffffffffffffffffffff', ('HEAD',)),
+        ('deadbef222222222222222222222222222222222', ('1.0-stable',
+                                                      u'1.0-stáblé')),
+        ('deadbef111111111111111111111111111111111', ('v1.0.1', u'vér1.0.1')),
+        ('deadbef000000000000000000000000000000000', ('v1.0', u'vér1.0')),
+        ('deadbeefffffffffffffffffffffffffffffffff', ('0.12-stable',
+                                                      u'0.12-stáblé')),
+        ('0000009876543210987654321098765432109876', ()),  # only digits
+        ('0000001234567890123456789012345678901234', ()),
+        ('1111111111111111111111111111111111111111', ()),  # oldest rev
+    ]
+
+    def __init__(self, reponame):
+        self.reponame = reponame
+        self.resource = Resource('repository', self.reponame)
+        self.youngest_rev = 'ffffffffffffffffffffffffffffffffffffffff'
+        self.oldest_rev = '1111111111111111111111111111111111111111'
+
+    def get_changeset(self, rev):
+        nrev = None
+        if not rev:
+            nrev = self.youngest_rev
+        else:
+            revs = [r for r, names in self._revs if r.startswith(rev)]
+            if len(revs) == 1:
+                nrev = revs[0]
+            else:
+                for r, names in self._revs:
+                    if rev in names:
+                        nrev = r
+                        break
+        if nrev:
+            return Mock(repos=self, rev=nrev, message='message %s' % nrev[:8],
+                        author='trac', is_viewable=lambda perm: True)
+        raise NoSuchChangeset(rev)
+
+    def normalize_rev(self, rev):
+        cset = self.get_changeset(rev)
+        return cset.rev
+
+    def get_node(self, path, rev=None):
+        return _get_node(path, rev)
+
+
 def _get_repository(reponame):
-    return Mock(reponame=reponame, youngest_rev='200',
-                get_changeset=_get_changeset,
+    if reponame.endswith('.git'):
+        return GitRepositoryStub(reponame)
+    return Mock(reponame=reponame, youngest_rev=YOUNGEST_REV,
+                get_changeset=_get_changeset, get_node=_get_node,
                 normalize_rev=_normalize_rev,
-                get_node=_get_node)
+                has_linear_changesets=True,
+                resource=Resource('repository', reponame))
+
+
+def _get_all_repositories():
+    return {'': {}, 'trac.git': {}}
+
 
 def repository_setup(tc):
     setattr(tc.env, 'get_repository', _get_repository)
     setattr(RepositoryManager(tc.env), 'get_repository', _get_repository)
+    setattr(RepositoryManager(tc.env), 'get_all_repositories',
+            _get_all_repositories)
 
 
 CHANGESET_TEST_CASES = u"""
@@ -158,6 +235,32 @@ LOG_TEST_CASES = u"""
 <a class="source" href="/log/trunk?revs=1-2">[2:1/trunk]</a> reversed, <a class="source" href="/log/trunk?revs=1-2">r2:1/trunk</a> reversed
 </p>
 ------------------------------
+============================== changeset and log shorthand syntax with hash ids
+[deadbeef/trac.git]
+[deadbeef/trac.git/trac]
+[deadbeef:deadbef1/trac.git]
+[deadbeef:deadbef1/trac.git/trac]
+------------------------------
+<p>
+<a class="changeset" href="/changeset/deadbeef/trac.git" title="message deadbeef">[deadbeef/trac.git]</a>
+<a class="changeset" href="/changeset/deadbeef/trac.git/trac" title="message deadbeef">[deadbeef/trac.git/trac]</a>
+<a class="source" href="/log/trac.git/?revs=deadbeef%3Adeadbef1">[deadbeef:deadbef1/trac.git]</a>
+<a class="source" href="/log/trac.git/trac?revs=deadbeef%3Adeadbef1">[deadbeef:deadbef1/trac.git/trac]</a>
+</p>
+------------------------------
+============================== changeset and log with digit hash on non linear changesets
+[00000012/trac.git]
+[00000012/trac.git/trac]
+[00000012:00000098/trac.git]
+[00000012:00000098/trac.git/trac]
+------------------------------
+<p>
+<a class="changeset" href="/changeset/00000012/trac.git" title="message 00000012">[00000012/trac.git]</a>
+<a class="changeset" href="/changeset/00000012/trac.git/trac" title="message 00000012">[00000012/trac.git/trac]</a>
+<a class="source" href="/log/trac.git/?revs=00000012%3A00000098">[00000012:00000098/trac.git]</a>
+<a class="source" href="/log/trac.git/trac?revs=00000012%3A00000098">[00000012:00000098/trac.git/trac]</a>
+</p>
+------------------------------
 ============================== Big ranges (#9955 regression)
 [1234567890:12345678901]
 ------------------------------
@@ -177,24 +280,84 @@ LOG_TEST_CASES = u"""
 ============================== log: link resolver
 log:@12
 log:trunk
+log:trunk@head
 log:trunk@12
 log:trunk@12:23
 log:trunk@12-23
 log:trunk:12:23
 log:trunk:12-23
+log:trunk@12:head
 log:trunk:12-head
-log:trunk:12@23 (bad, but shouldn't error out)
+log:trunk:12@23
 ------------------------------
 <p>
 <a class="source" href="/log/?rev=12">log:@12</a>
 <a class="source" href="/log/trunk">log:trunk</a>
+<a class="source" href="/log/trunk?rev=head">log:trunk@head</a>
 <a class="source" href="/log/trunk?rev=12">log:trunk@12</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12:23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk@12-23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk:12:23</a>
 <a class="source" href="/log/trunk?revs=12-23">log:trunk:12-23</a>
-<a class="source" href="/log/trunk?revs=12-200">log:trunk:12-head</a>
-<a class="source" href="/log/trunk">log:trunk:12@23</a> (bad, but shouldn't error out)
+<a class="source" href="/log/trunk?revs=12-head">log:trunk@12:head</a>
+<a class="source" href="/log/trunk?revs=12-head">log:trunk:12-head</a>
+<a class="missing source" title="No changeset 12@23 in the repository">log:trunk:12@23</a>
+</p>
+------------------------------
+============================== log: link resolver with hash revs and named revs
+log:trac.git@fffffff
+log:trac.git/trunk
+log:trac.git/trunk@HEAD
+log:trac.git/trunk@deadbeef
+log:trac.git/trunk@deadbeef:deadbef1
+log:trac.git/trunk@deadbeef-deadbef1
+log:trac.git/trunk:deadbeef:deadbef1
+log:trac.git/trunk:deadbeef-deadbef1
+log:trac.git/trunk@deadbeef:HEAD
+log:trac.git/trunk:deadbeef-HEAD
+log:trac.git/trunk:deadbeef@deadbef1
+log:trac.git/trunk@1.0-stable
+log:trac.git/trunk@0.12-stable:1.0-stable
+log:trac.git/trunk@v1.0-v1.0.1
+log:trac.git/trunk@0.12-stáblé:1.0-stáblé
+log:trac.git/trunk@vér1.0-vér1.0.1
+------------------------------
+<p>
+<a class="source" href="/log/trac.git/?rev=fffffff">log:trac.git@fffffff</a>
+<a class="source" href="/log/trac.git/trunk">log:trac.git/trunk</a>
+<a class="source" href="/log/trac.git/trunk?rev=HEAD">log:trac.git/trunk@HEAD</a>
+<a class="source" href="/log/trac.git/trunk?rev=deadbeef">log:trac.git/trunk@deadbeef</a>
+<a class="source" href="/log/trac.git/trunk?revs=deadbeef%3Adeadbef1">log:trac.git/trunk@deadbeef:deadbef1</a>
+<a class="source" href="/log/trac.git/trunk?revs=deadbeef%3Adeadbef1">log:trac.git/trunk@deadbeef-deadbef1</a>
+<a class="source" href="/log/trac.git/trunk?revs=deadbeef%3Adeadbef1">log:trac.git/trunk:deadbeef:deadbef1</a>
+<a class="source" href="/log/trac.git/trunk?revs=deadbeef%3Adeadbef1">log:trac.git/trunk:deadbeef-deadbef1</a>
+<a class="source" href="/log/trac.git/trunk?revs=deadbeef%3AHEAD">log:trac.git/trunk@deadbeef:HEAD</a>
+<a class="missing source" title="No changeset deadbeef-HEAD in the repository">log:trac.git/trunk:deadbeef-HEAD</a>
+<a class="missing source" title="No changeset deadbeef@deadbef1 in the repository">log:trac.git/trunk:deadbeef@deadbef1</a>
+<a class="source" href="/log/trac.git/trunk?rev=1.0-stable">log:trac.git/trunk@1.0-stable</a>
+<a class="source" href="/log/trac.git/trunk?revs=0.12-stable%3A1.0-stable">log:trac.git/trunk@0.12-stable:1.0-stable</a>
+<a class="missing source" title="No changeset v1.0-v1.0.1 in the repository">log:trac.git/trunk@v1.0-v1.0.1</a>
+<a class="source" href="/log/trac.git/trunk?revs=0.12-st%C3%A1bl%C3%A9%3A1.0-st%C3%A1bl%C3%A9">log:trac.git/trunk@0.12-stáblé:1.0-stáblé</a>
+<a class="missing source" title="No changeset vér1.0-vér1.0.1 in the repository">log:trac.git/trunk@vér1.0-vér1.0.1</a>
+</p>
+------------------------------
+============================== log: link resolver with missing revisions
+log:@4242
+log:@4242-4243
+log:@notfound
+log:@deadbeef:deadbef0
+log:trunk@4243
+log:trunk@notfound
+[4242:4243]
+------------------------------
+<p>
+<a class="missing source" title="No changeset 4242 in the repository">log:@4242</a>
+<a class="source" href="/log/?revs=4242-4243">log:@4242-4243</a>
+<a class="missing source" title="No changeset notfound in the repository">log:@notfound</a>
+<a class="source" href="/log/?revs=deadbeef-deadbef0">log:@deadbeef:deadbef0</a>
+<a class="missing source" title="No changeset 4243 in the repository">log:trunk@4243</a>
+<a class="missing source" title="No changeset notfound in the repository">log:trunk@notfound</a>
+<a class="source" href="/log/?revs=4242-4243">[4242:4243]</a>
 </p>
 ------------------------------
 ============================== log: link resolver + query
@@ -214,6 +377,21 @@ log:trunk@12?limit=10
 <a class="source" href="/log/trunk?revs=10-20&amp;verbose=yes&amp;format=changelog">[10:20/trunk?verbose=yes&amp;format=changelog]</a>
 </p>
 ------------------------------
+============================== log: link resolver + invalid ranges
+log:@10-20-30
+log:@10,20-30,40-50-60
+log:@10:20:30
+[10-20-30]
+[10:20:30]
+------------------------------
+<p>
+<a class="missing source" title="No changeset 10-20-30 in the repository">log:@10-20-30</a>
+<a class="source" href="/log/?revs=10%2C20-30%2C40-50-60">log:@10,20-30,40-50-60</a>
+<a class="missing source" title="No changeset 10:20:30 in the repository">log:@10:20:30</a>
+[10-20-30]
+[10:20:30]
+</p>
+------------------------------
 ============================== Multiple Log ranges
 r12:20,25,35:56,68,69,100-120
 [12:20,25,35:56,68,69,100-120]
@@ -361,17 +539,17 @@ export:/foo/bar.html#header
 """ # " (be Emacs friendly...)
 
 
-
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(formatter.suite(CHANGESET_TEST_CASES, repository_setup,
-                                  __file__))
-    suite.addTest(formatter.suite(LOG_TEST_CASES, repository_setup,
-                                  file=__file__))
-    suite.addTest(formatter.suite(DIFF_TEST_CASES, file=__file__))
-    suite.addTest(formatter.suite(SOURCE_TEST_CASES, repository_setup,
-                                  file=__file__))
+    suite.addTest(formatter.test_suite(CHANGESET_TEST_CASES, repository_setup,
+                                       __file__))
+    suite.addTest(formatter.test_suite(LOG_TEST_CASES, repository_setup,
+                                       __file__))
+    suite.addTest(formatter.test_suite(DIFF_TEST_CASES, file=__file__))
+    suite.addTest(formatter.test_suite(SOURCE_TEST_CASES, repository_setup,
+                                       __file__))
     return suite
 
+
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
