@@ -14,27 +14,31 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+from abc import ABCMeta
 from BaseHTTPServer import BaseHTTPRequestHandler
 from Cookie import CookieError, BaseCookie, SimpleCookie
 import cgi
 from datetime import datetime
-import errno
 from hashlib import md5
 import new
 import mimetypes
 import os
+import re
 import socket
 from StringIO import StringIO
 import sys
 import urlparse
 
-from trac.core import Interface, TracError
-from trac.util import get_last_traceback, unquote
+from genshi.builder import Fragment
+from trac.core import Interface, TracBaseError, TracError
+from trac.util import as_bool, as_int, get_last_traceback, lazy, unquote
 from trac.util.datefmt import http_date, localtz
-from trac.util.text import empty, to_unicode
-from trac.util.translation import _
+from trac.util.html import tag
+from trac.util.text import empty, exception_to_unicode, to_unicode
+from trac.util.translation import _, N_, tag_
 from trac.web.href import Href
-from trac.web.wsgi import _FileWrapper
+from trac.web.wsgi import _FileWrapper, is_client_disconnect_exception
+
 
 class IAuthenticator(Interface):
     """Extension point interface for components that can provide the name
@@ -46,7 +50,19 @@ class IAuthenticator(Interface):
 
 
 class IRequestHandler(Interface):
-    """Decide which `trac.core.Component` handles which `Request`, and how."""
+    """Decide which `trac.core.Component` handles which `Request`, and how.
+
+    The boolean property `is_valid_default_handler` determines whether the
+    `IRequestFilter` can be used as a `default_handler` and defaults to
+    `True`. To be suitable as a `default_handler`, an `IRequestFilter` must
+    return an HTML document and `data` dictionary for rendering the document,
+    and must not require that `match_request` be called prior to
+    `process_request`.
+
+    The boolean property `jquery_noconflict` determines whether jQuery's
+    `noConflict` mode will be activated by the handler, and defaults to
+    `False`.
+    """
 
     def match_request(req):
         """Return whether the handler wants to process the given request."""
@@ -63,9 +79,21 @@ class IRequestHandler(Interface):
         simply send the response itself and not return anything.
 
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` (xml, xhtml or text) may be
+           returned as a fourth parameter in the tuple, but if not specified
+           it will be inferred from the `content_type` when rendering the
+           template.
         """
 
 
+def is_valid_default_handler(handler):
+    """Returns `True` if the `handler` is a valid default handler, as
+    described in the `IRequestHandler` interface documentation.
+    """
+    return handler and getattr(handler, 'is_valid_default_handler', True)
+
+
 class IRequestFilter(Interface):
     """Enable components to interfere with the processing done by the
     main handler, either before and/or after it enters in action.
@@ -78,7 +106,7 @@ class IRequestFilter(Interface):
         Always returns the request handler, even if unchanged.
         """
 
-    def post_process_request(req, template, data, content_type):
+    def post_process_request(req, template, data, content_type, method=None):
         """Do any post-processing the request might need; typically adding
         values to the template `data` dictionary, or changing the Genshi
         template or mime type.
@@ -99,6 +127,11 @@ class IRequestFilter(Interface):
            templates.
 
         :Since 1.0: Clearsilver templates are no longer supported.
+
+        :Since 1.1.2: the rendering `method` will be passed if it is returned
+           by the request handler, otherwise `method` will be `None`. For
+           backward compatibility, the parameter is optional in the
+           implementation's signature.
         """
 
 
@@ -122,22 +155,67 @@ class ITemplateStreamFilter(Interface):
         """
 
 
+class TracNotImplementedError(TracError, NotImplementedError):
+    """Raised when a `NotImplementedError` is trapped.
+
+    This exception is for internal use and should not be raised by
+    plugins. Plugins should raise `NotImplementedError`.
+
+    :since: 1.0.11
+    """
+
+    title = N_("Not Implemented Error")
+
+
 HTTP_STATUS = dict([(code, reason.title()) for code, (reason, description)
                     in BaseHTTPRequestHandler.responses.items()])
 
 
-class HTTPException(Exception):
+class HTTPException(TracBaseError):
+
+    __metaclass__ = ABCMeta
 
     def __init__(self, detail, *args):
-        if isinstance(detail, TracError):
+        """Factory for HTTPException classes."""
+        if isinstance(detail, TracBaseError):
             self.detail = detail.message
             self.reason = detail.title
         else:
             self.detail = detail
         if args:
             self.detail = self.detail % args
-        Exception.__init__(self, '%s %s (%s)' % (self.code, self.reason,
-                                                 self.detail))
+        super(HTTPException, self).__init__('%s %s (%s)' % (self.code,
+                                                            self.reason,
+                                                            self.detail))
+
+    @property
+    def message(self):
+        # The message is based on the e.detail, which can be an Exception
+        # object, but not a TracError one: when creating HTTPException,
+        # a TracError.message is directly assigned to e.detail
+        if isinstance(self.detail, Exception): # not a TracBaseError
+            message = exception_to_unicode(self.detail)
+        elif isinstance(self.detail, Fragment): # TracBaseError markup
+            message = self.detail
+        else:
+            message = to_unicode(self.detail)
+        return message
+
+    @property
+    def title(self):
+        try:
+            # We first try to get localized error messages here, but we
+            # should ignore secondary errors if the main error was also
+            # due to i18n issues
+            title = _("Error")
+            if self.reason:
+                if title.lower() in self.reason.lower():
+                    title = self.reason
+                else:
+                    title = _("Error: %(message)s", message=self.reason)
+        except Exception:
+            title = "Error"
+        return title
 
     @classmethod
     def subclass(cls, name, code):
@@ -181,6 +259,84 @@ class _RequestArgs(dict):
     """Dictionary subclass that provides convenient access to request
     parameters that may contain multiple values."""
 
+    def as_int(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Return `default` if
+        if an exception is raised while converting the value to an
+        integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to an integer.
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_int(self.getfirst(name), default, min, max)
+
+    def as_bool(self, name, default=None):
+        """Return the value as a boolean. Return `default` if
+        if an exception is raised while converting the value to a
+        boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified or an exception occurs converting
+                          the value to a boolean.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        return as_bool(self.getfirst(name), default)
+
+    def getbool(self, name, default=None):
+        """Return the value as a boolean. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value to
+        a boolean.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified.
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = self[name]
+        if isinstance(value, list):
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        value = as_bool(value, None)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
+    def getint(self, name, default=None, min=None, max=None):
+        """Return the value as an integer. Raise an `HTTPBadRequest`
+        exception if an exception occurs while converting the value
+        to an integer.
+
+        :param name: the name of the request parameter
+        :keyword default: the value to return if the parameter is not
+                          specified
+        :keyword min: lower bound to which the value is limited
+        :keyword max: upper bound to which the value is limited
+
+        :since: 1.2
+        """
+        if name not in self:
+            return default
+        value = as_int(self[name], None, min, max)
+        if value is None:
+            raise HTTPBadRequest(tag_("Invalid value for request argument "
+                                      "%(name)s.", name=tag.em(name)))
+        return value
+
     def getfirst(self, name, default=None):
         """Return the first value for the specified parameter, or `default` if
         the parameter was not provided.
@@ -203,12 +359,28 @@ class _RequestArgs(dict):
             val = [val]
         return val
 
+    def require(self, name):
+        """Raise an `HTTPBadRequest` exception if the parameter is
+        not in the request.
+
+        :param name: the name of the request parameter
+
+        :since: 1.2
+        """
+        if name not in self:
+            raise HTTPBadRequest(
+                tag_("Missing request argument. The %(name)s argument "
+                     "must be included in the request.", name=tag.em(name)))
+
 
 def parse_arg_list(query_string):
-    """Parse a query string into a list of `(name, value)` tuples."""
+    """Parse a query string into a list of `(name, value)` tuples.
+
+    :Since 1.1.2: a leading `?` is stripped from `query_string`."""
     args = []
     if not query_string:
         return args
+    query_string = query_string.lstrip('?')
     for arg in query_string.split('&'):
         nv = arg.split('=', 1)
         if len(nv) == 2:
@@ -239,10 +411,14 @@ def arg_list_to_args(arg_list):
     return args
 
 
-class RequestDone(Exception):
+class RequestDone(TracBaseError):
     """Marker exception that indicates whether request processing has completed
     and a response was sent.
     """
+    iterable = None
+
+    def __init__(self, iterable=None):
+        self.iterable = iterable
 
 
 class Cookie(SimpleCookie):
@@ -319,12 +495,22 @@ class Request(object):
         raise AttributeError(name)
 
     def __repr__(self):
-        path_info = self.environ.get('PATH_INFO', '')
-        return '<%s "%s %r">' % (self.__class__.__name__, self.method,
-                                 path_info)
+        uri = self.environ.get('PATH_INFO', '')
+        qs = self.query_string
+        if qs:
+            uri += '?' + qs
+        return '<%s "%s %r">' % (self.__class__.__name__, self.method, uri)
 
     # Public API
 
+    @lazy
+    def is_xhr(self):
+        """Returns `True` if the request is an `XMLHttpRequest`.
+
+        :since: 1.1.6
+        """
+        return self.get_header('X-Requested-With') == 'XMLHttpRequest'
+
     @property
     def method(self):
         """The HTTP method of the request"""
@@ -356,7 +542,9 @@ class Request(object):
 
         Will be `None` if the user has not logged in using HTTP authentication.
         """
-        return self.environ.get('REMOTE_USER')
+        user = self.environ.get('REMOTE_USER')
+        if user is not None:
+            return to_unicode(user)
 
     @property
     def scheme(self):
@@ -405,11 +593,12 @@ class Request(object):
         `value` must either be an `unicode` string or can be converted to one
         (e.g. numbers, ...)
         """
-        if name.lower() == 'content-type':
+        lower_name = name.lower()
+        if lower_name == 'content-type':
             ctpos = value.find('charset=')
             if ctpos >= 0:
                 self._outcharset = value[ctpos + 8:].strip()
-        elif name.lower() == 'content-length':
+        elif lower_name == 'content-length':
             self._content_length = int(value)
         self._outheaders.append((name, unicode(value).encode('utf-8')))
 
@@ -442,7 +631,7 @@ class Request(object):
             extra = m.hexdigest()
         etag = 'W/"%s/%s/%s"' % (self.authname, http_date(datetime), extra)
         inm = self.get_header('If-None-Match')
-        if (not inm or inm != etag):
+        if not inm or inm != etag:
             self.send_header('ETag', etag)
         else:
             self.send_response(304)
@@ -450,6 +639,8 @@ class Request(object):
             self.end_headers()
             raise RequestDone
 
+    _trident_re = re.compile(r' Trident/([0-9]+)')
+
     def redirect(self, url, permanent=False):
         """Send a redirect to the client, forwarding to the specified URL.
 
@@ -472,10 +663,13 @@ class Request(object):
             scheme, host = urlparse.urlparse(self.base_url)[:2]
             url = urlparse.urlunparse((scheme, host, url, None, None, None))
 
-        # Workaround #10382, IE6+ bug when post and redirect with hash
-        if status == 303 and '#' in url and \
-                ' MSIE ' in self.environ.get('HTTP_USER_AGENT', ''):
-            url = url.replace('#', '#__msie303:')
+        # Workaround #10382, IE6-IE9 bug when post and redirect with hash
+        if status == 303 and '#' in url:
+            user_agent = self.environ.get('HTTP_USER_AGENT', '')
+            match_trident = self._trident_re.search(user_agent)
+            if ' MSIE ' in user_agent and \
+                    (not match_trident or int(match_trident.group(1)) < 6):
+                url = url.replace('#', '#__msie303:')
 
         self.send_header('Location', url)
         self.send_header('Content-Type', 'text/plain')
@@ -491,7 +685,8 @@ class Request(object):
         self.send_header('Cache-Control', 'must-revalidate')
         self.send_header('Expires', 'Fri, 01 Jan 1999 00:00:00 GMT')
         self.send_header('Content-Type', content_type + ';charset=utf-8')
-        self.send_header('Content-Length', len(content))
+        if isinstance(content, basestring):
+            self.send_header('Content-Length', len(content))
         self.end_headers()
 
         if self.method != 'HEAD':
@@ -503,7 +698,8 @@ class Request(object):
         try:
             if template.endswith('.html'):
                 if env:
-                    from trac.web.chrome import Chrome
+                    from trac.web.chrome import Chrome, add_stylesheet
+                    add_stylesheet(self, 'common/css/code.css')
                     try:
                         data = Chrome(env).render_template(self, template,
                                                            data, 'text/html')
@@ -577,11 +773,15 @@ class Request(object):
         self.send_header('Last-Modified', last_modified)
         use_xsendfile = getattr(self, 'use_xsendfile', False)
         if use_xsendfile:
-            self.send_header('X-Sendfile', os.path.abspath(path))
+            xsendfile_header = getattr(self, 'xsendfile_header', None)
+            if xsendfile_header:
+                self.send_header(xsendfile_header, os.path.abspath(path))
+            else:
+                use_xsendfile = False
         self.end_headers()
 
         if not use_xsendfile and self.method != 'HEAD':
-            fileobj = file(path, 'rb')
+            fileobj = open(path, 'rb')
             file_wrapper = self.environ.get('wsgi.file_wrapper', _FileWrapper)
             self._response = file_wrapper(fileobj, 4096)
         raise RequestDone
@@ -598,28 +798,45 @@ class Request(object):
         data = fileobj.read(size)
         return data
 
+    CHUNK_SIZE = 4096
+
     def write(self, data):
         """Write the given data to the response body.
 
-        `data` *must* be a `str` string, encoded with the charset
-        which has been specified in the ''Content-Type'' header
-        or 'utf-8' otherwise.
+        *data* **must** be a `str` string or an iterable instance
+        which iterates `str` strings, encoded with the charset which
+        has been specified in the ``'Content-Type'`` header or UTF-8
+        otherwise.
 
-        Note that the ''Content-Length'' header must have been specified.
-        Its value either corresponds to the length of `data`, or, if there
-        are multiple calls to `write`, to the cumulated length of the `data`
-        arguments.
+        Note that when the ``'Content-Length'`` header is specified,
+        its value either corresponds to the length of *data*, or, if
+        there are multiple calls to `write`, to the cumulative length
+        of the *data* arguments.
         """
         if not self._write:
             self.end_headers()
-        if not hasattr(self, '_content_length'):
-            raise RuntimeError("No Content-Length header set")
-        if isinstance(data, unicode):
-            raise ValueError("Can't send unicode content")
         try:
-            self._write(data)
-        except (IOError, socket.error), e:
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
+            chunk_size = self.CHUNK_SIZE
+            bufsize = 0
+            buf = []
+            buf_append = buf.append
+            if isinstance(data, basestring):
+                data = [data]
+            for chunk in data:
+                if isinstance(chunk, unicode):
+                    raise ValueError("Can't send unicode content")
+                if not chunk:
+                    continue
+                bufsize += len(chunk)
+                buf_append(chunk)
+                if bufsize >= chunk_size:
+                    self._write(''.join(buf))
+                    bufsize = 0
+                    buf[:] = ()
+            if bufsize > 0:
+                self._write(''.join(buf))
+        except (IOError, socket.error) as e:
+            if self._is_client_disconnected(e):
                 raise RequestDone
             raise
 
@@ -645,15 +862,39 @@ class Request(object):
         # requests. We'll keep the pre 2.6 behaviour for now...
         if self.method == 'POST':
             qs_on_post = self.environ.pop('QUERY_STRING', '')
-        fs = _FieldStorage(fp, environ=self.environ, keep_blank_values=True)
+        try:
+            fs = _FieldStorage(fp, environ=self.environ,
+                               keep_blank_values=True)
+        except (IOError, socket.error) as e:
+            if self._is_client_disconnected(e):
+                raise HTTPBadRequest(
+                    _("Exception caught while reading request: %(msg)s",
+                      msg=exception_to_unicode(e)))
+            raise
         if self.method == 'POST':
             self.environ['QUERY_STRING'] = qs_on_post
 
+        def raise_if_null_bytes(value):
+            if value and '\x00' in value:
+                raise HTTPBadRequest(_("Invalid request arguments."))
+
         args = []
         for value in fs.list or ():
             name = value.name
-            if not value.filename:
-                value = unicode(value.value, 'utf-8')
+            raise_if_null_bytes(name)
+            try:
+                if name is not None:
+                    name = unicode(name, 'utf-8')
+                if value.filename:
+                    raise_if_null_bytes(value.filename)
+                else:
+                    value = value.value
+                    raise_if_null_bytes(value)
+                    value = unicode(value, 'utf-8')
+            except UnicodeDecodeError as e:
+                raise HTTPBadRequest(
+                    _("Invalid encoding in form data: %(msg)s",
+                      msg=exception_to_unicode(e)))
             args.append((name, value))
         return args
 
@@ -700,7 +941,7 @@ class Request(object):
             # server name and port
             default_port = {'http': 80, 'https': 443}
             if self.server_port and self.server_port != \
-                   default_port[self.scheme]:
+                    default_port[self.scheme]:
                 host = '%s:%d' % (self.server_name, self.server_port)
             else:
                 host = self.server_name
@@ -720,4 +961,15 @@ class Request(object):
         for cookie in cookies.splitlines():
             self._outheaders.append(('Set-Cookie', cookie.strip()))
 
+    def _is_client_disconnected(self, e):
+        if is_client_disconnect_exception(e):
+            return True
+        # Note that mod_wsgi raises an IOError with only a message
+        # if the client disconnects
+        if 'mod_wsgi.version' in self.environ:
+            return e.args[0] in ('failed to write data',
+                                 'client connection closed',
+                                 'request data read error')
+        return False
+
 __no_apidoc__ = _HTTPException_subclass_names
