@@ -14,18 +14,14 @@
 #
 # Author: Jonas Borgström <jonas@edgewall.com>
 
-from __future__ import with_statement
+from __future__ import print_function
 
-try:
-    from base64 import b64decode, b64encode
-except ImportError:
-    from base64 import decodestring as b64decode
-    from base64 import encodestring as b64encode
+from abc import ABCMeta, abstractmethod
+from base64 import b64decode, b64encode
 from hashlib import md5, sha1
 import os
 import re
 import sys
-import time
 import urllib2
 import urlparse
 
@@ -34,9 +30,11 @@ from genshi.builder import tag
 from trac.config import BoolOption, IntOption, Option
 from trac.core import *
 from trac.web.api import IAuthenticator, IRequestHandler
-from trac.web.chrome import INavigationContributor
+from trac.web.chrome import Chrome, INavigationContributor
 from trac.util import hex_entropy, md5crypt
+from trac.util.compat import crypt
 from trac.util.concurrency import threading
+from trac.util.datefmt import time_now
 from trac.util.translation import _, tag_
 
 
@@ -58,20 +56,28 @@ class LoginModule(Component):
 
     implements(IAuthenticator, INavigationContributor, IRequestHandler)
 
+    is_valid_default_handler = False
+
     check_ip = BoolOption('trac', 'check_auth_ip', 'false',
          """Whether the IP address of the user should be checked for
-         authentication (''since 0.9'').""")
+         authentication.""")
 
     ignore_case = BoolOption('trac', 'ignore_auth_case', 'false',
-        """Whether login names should be converted to lower case
-        (''since 0.9'').""")
+        """Whether login names should be converted to lower case.""")
+
+    auth_cookie_domain = Option('trac', 'auth_cookie_domain', '',
+        """Auth cookie domain attribute.
+
+        The auth cookie can be shared among multiple subdomains
+        by setting the value to the domain. (//since 1.2//)
+        """)
 
     auth_cookie_lifetime = IntOption('trac', 'auth_cookie_lifetime', 0,
         """Lifetime of the authentication cookie, in seconds.
 
         This value determines how long the browser will cache
         authentication information, and therefore, after how much
-        inactivity a user will have to log in again. The default value
+        inactivity a user will have to log in again. The value
         of 0 makes the cookie expire at the end of the browsing
         session. (''since 0.12'')""")
 
@@ -86,7 +92,7 @@ class LoginModule(Component):
         authname = None
         if req.remote_user:
             authname = req.remote_user
-        elif req.incookie.has_key('trac_auth'):
+        elif 'trac_auth' in req.incookie:
             authname = self._get_name_for_cookie(req,
                                                  req.incookie['trac_auth'])
 
@@ -105,13 +111,17 @@ class LoginModule(Component):
 
     def get_navigation_items(self, req):
         if req.authname and req.authname != 'anonymous':
-            yield ('metanav', 'login', _('logged in as %(user)s',
-                                         user=req.authname))
+            yield ('metanav', 'login',
+                   tag_("logged in as %(user)s",
+                        user=Chrome(self.env).authorinfo(req, req.authname)))
             yield ('metanav', 'logout',
-                   tag.a(_('Logout'), href=req.href.logout()))
+                   tag.form(tag.div(tag.button(_("Logout"),
+                                               name='logout', type='submit')),
+                            action=req.href.logout(), method='post',
+                            id='logout', class_='trac-logout'))
         else:
             yield ('metanav', 'login',
-                   tag.a(_('Login'), href=req.href.login()))
+                   tag.a(_("Login"), href=req.href.login()))
 
     # IRequestHandler methods
 
@@ -144,10 +154,10 @@ class LoginModule(Component):
         """
         if not req.remote_user:
             # TRANSLATOR: ... refer to the 'installation documentation'. (link)
-            inst_doc = tag.a(_('installation documentation'),
+            inst_doc = tag.a(_("installation documentation"),
                              title=_("Configuring Authentication"),
-                             href=req.href.wiki('TracInstall')
-                                  + "#ConfiguringAuthentication")
+                             href=req.href.wiki('TracInstall') +
+                                                "#ConfiguringAuthentication")
             raise TracError(tag_("Authentication information not available. "
                                  "Please refer to the %(inst_doc)s.",
                                  inst_doc=inst_doc))
@@ -155,13 +165,14 @@ class LoginModule(Component):
         if self.ignore_case:
             remote_user = remote_user.lower()
 
-        assert req.authname in ('anonymous', remote_user), \
-               _('Already logged in as %(user)s.', user=req.authname)
+        if req.authname not in ('anonymous', remote_user):
+            raise TracError(_("Already logged in as %(user)s.",
+                              user=req.authname))
 
         with self.env.db_transaction as db:
             # Delete cookies older than 10 days
             db("DELETE FROM auth_cookie WHERE time < %s",
-               (int(time.time()) - 86400 * 10,))
+               (int(time_now()) - 86400 * 10,))
             # Insert a new cookie if we haven't already got one
             cookie = None
             trac_auth = req.incookie.get('trac_auth')
@@ -174,15 +185,16 @@ class LoginModule(Component):
                     INSERT INTO auth_cookie (cookie, name, ipnr, time)
                          VALUES (%s, %s, %s, %s)
                    """, (cookie, remote_user, req.remote_addr,
-                         int(time.time())))
+                         int(time_now())))
         req.authname = remote_user
         req.outcookie['trac_auth'] = cookie
+        if self.auth_cookie_domain:
+            req.outcookie['trac_auth']['domain'] = self.auth_cookie_domain
         req.outcookie['trac_auth']['path'] = self.auth_cookie_path \
                                              or req.base_path or '/'
         if self.env.secure_cookies:
             req.outcookie['trac_auth']['secure'] = True
-        if sys.version_info >= (2, 6):
-            req.outcookie['trac_auth']['httponly'] = True
+        req.outcookie['trac_auth']['httponly'] = True
         if self.auth_cookie_lifetime > 0:
             req.outcookie['trac_auth']['expires'] = self.auth_cookie_lifetime
 
@@ -192,6 +204,8 @@ class LoginModule(Component):
         Simply deletes the corresponding record from the auth_cookie
         table.
         """
+        if req.method != 'POST':
+            return
         if req.authname == 'anonymous':
             # Not logged in
             return
@@ -214,13 +228,14 @@ class LoginModule(Component):
         the "expires" property to a date in the past.
         """
         req.outcookie['trac_auth'] = ''
+        if self.auth_cookie_domain:
+            req.outcookie['trac_auth']['domain'] = self.auth_cookie_domain
         req.outcookie['trac_auth']['path'] = self.auth_cookie_path \
                                              or req.base_path or '/'
         req.outcookie['trac_auth']['expires'] = -10000
         if self.env.secure_cookies:
             req.outcookie['trac_auth']['secure'] = True
-        if sys.version_info >= (2, 6):
-            req.outcookie['trac_auth']['httponly'] = True
+        req.outcookie['trac_auth']['httponly'] = True
 
     def _cookie_to_name(self, req, cookie):
         # This is separated from _get_name_for_cookie(), because the
@@ -272,8 +287,11 @@ class LoginModule(Component):
 
 class HTTPAuthentication(object):
 
+    __metaclass__ = ABCMeta
+
+    @abstractmethod
     def do_auth(self, environ, start_response):
-        raise NotImplementedError
+        pass
 
 
 class PasswordFileAuthentication(HTTPAuthentication):
@@ -286,7 +304,7 @@ class PasswordFileAuthentication(HTTPAuthentication):
     def check_reload(self):
         with self._lock:
             mtime = os.stat(self.filename).st_mtime
-            if mtime > self.mtime:
+            if mtime != self.mtime:
                 self.mtime = mtime
                 self.load(self.filename)
 
@@ -296,15 +314,8 @@ class BasicAuthentication(PasswordFileAuthentication):
     def __init__(self, htpasswd, realm):
         # FIXME pass a logger
         self.realm = realm
-        try:
-            import crypt
-            self.crypt = crypt.crypt
-        except ImportError:
-            try:
-                import fcrypt
-                self.crypt = fcrypt.crypt
-            except ImportError:
-                self.crypt = None
+        self.crypt = crypt
+        self.hash = {}
         PasswordFileAuthentication.__init__(self, htpasswd)
 
     def load(self, filename):
@@ -318,17 +329,19 @@ class BasicAuthentication(PasswordFileAuthentication):
             try:
                 u, h = line.split(':')
             except ValueError:
-                print>>sys.stderr, 'Warning: invalid password line in %s: ' \
-                    '%s' % (filename, line)
+                print("Warning: invalid password line in %s: %s"
+                      % (filename, line), file=sys.stderr)
                 continue
             if '$' in h or h.startswith('{SHA}') or self.crypt:
                 self.hash[u] = h
             else:
-                print>>sys.stderr, 'Warning: cannot parse password for ' \
-                    'user "%s" without the "crypt" module' % u
+                print('Warning: cannot parse password for user "%s" '
+                      'without the "crypt" module. Install the passlib '
+                      'package from PyPI' % u, file=sys.stderr)
 
         if self.hash == {}:
-            print>>sys.stderr, "Warning: found no users in file:", filename
+            print("Warning: found no users in file:", filename,
+                  file=sys.stderr)
 
     def test(self, user, password):
         self.check_reload()
@@ -339,7 +352,7 @@ class BasicAuthentication(PasswordFileAuthentication):
         if the_hash.startswith('{SHA}'):
             return b64encode(sha1(password).digest()) == the_hash[5:]
 
-        if not '$' in the_hash:
+        if '$' not in the_hash:
             return self.crypt(password, the_hash[:2]) == the_hash
 
         magic, salt = the_hash[1:].split('$')[:2]
@@ -370,6 +383,7 @@ class DigestAuthentication(PasswordFileAuthentication):
         # FIXME pass a logger
         self.active_nonces = []
         self.realm = realm
+        self.hash = {}
         PasswordFileAuthentication.__init__(self, htdigest)
 
     def load(self, filename):
@@ -386,13 +400,14 @@ class DigestAuthentication(PasswordFileAuthentication):
             try:
                 u, r, a1 = line.split(':')
             except ValueError:
-                print>>sys.stderr, 'Warning: invalid digest line in %s: %s' \
-                    % (filename, line)
+                print("Warning: invalid digest line in %s: %s"
+                      % (filename, line), file=sys.stderr)
                 continue
             if r == self.realm:
                 self.hash[u] = a1
         if self.hash == {}:
-            print>>sys.stderr, "Warning: found no users in realm:", self.realm
+            print("Warning: found no users in realm:", self.realm,
+                  file=sys.stderr)
 
     def parse_auth_header(self, authorization):
         values = {}
@@ -429,12 +444,12 @@ class DigestAuthentication(PasswordFileAuthentication):
                          'nc', 'cnonce']
         # Invalid response?
         for key in required_keys:
-            if not auth.has_key(key):
+            if key not in auth:
                 self.send_auth_request(environ, start_response)
                 return None
         # Unknown user?
         self.check_reload()
-        if not self.hash.has_key(auth['username']):
+        if auth['username'] not in self.hash:
             self.send_auth_request(environ, start_response)
             return None
 
