@@ -16,60 +16,90 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 #         Matthew Good <trac@matt-good.net>
 
+from __future__ import print_function
+
 import cgi
 import dircache
 import fnmatch
 from functools import partial
 import gc
+import io
 import locale
 import os
 import pkg_resources
 from pprint import pformat, pprint
 import re
 import sys
+import traceback
+from urlparse import urlparse
 
-from genshi.builder import Fragment, tag
+from genshi.builder import tag
 from genshi.output import DocType
 from genshi.template import TemplateLoader
 
 from trac import __version__ as TRAC_VERSION
-from trac.config import BoolOption, ExtensionOption, Option, \
-                        OrderedExtensionsOption
+from trac.config import BoolOption, ChoiceOption, ConfigurationError, \
+                        ExtensionOption, Option, OrderedExtensionsOption
 from trac.core import *
 from trac.env import open_environment
 from trac.loader import get_plugin_info, match_plugins_to_frames
 from trac.perm import PermissionCache, PermissionError
 from trac.resource import ResourceNotFound
 from trac.util import arity, get_frame_info, get_last_traceback, hex_entropy, \
-                      read_file, safe_repr, translation
+                      lazy, read_file, safe_repr, translation, \
+                      warn_setuptools_issue
 from trac.util.concurrency import threading
 from trac.util.datefmt import format_datetime, localtz, timezone, user_time
-from trac.util.text import exception_to_unicode, shorten_line, to_unicode
+from trac.util.text import exception_to_unicode, shorten_line, to_unicode, \
+                           to_utf8, unicode_quote
 from trac.util.translation import _, get_negotiated_locale, has_babel, \
                                   safefmt, tag_
-from trac.web.api import *
-from trac.web.chrome import Chrome
+from trac.web.api import HTTPBadRequest, HTTPException, HTTPForbidden, \
+                         HTTPInternalError, HTTPNotFound, IAuthenticator, \
+                         IRequestFilter, IRequestHandler, Request, \
+                         RequestDone, TracNotImplementedError, \
+                         is_valid_default_handler
+from trac.web.chrome import Chrome, ITemplateProvider, add_notice, add_warning
 from trac.web.href import Href
-from trac.web.session import Session
+from trac.web.session import SessionDict, Session
 
 #: This URL is used for semi-automatic bug reports (see
 #: `send_internal_error`).  Please modify it to point to your own
 #: Trac instance if you distribute a patched version of Trac.
-default_tracker = 'http://trac.edgewall.org'
+default_tracker = 'https://trac.edgewall.org'
 
 
-class FakeSession(dict):
-    sid = None
+class FakeSession(SessionDict):
+
+    def get_session(self, sid, authenticated=False):
+        pass
+
     def save(self):
         pass
 
 
-class FakePerm(dict):
-    def require(self, *args):
-        return False
-    def __call__(self, *args):
+class FakePerm(object):
+
+    username = 'anonymous'
+
+    def __call__(self, realm_or_resource, id=False, version=False):
         return self
 
+    def has_permission(self, action, realm_or_resource=None, id=False,
+                       version=False):
+        return False
+
+    __contains__ = has_permission
+
+    def require(self, action, realm_or_resource=None, id=False, version=False,
+                message=None):
+        if message is None:
+            raise PermissionError(action)
+        else:
+            raise PermissionError(msg=message)
+
+    assert_permission = require
+
 
 class RequestWithSession(Request):
     """A request that saves its associated session when sending the reply."""
@@ -89,13 +119,14 @@ class RequestDispatcher(Component):
     """
     required = True
 
+    implements(ITemplateProvider)
+
     authenticators = ExtensionPoint(IAuthenticator)
     handlers = ExtensionPoint(IRequestHandler)
 
     filters = OrderedExtensionsOption('trac', 'request_filters',
                                       IRequestFilter,
-        doc="""Ordered list of filters to apply to all requests
-            (''since 0.10'').""")
+        doc="""Ordered list of filters to apply to all requests.""")
 
     default_handler = ExtensionOption('trac', 'default_handler',
                                       IRequestHandler, 'WikiModule',
@@ -104,7 +135,7 @@ class RequestDispatcher(Component):
 
         Options include `TimelineModule`, `RoadmapModule`,
         `BrowserModule`, `QueryModule`, `ReportModule`, `TicketModule`
-        and `WikiModule`. The default is `WikiModule`. (''since 0.9'')""")
+        and `WikiModule`.""")
 
     default_timezone = Option('trac', 'default_timezone', '',
         """The default timezone to use""")
@@ -114,7 +145,8 @@ class RequestDispatcher(Component):
         been set. (''since 0.12.1'')
         """)
 
-    default_date_format = Option('trac', 'default_date_format', '',
+    default_date_format = ChoiceOption('trac', 'default_date_format',
+                                       ('', 'iso8601'),
         """The date format. Valid options are 'iso8601' for selecting
         ISO 8601 format, or leave it empty which means the default
         date format will be inferred from the browser's default
@@ -128,15 +160,26 @@ class RequestDispatcher(Component):
         like Apache with `mod_xsendfile` or lighttpd. (''since 1.0'')
         """)
 
+    xsendfile_header = Option('trac', 'xsendfile_header', 'X-Sendfile',
+        """The header to use if `use_xsendfile` is enabled. If Nginx is used,
+        set `X-Accel-Redirect`. (''since 1.0.6'')""")
+
     # Public API
 
     def authenticate(self, req):
         for authenticator in self.authenticators:
-            authname = authenticator.authenticate(req)
+            try:
+                authname = authenticator.authenticate(req)
+            except TracError as e:
+                self.log.error("Can't authenticate using %s: %s",
+                               authenticator.__class__.__name__,
+                               exception_to_unicode(e, traceback=True))
+                add_warning(req, _("Authentication error. "
+                                   "Please contact your administrator."))
+                break  # don't fallback to other authenticators
             if authname:
                 return authname
-        else:
-            return 'anonymous'
+        return 'anonymous'
 
     def dispatch(self, req):
         """Find a registered handler that matches the request and let
@@ -152,121 +195,160 @@ class RequestDispatcher(Component):
         req.callbacks.update({
             'authname': self.authenticate,
             'chrome': chrome.prepare_request,
+            'form_token': self._get_form_token,
+            'lc_time': self._get_lc_time,
+            'locale': self._get_locale,
             'perm': self._get_perm,
             'session': self._get_session,
-            'locale': self._get_locale,
-            'lc_time': self._get_lc_time,
             'tz': self._get_timezone,
-            'form_token': self._get_form_token,
             'use_xsendfile': self._get_use_xsendfile,
+            'xsendfile_header': self._get_xsendfile_header,
         })
 
         try:
+            # Select the component that should handle the request
+            chosen_handler = None
+            for handler in self._request_handlers.values():
+                if handler.match_request(req):
+                    chosen_handler = handler
+                    break
+            if not chosen_handler and req.path_info in ('', '/'):
+                chosen_handler = self._get_valid_default_handler(req)
+            # pre-process any incoming request, whether a handler
+            # was found or not
+            self.log.debug("Chosen handler is %s", chosen_handler)
+            chosen_handler = self._pre_process_request(req, chosen_handler)
+            if not chosen_handler:
+                if req.path_info.endswith('/'):
+                    # Strip trailing / and redirect
+                    target = unicode_quote(req.path_info.rstrip('/'))
+                    if req.query_string:
+                        target += '?' + req.query_string
+                    req.redirect(req.href + target, permanent=True)
+                raise HTTPNotFound('No handler matched request to %s',
+                                   req.path_info)
+
+            req.callbacks['chrome'] = partial(chrome.prepare_request,
+                                              handler=chosen_handler)
+
+            # Protect against CSRF attacks: we validate the form token
+            # for all POST requests with a content-type corresponding
+            # to form submissions
+            if req.method == 'POST':
+                ctype = req.get_header('Content-Type')
+                if ctype:
+                    ctype, options = cgi.parse_header(ctype)
+                if ctype in ('application/x-www-form-urlencoded',
+                             'multipart/form-data') and \
+                        req.args.get('__FORM_TOKEN') != req.form_token:
+                    if self.env.secure_cookies and req.scheme == 'http':
+                        msg = _('Secure cookies are enabled, you must '
+                                'use https to submit forms.')
+                    else:
+                        msg = _('Do you have cookies enabled?')
+                    raise HTTPBadRequest(_('Missing or invalid form token.'
+                                           ' %(msg)s', msg=msg))
+
+            # Process the request and render the template
+            resp = chosen_handler.process_request(req)
+            if resp:
+                if len(resp) == 2: # old Clearsilver template and HDF data
+                    self.log.error("Clearsilver template are no longer "
+                                   "supported (%s)", resp[0])
+                    raise TracError(
+                        _("Clearsilver templates are no longer supported, "
+                          "please contact your Trac administrator."))
+                # Genshi
+                template, data, content_type, method = \
+                    self._post_process_request(req, *resp)
+                if 'hdfdump' in req.args:
+                    req.perm.require('TRAC_ADMIN')
+                    # debugging helper - no need to render first
+                    out = io.BytesIO()
+                    pprint(data, out)
+                    req.send(out.getvalue(), 'text/plain')
+                self.log.debug("Rendering response from handler")
+                output = chrome.render_template(
+                        req, template, data, content_type, method=method,
+                        iterable=chrome.use_chunked_encoding)
+                req.send(output, content_type or 'text/html')
+            else:
+                self.log.debug("Empty or no response from handler. "
+                               "Entering post_process_request.")
+                self._post_process_request(req)
+        except RequestDone:
+            raise
+        except Exception as e:
+            # post-process the request in case of errors
+            err = sys.exc_info()
             try:
-                # Select the component that should handle the request
-                chosen_handler = None
-                try:
-                    for handler in self.handlers:
-                        if handler.match_request(req):
-                            chosen_handler = handler
-                            break
-                    if not chosen_handler:
-                        if not req.path_info or req.path_info == '/':
-                            chosen_handler = self.default_handler
-                    # pre-process any incoming request, whether a handler
-                    # was found or not
-                    chosen_handler = self._pre_process_request(req,
-                                                            chosen_handler)
-                except TracError, e:
-                    raise HTTPInternalError(e)
-                if not chosen_handler:
-                    if req.path_info.endswith('/'):
-                        # Strip trailing / and redirect
-                        target = req.path_info.rstrip('/').encode('utf-8')
-                        if req.query_string:
-                            target += '?' + req.query_string
-                        req.redirect(req.href + target, permanent=True)
-                    raise HTTPNotFound('No handler matched request to %s',
-                                       req.path_info)
-
-                req.callbacks['chrome'] = partial(chrome.prepare_request,
-                                                  handler=chosen_handler)
-
-                # Protect against CSRF attacks: we validate the form token
-                # for all POST requests with a content-type corresponding
-                # to form submissions
-                if req.method == 'POST':
-                    ctype = req.get_header('Content-Type')
-                    if ctype:
-                        ctype, options = cgi.parse_header(ctype)
-                    if ctype in ('application/x-www-form-urlencoded',
-                                 'multipart/form-data') and \
-                            req.args.get('__FORM_TOKEN') != req.form_token:
-                        if self.env.secure_cookies and req.scheme == 'http':
-                            msg = _('Secure cookies are enabled, you must '
-                                    'use https to submit forms.')
-                        else:
-                            msg = _('Do you have cookies enabled?')
-                        raise HTTPBadRequest(_('Missing or invalid form token.'
-                                               ' %(msg)s', msg=msg))
-
-                # Process the request and render the template
-                resp = chosen_handler.process_request(req)
-                if resp:
-                    if len(resp) == 2: # old Clearsilver template and HDF data
-                        self.log.error("Clearsilver template are no longer "
-                                       "supported (%s)", resp[0])
-                        raise TracError(
-                            _("Clearsilver templates are no longer supported, "
-                              "please contact your Trac administrator."))
-                    # Genshi
-                    template, data, content_type = \
-                              self._post_process_request(req, *resp)
-                    if 'hdfdump' in req.args:
-                        req.perm.require('TRAC_ADMIN')
-                        # debugging helper - no need to render first
-                        out = StringIO()
-                        pprint(data, out)
-                        req.send(out.getvalue(), 'text/plain')
-
-                    output = chrome.render_template(req, template, data,
-                                                    content_type)
-                    req.send(output, content_type or 'text/html')
-                else:
-                    self._post_process_request(req)
+                self._post_process_request(req)
             except RequestDone:
-                raise
-            except:
-                # post-process the request in case of errors
-                err = sys.exc_info()
-                try:
-                    self._post_process_request(req)
-                except RequestDone:
-                    raise
-                except Exception, e:
-                    self.log.error("Exception caught while post-processing"
-                                   " request: %s",
-                                   exception_to_unicode(e, traceback=True))
-                raise err[0], err[1], err[2]
-        except PermissionError, e:
-            raise HTTPForbidden(to_unicode(e))
-        except ResourceNotFound, e:
-            raise HTTPNotFound(e)
-        except TracError, e:
-            raise HTTPInternalError(e)
+                pass
+            except Exception as e2:
+                self.log.error("Exception caught while post-processing"
+                               " request: %s",
+                               exception_to_unicode(e2, traceback=True))
+            if isinstance(e, PermissionError):
+                raise HTTPForbidden(e)
+            if isinstance(e, ResourceNotFound):
+                raise HTTPNotFound(e)
+            if isinstance(e, NotImplementedError):
+                tb = traceback.extract_tb(err[2])[-1]
+                self.log.warning("%s caught from %s:%d in %s: %s",
+                                 e.__class__.__name__, tb[0], tb[1], tb[2],
+                                 to_unicode(e) or "(no message)")
+                raise HTTPInternalError(TracNotImplementedError(e))
+            if isinstance(e, TracError):
+                raise HTTPInternalError(e)
+            raise err[0], err[1], err[2]
+
+    # ITemplateProvider methods
+
+    def get_htdocs_dirs(self):
+        return []
+
+    def get_templates_dirs(self):
+        return [pkg_resources.resource_filename('trac.web', 'templates')]
 
     # Internal methods
 
+    @lazy
+    def _request_handlers(self):
+        return dict((handler.__class__.__name__, handler)
+                    for handler in self.handlers)
+
+    def _get_valid_default_handler(self, req):
+        # Use default_handler from the Session if it is a valid value.
+        name = req.session.get('default_handler')
+        handler = self._request_handlers.get(name)
+        if handler and not is_valid_default_handler(handler):
+            handler = None
+
+        if not handler:
+            # Use default_handler from project configuration.
+            handler = self.default_handler
+            if not is_valid_default_handler(handler):
+                raise ConfigurationError(
+                    tag_("%(handler)s is not a valid default handler. Please "
+                         "update %(option)s through the %(page)s page or by "
+                         "directly editing trac.ini.",
+                         handler=tag.code(handler.__class__.__name__),
+                         option=tag.code("[trac] default_handler"),
+                         page=tag.a(_("Basic Settings"),
+                                    href=req.href.admin('general/basics'))))
+        return handler
+
     def _get_perm(self, req):
         if isinstance(req.session, FakeSession):
             return FakePerm()
         else:
-            return PermissionCache(self.env, self.authenticate(req))
+            return PermissionCache(self.env, req.authname)
 
     def _get_session(self, req):
         try:
             return Session(self.env, req)
-        except TracError, e:
+        except TracError as e:
             self.log.error("can't retrieve session: %s",
                            exception_to_unicode(e))
             return FakeSession()
@@ -274,10 +356,11 @@ class RequestDispatcher(Component):
     def _get_locale(self, req):
         if has_babel:
             preferred = req.session.get('language')
-            default = self.env.config.get('trac', 'default_language', '')
+            default = self.default_language
             negotiated = get_negotiated_locale([preferred, default] +
                                                req.languages)
-            self.log.debug("Negotiated locale: %s -> %s", preferred, negotiated)
+            self.log.debug("Negotiated locale: %s -> %s", preferred,
+                           negotiated)
             return negotiated
 
     def _get_lc_time(self, req):
@@ -306,28 +389,46 @@ class RequestDispatcher(Component):
         If the the user does not have a `trac_form_token` cookie a new
         one is generated.
         """
-        if req.incookie.has_key('trac_form_token'):
+        if 'trac_form_token' in req.incookie:
             return req.incookie['trac_form_token'].value
         else:
             req.outcookie['trac_form_token'] = hex_entropy(24)
             req.outcookie['trac_form_token']['path'] = req.base_path or '/'
             if self.env.secure_cookies:
                 req.outcookie['trac_form_token']['secure'] = True
-            if sys.version_info >= (2, 6):
-                req.outcookie['trac_form_token']['httponly'] = True
+            req.outcookie['trac_form_token']['httponly'] = True
             return req.outcookie['trac_form_token'].value
 
     def _get_use_xsendfile(self, req):
         return self.use_xsendfile
 
+    # RFC7230 3.2 Header Fields
+    _xsendfile_header_re = re.compile(r"[-0-9A-Za-z!#$%&'*+.^_`|~]+\Z")
+    _warn_xsendfile_header = False
+
+    def _get_xsendfile_header(self, req):
+        header = self.xsendfile_header.strip()
+        if self._xsendfile_header_re.match(header):
+            return to_utf8(header)
+        else:
+            if not self._warn_xsendfile_header:
+                self._warn_xsendfile_header = True
+                self.log.warn("[trac] xsendfile_header is invalid: '%s'",
+                              header)
+            return None
+
     def _pre_process_request(self, req, chosen_handler):
         for filter_ in self.filters:
             chosen_handler = filter_.pre_process_request(req, chosen_handler)
         return chosen_handler
 
     def _post_process_request(self, req, *args):
-        nbargs = len(args)
         resp = args
+        # `method` is optional in IRequestHandler's response. If not
+        # specified, the default value is appended to response.
+        if len(resp) == 3:
+            resp += (None,)
+        nbargs = len(resp)
         for f in reversed(self.filters):
             # As the arity of `post_process_request` has changed since
             # Trac 0.10, only filters with same arity gets passed real values.
@@ -336,10 +437,18 @@ class RequestDispatcher(Component):
             extra_arg_count = arity(f.post_process_request) - 1
             if extra_arg_count == nbargs:
                 resp = f.post_process_request(req, *resp)
+            elif extra_arg_count == nbargs - 1:
+                # IRequestFilters may modify the `method`, but the `method`
+                # is forwarded when not accepted by the IRequestFilter.
+                method = resp[-1]
+                resp = f.post_process_request(req, *resp[:-1])
+                resp += (method,)
             elif nbargs == 0:
                 f.post_process_request(req, *(None,)*extra_arg_count)
         return resp
 
+
+_warn_setuptools = False
 _slashes_re = re.compile(r'/+')
 
 def dispatch_request(environ, start_response):
@@ -349,6 +458,14 @@ def dispatch_request(environ, start_response):
     :param start_response: the WSGI callback for starting the response
     """
 
+    global _warn_setuptools
+    if _warn_setuptools is False:
+        _warn_setuptools = True
+        warn_setuptools_issue(out=environ.get('wsgi.errors'))
+
+    if sys.flags.optimize != 0:
+        raise EnvironmentError("Python with optimizations is not supported.")
+
     # SCRIPT_URL is an Apache var containing the URL before URL rewriting
     # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
     script_url = environ.get('SCRIPT_URL')
@@ -454,9 +571,8 @@ def dispatch_request(environ, start_response):
                     'trac.web.version': mod_wsgi_version})
             env.webfrontend = environ.get('trac.web.frontend')
             if env.webfrontend:
-                env.systeminfo.append((env.webfrontend,
-                                       environ['trac.web.version']))
-    except Exception, e:
+                env.webfrontend_version = environ['trac.web.version']
+    except Exception as e:
         env_error = e
 
     req = RequestWithSession(environ, start_response)
@@ -487,7 +603,7 @@ def _dispatch_request(req, env, env_error):
 
     # fixup env.abs_href if `[trac] base_url` was not specified
     if env and not env.abs_href.base:
-        env._abs_href = req.abs_href
+        env.abs_href = req.abs_href
 
     try:
         if not env and env_error:
@@ -495,12 +611,12 @@ def _dispatch_request(req, env, env_error):
         try:
             dispatcher = RequestDispatcher(env)
             dispatcher.dispatch(req)
-        except RequestDone:
-            pass
-        resp = req._response or []
-    except HTTPException, e:
+        except RequestDone as req_done:
+            resp = req_done.iterable
+        resp = resp or req._response or []
+    except HTTPException as e:
         _send_user_error(req, env, e)
-    except Exception, e:
+    except Exception:
         send_internal_error(env, req, sys.exc_info())
     return resp
 
@@ -508,49 +624,29 @@ def _dispatch_request(req, env, env_error):
 def _send_user_error(req, env, e):
     # See trac/web/api.py for the definition of HTTPException subclasses.
     if env:
-        env.log.warn('[%s] %s' % (req.remote_addr, exception_to_unicode(e)))
-    try:
-        # We first try to get localized error messages here, but we
-        # should ignore secondary errors if the main error was also
-        # due to i18n issues
-        title = _('Error')
-        if e.reason:
-            if title.lower() in e.reason.lower():
-                title = e.reason
-            else:
-                title = _('Error: %(message)s', message=e.reason)
-    except Exception:
-        title = 'Error'
-    # The message is based on the e.detail, which can be an Exception
-    # object, but not a TracError one: when creating HTTPException,
-    # a TracError.message is directly assigned to e.detail
-    if isinstance(e.detail, Exception): # not a TracError
-        message = exception_to_unicode(e.detail)
-    elif isinstance(e.detail, Fragment): # markup coming from a TracError
-        message = e.detail
-    else:
-        message = to_unicode(e.detail)
-    data = {'title': title, 'type': 'TracError', 'message': message,
+        env.log.warn('[%s] %s', req.remote_addr, exception_to_unicode(e))
+    data = {'title': e.title, 'type': 'TracError', 'message': e.message,
             'frames': [], 'traceback': None}
     if e.code == 403 and req.authname == 'anonymous':
         # TRANSLATOR: ... not logged in, you may want to 'do so' now (link)
         do_so = tag.a(_("do so"), href=req.href.login())
-        req.chrome['notices'].append(
-            tag_("You are currently not logged in. You may want to "
-                 "%(do_so)s now.", do_so=do_so))
+        add_notice(req, tag_("You are currently not logged in. You may want "
+                             "to %(do_so)s now.", do_so=do_so))
     try:
         req.send_error(sys.exc_info(), status=e.code, env=env, data=data)
     except RequestDone:
         pass
 
+
 def send_internal_error(env, req, exc_info):
     if env:
-        env.log.error("Internal Server Error: %s",
+        env.log.error("Internal Server Error: %r, referrer %r%s",
+                      req, req.environ.get('HTTP_REFERER'),
                       exception_to_unicode(exc_info[1], traceback=True))
     message = exception_to_unicode(exc_info[1])
     traceback = get_last_traceback()
 
-    frames, plugins, faulty_plugins = [], [], []
+    frames, plugins, faulty_plugins, interface_custom = [], [], [], []
     th = 'http://trac-hacks.org'
     has_admin = False
     try:
@@ -559,6 +655,7 @@ def send_internal_error(env, req, exc_info):
         pass
 
     tracker = default_tracker
+    tracker_args = {}
     if has_admin and not isinstance(exc_info[1], MemoryError):
         # Collect frame and plugin information
         frames = get_frame_info(exc_info[2])
@@ -574,23 +671,36 @@ def send_internal_error(env, req, exc_info):
             faulty_plugins.sort(key=lambda p: p['frame_idx'])
             if faulty_plugins:
                 info = faulty_plugins[0]['info']
+                home_page = info.get('home_page', '')
                 if 'trac' in info:
                     tracker = info['trac']
-                elif info.get('home_page', '').startswith(th):
+                elif urlparse(home_page).netloc == urlparse(th).netloc:
                     tracker = th
+                    plugin_name = info.get('home_page', '').rstrip('/') \
+                                                           .split('/')[-1]
+                    tracker_args = {'component': plugin_name}
+        interface_custom = Chrome(env).get_interface_customization_files()
 
     def get_description(_):
         if env and has_admin:
             sys_info = "".join("|| '''`%s`''' || `%s` ||\n"
                                % (k, v.replace('\n', '` [[br]] `'))
                                for k, v in env.get_systeminfo())
-            sys_info += "|| '''`jQuery`''' || `#JQUERY#` ||\n"
+            sys_info += "|| '''`jQuery`''' || `#JQUERY#` ||\n" \
+                        "|| '''`jQuery UI`''' || `#JQUERYUI#` ||\n" \
+                        "|| '''`jQuery Timepicker`''' || `#JQUERYTP#` ||\n"
             enabled_plugins = "".join("|| '''`%s`''' || `%s` ||\n"
                                       % (p['name'], p['version'] or _('N/A'))
                                       for p in plugins)
+            files = Chrome(env).get_interface_customization_files().items()
+            interface_files = "".join("|| **%s** || %s ||\n"
+                                      % (k, ", ".join("`%s`" % f for f in v))
+                                      for k, v in sorted(files))
         else:
             sys_info = _("''System information not available''\n")
             enabled_plugins = _("''Plugin information not available''\n")
+            interface_files = _("''Interface customization information not "
+                                 "available''\n")
         return _("""\
 ==== How to Reproduce ====
 
@@ -609,12 +719,16 @@ User agent: `#USER_AGENT#`
 %(sys_info)s
 ==== Enabled Plugins ====
 %(enabled_plugins)s
+==== Interface Customization ====
+%(interface_customization)s
 ==== Python Traceback ====
 {{{
 %(traceback)s}}}""",
             method=req.method, path_info=req.path_info,
             req_args=pformat(req.args), sys_info=sys_info,
-            enabled_plugins=enabled_plugins, traceback=to_unicode(traceback))
+            enabled_plugins=enabled_plugins,
+            interface_customization=interface_files,
+            traceback=to_unicode(traceback))
 
     # Generate the description once in English, once in the current locale
     description_en = get_description(lambda s, **kw: safefmt(s, kw))
@@ -628,9 +742,11 @@ User agent: `#USER_AGENT#`
             'traceback': traceback, 'frames': frames,
             'shorten_line': shorten_line, 'repr': safe_repr,
             'plugins': plugins, 'faulty_plugins': faulty_plugins,
-            'tracker': tracker,
+            'interface': interface_custom,
+            'tracker': tracker, 'tracker_args': tracker_args,
             'description': description, 'description_en': description_en}
 
+    Chrome(env).add_jquery_ui(req)
     try:
         req.send_error(exc_info, status=500, env=env, data=data)
     except RequestDone:
@@ -669,7 +785,7 @@ def send_project_index(environ, start_response, parent_dir=None,
                     'description': env.project_description,
                     'href': href(env_name)
                 }
-            except Exception, e:
+            except Exception as e:
                 proj = {'name': env_name, 'description': to_unicode(e)}
             projects.append(proj)
         projects.sort(lambda x, y: cmp(x['name'].lower(), y['name'].lower()))
@@ -722,7 +838,7 @@ def get_environments(environ, warn=False):
         paths = [path[:-1] for path in paths if path[-1] == '/'
                  and not any(fnmatch.fnmatch(path[:-1], pattern)
                              for pattern in ignore_patterns)]
-        env_paths.extend(os.path.join(env_parent_dir, project) \
+        env_paths.extend(os.path.join(env_parent_dir, project)
                          for project in paths)
     envs = {}
     for env_path in env_paths:
@@ -732,9 +848,9 @@ def get_environments(environ, warn=False):
         env_name = os.path.split(env_path)[1]
         if env_name in envs:
             if warn:
-                print >> sys.stderr, ('Warning: Ignoring project "%s" since '
-                                      'it conflicts with project "%s"'
-                                      % (env_path, envs[env_name]))
+                print('Warning: Ignoring project "%s" since it conflicts with'
+                      ' project "%s"' % (env_path, envs[env_name]),
+                      file=sys.stderr)
         else:
             envs[env_name] = env_path
     return envs
