@@ -19,10 +19,11 @@
 #         Matthew Good <trac@matt-good.net>
 #         Christopher Lenz <cmlenz@gmx.de>
 
+from __future__ import print_function
+
 import pkg_resources
 import os
 import socket
-import select
 import sys
 from SocketServer import ThreadingMixIn
 
@@ -76,8 +77,7 @@ class TracEnvironMiddleware(object):
 
     def __init__(self, application, env_parent_dir, env_paths, single_env):
         self.application = application
-        self.environ = {}
-        self.environ['trac.env_path'] = None
+        self.environ = {'trac.env_path': None}
         if env_parent_dir:
             self.environ['trac.env_parent_dir'] = env_parent_dir
         elif single_env:
@@ -100,13 +100,6 @@ class TracHTTPServer(ThreadingMixIn, WSGIServer):
         WSGIServer.__init__(self, server_address, application,
                             request_handler=request_handlers[bool(use_http_11)])
 
-    if sys.version_info < (2, 6):
-        def serve_forever(self, poll_interval=0.5):
-            while True:
-                r, w, e = select.select([self], [], [], poll_interval)
-                if self in r:
-                    self.handle_request()
-
 
 class TracHTTPRequestHandler(WSGIRequestHandler):
 
@@ -135,8 +128,8 @@ def main():
 
         env_name, filename, realm = info
         if env_name in auths:
-            print >> sys.stderr, 'Ignoring duplicate authentication option ' \
-                                 'for project: %s' % env_name
+            print('Ignoring duplicate authentication option for project: %s'
+                  % env_name, file=sys.stderr)
         else:
             auths[env_name] = cls(os.path.abspath(filename), realm)
 
@@ -296,15 +289,15 @@ def main():
                 httpd = TracHTTPServer(server_address, wsgi_app,
                                        options.env_parent_dir, args,
                                        use_http_11=options.http11)
-            except socket.error, e:
-                print 'Error starting Trac server on %s' % loc
-                print '[Errno %s] %s' % e.args
+            except socket.error as e:
+                print("Error starting Trac server on %s" % loc)
+                print("[Errno %s] %s" % e.args)
                 sys.exit(1)
 
-            print 'Server starting in PID %i.' % os.getpid()
-            print 'Serving on %s' % loc
+            print("Server starting in PID %s." % os.getpid())
+            print("Serving on %s" % loc)
             if options.http11:
-                print 'Using HTTP/1.1 protocol version'
+                print("Using HTTP/1.1 protocol version")
             httpd.serve_forever()
     elif options.protocol in ('scgi', 'ajp', 'fcgi'):
         def serve():
@@ -328,14 +321,14 @@ def main():
 
         if options.autoreload:
             def modification_callback(file):
-                print >> sys.stderr, 'Detected modification of %s, ' \
-                                     'restarting.' % file
+                print("Detected modification of %s, restarting." % file,
+                      file=sys.stderr)
             autoreload.main(serve, modification_callback)
         else:
             serve()
 
-    except OSError, e:
-        print >> sys.stderr, '%s: %s' % (e.__class__.__name__, e)
+    except OSError as e:
+        print("%s: %s" % (e.__class__.__name__, e), file=sys.stderr)
         sys.exit(1)
     except KeyboardInterrupt:
         pass
