@@ -28,6 +28,7 @@ from trac.util.datefmt import utc
 from trac.util.text import shorten_line
 from trac.web.api import HTTPBadRequest, HTTPInternalError, Request, \
                          RequestDone, parse_arg_list
+from trac.web.main import FakeSession
 from tracopt.perm.authz_policy import AuthzPolicy
 
 
@@ -77,7 +78,7 @@ def _make_environ(scheme='http', server_name='example.org',
 
 def _make_req(environ, start_response, args={}, arg_list=(), authname='admin',
               form_token='A' * 40, chrome={'links': {}, 'scripts': []},
-              perm=MockPerm(), session={}, tz=utc, locale=None, **kwargs):
+              perm=MockPerm(), tz=utc, locale=None, **kwargs):
     req = Request(environ, start_response)
     req.args = args
     req.arg_list = arg_list
@@ -85,7 +86,7 @@ def _make_req(environ, start_response, args={}, arg_list=(), authname='admin',
     req.form_token = form_token
     req.chrome = chrome
     req.perm = perm
-    req.session = session
+    req.session = FakeSession()
     req.tz = tz
     req.locale = locale
     for name, value in kwargs.iteritems():
@@ -95,48 +96,131 @@ def _make_req(environ, start_response, args={}, arg_list=(), authname='admin',
 
 class RequestTestCase(unittest.TestCase):
 
-    def _make_environ(self, *args, **kwargs):
-        return _make_environ(*args, **kwargs)
+    def test_as_bool(self):
+        qs = 'arg1=0&arg2=1&arg3=yes&arg4=a&arg5=1&arg5=0'
+        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        req = Request(environ, None)
+
+        self.assertIsNone(req.args.as_bool('arg0'))
+        self.assertTrue(req.args.as_bool('arg0', True))
+        self.assertFalse(req.args.as_bool('arg1'))
+        self.assertFalse(req.args.as_bool('arg1', True))
+        self.assertTrue(req.args.as_bool('arg2'))
+        self.assertTrue(req.args.as_bool('arg3'))
+        self.assertFalse(req.args.as_bool('arg4'))
+        self.assertTrue(req.args.as_bool('arg4', True))
+        self.assertTrue(req.args.as_bool('arg5'))
+
+    def test_as_int(self):
+        qs = 'arg1=1&arg2=a&arg3=3&arg3=4'
+        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        req = Request(environ, None)
+
+        self.assertIsNone(req.args.as_int('arg0'))
+        self.assertEqual(2, req.args.as_int('arg0', 2))
+        self.assertEqual(1, req.args.as_int('arg1'))
+        self.assertEqual(1, req.args.as_int('arg1', 2))
+        self.assertEqual(2, req.args.as_int('arg1', min=2))
+        self.assertEqual(2, req.args.as_int('arg1', None, 2))
+        self.assertEqual(0, req.args.as_int('arg1', max=0))
+        self.assertEqual(0, req.args.as_int('arg1', None, max=0))
+        self.assertEqual(0, req.args.as_int('arg1', None, -1, 0))
+        self.assertIsNone(req.args.as_int('arg2'))
+        self.assertEqual(2, req.args.as_int('arg2', 2))
+        self.assertEqual(3, req.args.as_int('arg3'))
+
+    def test_getbool(self):
+        qs = 'arg1=0&arg2=1&arg3=yes&arg4=a&arg5=1&arg5=0'
+        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        req = Request(environ, None)
+
+        self.assertIsNone(req.args.getbool('arg0'))
+        self.assertTrue(req.args.getbool('arg0', True))
+        self.assertFalse(req.args.getbool('arg1'))
+        self.assertFalse(req.args.getbool('arg1', True))
+        self.assertTrue(req.args.getbool('arg2'))
+        self.assertTrue(req.args.getbool('arg3'))
+        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg4')
+        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg4', True)
+        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg5')
+        self.assertRaises(HTTPBadRequest, req.args.getbool, 'arg5', True)
+
+    def test_getint(self):
+        qs = 'arg1=1&arg2=a&arg3=3&arg3=4'
+        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        req = Request(environ, None)
+
+        self.assertIsNone(req.args.getint('arg0'))
+        self.assertEqual(2, req.args.getint('arg0', 2))
+        self.assertEqual(1, req.args.getint('arg1'))
+        self.assertEqual(1, req.args.getint('arg1', 2))
+        self.assertEqual(2, req.args.getint('arg1', min=2))
+        self.assertEqual(2, req.args.getint('arg1', None, 2))
+        self.assertEqual(0, req.args.getint('arg1', max=0))
+        self.assertEqual(0, req.args.getint('arg1', None, max=0))
+        self.assertEqual(0, req.args.getint('arg1', None, -1, 0))
+        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg2')
+        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg2', 2)
+        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg3')
+        self.assertRaises(HTTPBadRequest, req.args.getint, 'arg3', 2)
+
+    def test_require(self):
+        qs = 'arg1=1'
+        environ = _make_environ(method='GET', **{'QUERY_STRING': qs})
+        req = Request(environ, None)
+
+        self.assertRaises(HTTPBadRequest, req.args.require, 'arg0')
+        self.assertIsNone(req.args.require('arg1'))
+
+    def test_is_xhr_true(self):
+        environ = _make_environ(HTTP_X_REQUESTED_WITH='XMLHttpRequest')
+        req = Request(environ, None)
+        self.assertTrue(req.is_xhr)
+
+    def test_is_xhr_false(self):
+        environ = _make_environ()
+        req = Request(environ, None)
+        self.assertFalse(req.is_xhr)
 
     def test_base_url(self):
-        environ = self._make_environ()
+        environ = _make_environ()
         req = Request(environ, None)
         self.assertEqual('http://example.org/trac', req.base_url)
 
     def test_base_url_host(self):
-        environ = self._make_environ(server_port=8080, HTTP_HOST='example.com')
+        environ = _make_environ(server_port=8080, HTTP_HOST='example.com')
         req = Request(environ, None)
         self.assertEqual('http://example.com/trac', req.base_url)
 
     def test_base_url_nondefaultport(self):
-        environ = self._make_environ(server_port=8080)
+        environ = _make_environ(server_port=8080)
         req = Request(environ, None)
         self.assertEqual('http://example.org:8080/trac', req.base_url)
 
     def test_base_url_https(self):
-        environ = self._make_environ(scheme='https', server_port=443)
+        environ = _make_environ(scheme='https', server_port=443)
         req = Request(environ, None)
         self.assertEqual('https://example.org/trac', req.base_url)
 
     def test_base_url_https_host(self):
-        environ = self._make_environ(scheme='https', server_port=443,
+        environ = _make_environ(scheme='https', server_port=443,
                                      HTTP_HOST='example.com')
         req = Request(environ, None)
         self.assertEqual('https://example.com/trac', req.base_url)
 
     def test_base_url_https_nondefaultport(self):
-        environ = self._make_environ(scheme='https', server_port=8443)
+        environ = _make_environ(scheme='https', server_port=8443)
         req = Request(environ, None)
         self.assertEqual('https://example.org:8443/trac', req.base_url)
 
     def test_base_url_proxy(self):
-        environ = self._make_environ(HTTP_HOST='localhost',
+        environ = _make_environ(HTTP_HOST='localhost',
                                      HTTP_X_FORWARDED_HOST='example.com')
         req = Request(environ, None)
         self.assertEqual('http://localhost/trac', req.base_url)
 
     def test_languages(self):
-        environ = self._make_environ()
+        environ = _make_environ()
         environ['HTTP_ACCEPT_LANGUAGE'] = 'en-us,en;q=0.5'
         req = Request(environ, None)
         self.assertEqual(['en-us', 'en'], req.languages)
@@ -147,7 +231,7 @@ class RequestTestCase(unittest.TestCase):
         def start_response(status, headers):
             status_sent.append(status)
             headers_sent.update(dict(headers))
-        environ = self._make_environ(method='HEAD')
+        environ = _make_environ(method='HEAD')
         req = Request(environ, start_response)
         req.session = Mock(save=lambda: None)
         self.assertRaises(RequestDone, req.redirect, '/trac/test')
@@ -161,7 +245,7 @@ class RequestTestCase(unittest.TestCase):
         def start_response(status, headers):
             status_sent.append(status)
             headers_sent.update(dict(headers))
-        environ = self._make_environ(method='HEAD')
+        environ = _make_environ(method='HEAD')
         req = Request(environ, start_response,)
         req.session = Mock(save=lambda: None)
         self.assertRaises(RequestDone, req.redirect,
@@ -180,7 +264,7 @@ class RequestTestCase(unittest.TestCase):
             def start_response(status, headers):
                 status_sent.append(status)
                 headers_sent.update(dict(headers))
-            environ = self._make_environ(method='POST', HTTP_USER_AGENT=ua)
+            environ = _make_environ(method='POST', HTTP_USER_AGENT=ua)
             req = Request(environ, start_response,)
             req.session = Mock(save=lambda: None)
             self.assertRaises(RequestDone, req.redirect, url)
@@ -225,7 +309,7 @@ class RequestTestCase(unittest.TestCase):
             buf.write(data)
         def start_response(status, headers):
             return write
-        environ = self._make_environ(method='GET')
+        environ = _make_environ(method='GET')
 
         buf = StringIO()
         req = Request(environ, start_response)
@@ -239,7 +323,7 @@ class RequestTestCase(unittest.TestCase):
             buf.write(data)
         def start_response(status, headers):
             return write
-        environ = self._make_environ(method='HEAD')
+        environ = _make_environ(method='HEAD')
 
         req = Request(environ, start_response)
         req.send_header('Content-Type', 'text/plain;charset=utf-8')
@@ -256,7 +340,7 @@ class RequestTestCase(unittest.TestCase):
             baton['status'] = status
             baton['headers'] = headers
             return write
-        environ = self._make_environ(method='GET')
+        environ = _make_environ(method='GET')
 
         def iterable():
             yield 'line1,'
@@ -274,23 +358,23 @@ class RequestTestCase(unittest.TestCase):
         self.assertEqual('line1,line2,line3\n', baton['content'].getvalue())
 
     def test_invalid_cookies(self):
-        environ = self._make_environ(HTTP_COOKIE='bad:key=value;')
+        environ = _make_environ(HTTP_COOKIE='bad:key=value;')
         req = Request(environ, None)
         self.assertEqual('', str(req.incookie))
 
     def test_multiple_cookies(self):
-        environ = self._make_environ(HTTP_COOKIE='key=value1; key=value2;')
+        environ = _make_environ(HTTP_COOKIE='key=value1; key=value2;')
         req = Request(environ, None)
         self.assertEqual('Set-Cookie: key=value1',
                          str(req.incookie).rstrip(';'))
 
     def test_read(self):
-        environ = self._make_environ(**{'wsgi.input': StringIO('test input')})
+        environ = _make_environ(**{'wsgi.input': StringIO('test input')})
         req = Request(environ, None)
         self.assertEqual('test input', req.read())
 
     def test_read_size(self):
-        environ = self._make_environ(**{'wsgi.input': StringIO('test input')})
+        environ = _make_environ(**{'wsgi.input': StringIO('test input')})
         req = Request(environ, None)
         self.assertEqual('test', req.read(size=4))
 
@@ -298,19 +382,19 @@ class RequestTestCase(unittest.TestCase):
         req = Request(environ, None)
         try:
             req.args['action']
-        except HTTPBadRequest, e:
+        except HTTPBadRequest as e:
             self.assertEqual("400 Bad Request (Invalid request arguments.)",
                              unicode(e))
         else:
             self.fail("HTTPBadRequest not raised.")
 
     def test_qs_with_null_bytes_for_name(self):
-        environ = self._make_environ(method='GET',
+        environ = _make_environ(method='GET',
                                      **{'QUERY_STRING': 'acti\x00n=fOO'})
         self._test_qs_with_null_bytes(environ)
 
     def test_qs_with_null_bytes_for_value(self):
-        environ = self._make_environ(method='GET',
+        environ = _make_environ(method='GET',
                                      **{'QUERY_STRING': 'action=f\x00O'})
         self._test_qs_with_null_bytes(environ)
 
@@ -333,7 +417,7 @@ unnamed value\r\n\
 """
         form_data %= {'boundary': boundary}
         content_type = 'multipart/form-data; boundary="%s"' % boundary
-        environ = self._make_environ(method='POST', **{
+        environ = _make_environ(method='POST', **{
             'wsgi.input': StringIO(form_data),
             'CONTENT_LENGTH': str(len(form_data)),
             'CONTENT_TYPE': content_type
@@ -349,7 +433,7 @@ unnamed value\r\n\
         content_type = 'multipart/form-data; boundary="%s"' % boundary
         form_data %= {'boundary': boundary}
 
-        environ = self._make_environ(method='POST', **{
+        environ = _make_environ(method='POST', **{
             'wsgi.input': StringIO(form_data),
             'CONTENT_LENGTH': str(len(form_data)),
             'CONTENT_TYPE': content_type
@@ -358,7 +442,7 @@ unnamed value\r\n\
 
         try:
             req.args['action']
-        except HTTPBadRequest, e:
+        except HTTPBadRequest as e:
             self.assertEqual("400 Bad Request (Invalid request arguments.)",
                              unicode(e))
         else:
@@ -404,11 +488,11 @@ ne\x00w\r\n\
         """Make sure req.args parsing is consistent even after the backwards
         incompatible change introduced in Python 2.6.
         """
-        environ = self._make_environ(method='GET',
+        environ = _make_environ(method='GET',
                                      **{'QUERY_STRING': 'action=foo'})
         req = Request(environ, None)
         self.assertEqual('foo', req.args['action'])
-        environ = self._make_environ(method='POST',
+        environ = _make_environ(method='POST',
                                      **{'wsgi.input': StringIO('action=bar'),
                                         'CONTENT_LENGTH': '10',
                                         'CONTENT_TYPE': 'application/x-www-form-urlencoded',
@@ -417,12 +501,12 @@ ne\x00w\r\n\
         self.assertEqual('bar', req.args['action'])
 
     def test_qs_invalid_value_bytes(self):
-        environ = self._make_environ(**{'QUERY_STRING': 'name=%FF'})
+        environ = _make_environ(**{'QUERY_STRING': 'name=%FF'})
         req = Request(environ, None)
         self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
 
     def test_qs_invalid_name_bytes(self):
-        environ = self._make_environ(**{'QUERY_STRING': '%FF=value'})
+        environ = _make_environ(**{'QUERY_STRING': '%FF=value'})
         req = Request(environ, None)
         self.assertRaises(HTTPBadRequest, lambda: req.arg_list)
 
@@ -627,6 +711,16 @@ class ParseArgListTestCase(unittest.TestCase):
         self.assertTrue(unicode, type(args[1][0]))
         self.assertEqual(u'résumé', args[1][0])
 
+    def test_qs_str_with_prefix(self):
+        """The leading `?` should be stripped from the query string."""
+        args = parse_arg_list('?k%C3%A9y=resum%C3%A9&r%C3%A9sum%C3%A9')
+        self.assertTrue(unicode, type(args[0][0]))
+        self.assertTrue(unicode, type(args[0][1]))
+        self.assertEqual(u'kéy', args[0][0])
+        self.assertEqual(u'resumé', args[0][1])
+        self.assertTrue(unicode, type(args[1][0]))
+        self.assertEqual(u'résumé', args[1][0])
+
     def test_qs_unicode(self):
         args = parse_arg_list(u'ké%3Dy=re%26su=mé&résu%26mé')
         self.assertTrue(unicode, type(args[0][0]))
@@ -662,7 +756,7 @@ class HTTPExceptionTestCase(unittest.TestCase):
                          unicode(e2))
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
     suite.addTest(unittest.makeSuite(RequestTestCase))
     suite.addTest(unittest.makeSuite(RequestSendFileTestCase))
@@ -673,4 +767,4 @@ def suite():
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
