@@ -1,14 +1,28 @@
-from __future__ import with_statement
+# -*- coding: utf-8 -*-
+#
+# Copyright (C) 2005-2013 Edgewall Software
+# All rights reserved.
+#
+# This software is licensed as described in the file COPYING, which
+# you should have received as part of this distribution. The terms
+# are also available at http://trac.edgewall.org/wiki/TracLicense.
+#
+# This software consists of voluntary contributions made by many
+# individuals. For the exact contribution history, see the revision
+# history and logs, available at http://trac.edgewall.org/log/.
 
-from Cookie import SimpleCookie as Cookie
 import time
 from datetime import datetime
 import unittest
 
-from trac.test import EnvironmentStub, Mock
+import trac.tests.compat
+from trac.admin.api import console_date_format
+from trac.test import EnvironmentStub, MockRequest
+from trac.util.datefmt import format_date, to_datetime
 from trac.web.session import DetachedSession, Session, PURGE_AGE, \
                              UPDATE_INTERVAL, SessionAdmin
 from trac.core import TracError
+from trac.util.datefmt import time_now
 
 
 def _prep_session_table(env, spread_visits=False):
@@ -29,7 +43,9 @@ def _prep_session_table(env, spread_visits=False):
             authenticated = int(x < 10)
             last_visit = last_visit_base + (visit_delta * x)
             val = 'val%02d' % x
-            data = (sid, authenticated, last_visit, val, val)
+            data = (sid, authenticated,
+                    format_date(to_datetime(last_visit), console_date_format),
+                    val, val, None)
             if authenticated:
                 auth_list.append(data)
             else:
@@ -41,19 +57,27 @@ def _prep_session_table(env, spread_visits=False):
             db("INSERT INTO session_attribute VALUES (%s, %s, 'email', %s)",
                (sid, authenticated, val))
     all_list = auth_list + anon_list
-    return (auth_list, anon_list, all_list)
+    return auth_list, anon_list, all_list
+
 
 def get_session_info(env, sid):
-    """:since 1.0: changed `db` input parameter to `env`"""
+    """
+    :since 1.0: changed `db` input parameter to `env`
+    :since 1.1.2: returns `default_handler` as the fourth entry in the tuple.
+    """
     for row in env.db_query("""
-            SELECT DISTINCT s.sid, n.value, e.value FROM session AS s
-            LEFT JOIN session_attribute AS n ON (n.sid=s.sid AND n.name='name')
-            LEFT JOIN session_attribute AS e ON (e.sid=s.sid AND e.name='email')
+            SELECT DISTINCT s.sid, n.value, e.value, h.value FROM session AS s
+            LEFT JOIN session_attribute AS n
+              ON (n.sid=s.sid AND n.name='name')
+            LEFT JOIN session_attribute AS e
+              ON (e.sid=s.sid AND e.name='email')
+            LEFT JOIN session_attribute AS h
+              ON (h.sid=s.sid AND h.name='default_handler')
             WHERE s.sid=%s
             """, (sid,)):
         return row
     else:
-        return (None, None, None)
+        return None, None, None, None
 
 
 class SessionTestCase(unittest.TestCase):
@@ -70,11 +94,9 @@ class SessionTestCase(unittest.TestCase):
         Verify that a session cookie gets sent back to the client for a new
         session.
         """
-        cookie = Cookie()
-        req = Mock(incookie=Cookie(), outcookie=cookie, authname='anonymous',
-                   base_path='/')
+        req = MockRequest(self.env, authname='anonymous')
         session = Session(self.env, req)
-        self.assertEqual(session.sid, cookie['trac_session'].value)
+        self.assertEqual(session.sid, req.outcookie['trac_session'].value)
         self.assertEqual(0, self.env.db_query(
                 "SELECT COUNT(*) FROM session")[0][0])
 
@@ -82,78 +104,127 @@ class SessionTestCase(unittest.TestCase):
         """
         Verify that session variables are stored in the database.
         """
-        incookie = Cookie()
-        incookie['trac_session'] = '123456'
-        outcookie = Cookie()
-        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                   outcookie=outcookie)
+        req = MockRequest(self.env, authname='anonymous')
+        req.incookie['trac_session'] = '123456'
         session = Session(self.env, req)
-        self.assertEquals('123456', session.sid)
-        self.failIf(outcookie.has_key('trac_session'))
+        self.assertEqual('123456', session.sid)
+        self.assertNotIn('trac_session', req.outcookie)
 
-    def test_authenticated_session(self):
+    def _test_authenticated_session(self, username):
         """
         Verifies that a session cookie does not get used if the user is logged
         in, and that Trac expires the cookie.
         """
-        incookie = Cookie()
-        incookie['trac_session'] = '123456'
-        outcookie = Cookie()
-        req = Mock(authname='john', base_path='/', incookie=incookie,
-                   outcookie=outcookie)
+        req = MockRequest(self.env, authname=username)
+        req.incookie['trac_session'] = '123456'
         session = Session(self.env, req)
-        self.assertEqual('john', session.sid)
+        self.assertEqual(username, session.sid)
         session['foo'] = 'bar'
         session.save()
-        self.assertEquals(0, outcookie['trac_session']['expires'])
+        self.assertEqual(0, req.outcookie['trac_session']['expires'])
 
-    def test_session_promotion(self):
+    def test_authenticated_session(self):
+        self._test_authenticated_session('john')
+        self._test_authenticated_session('j.smith')
+        self._test_authenticated_session(u'Jöhn')  # non-ascii username
+        self._test_authenticated_session('john@EXAMPLE.LOCAL')  # LDAP username
+
+    def _test_session_promotion(self, username):
         """
         Verifies that an existing anonymous session gets promoted to an
         authenticated session when the user logs in.
         """
         with self.env.db_transaction as db:
             db("INSERT INTO session VALUES ('123456', 0, 0)")
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            outcookie = Cookie()
-            req = Mock(authname='john', base_path='/', incookie=incookie,
-                       outcookie=outcookie)
+            req = MockRequest(self.env, authname=username)
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
-            self.assertEqual('john', session.sid)
+            self.assertEqual(username, session.sid)
             session.save()
 
-        self.assertEqual([('john', 1)], self.env.db_query(
-            "SELECT sid, authenticated FROM session"))
+        self.assertEqual([(username, 1)],
+            self.env.db_query("""SELECT sid, authenticated FROM session
+                                 WHERE sid=%s""", (username,)))
 
-    def test_new_session_promotion(self):
+    def test_session_promotion(self):
+        self._test_session_promotion('john')
+        self._test_session_promotion('j.smith')
+        self._test_session_promotion(u'Jöhn')  # non-ascii username
+        self._test_session_promotion('john@EXAMPLE.LOCAL')  # LDAP username
+
+        sessions = self.env.db_query("SELECT sid, authenticated FROM session")
+        self.assertEqual(set([('john', 1), ('j.smith', 1), (u'Jöhn', 1),
+                              ('john@EXAMPLE.LOCAL', 1)]),
+                         set(sessions))
+
+    def _test_new_session_promotion(self, username):
         """
         Verifies that even without a preexisting anonymous session,
         an authenticated session will be created when the user logs in.
         (same test as above without the initial INSERT)
         """
-        with self.env.db_transaction as db:
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            outcookie = Cookie()
-            req = Mock(authname='john', base_path='/', incookie=incookie,
-                       outcookie=outcookie)
+        with self.env.db_transaction:
+            req = MockRequest(self.env, authname=username)
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
-            self.assertEqual('john', session.sid)
+            self.assertEqual(username, session.sid)
             session.save()
 
-        self.assertEqual([('john', 1)], self.env.db_query(
-                "SELECT sid, authenticated FROM session"))
+        self.assertEqual([(username, 1)],
+            self.env.db_query("""SELECT sid, authenticated FROM session
+                                 WHERE sid=%s""", (username,)))
+
+    def test_new_session_promotion(self):
+        self._test_new_session_promotion('john')
+        self._test_new_session_promotion('j.smith')
+        self._test_new_session_promotion(u'Jöhn')  # non-ascii username
+        self._test_new_session_promotion('john@EXAMPLE.LOCAL')  # LDAP username
+
+        sessions = self.env.db_query("SELECT sid, authenticated FROM session")
+        self.assertEqual(set([('john', 1), ('j.smith', 1), (u'Jöhn', 1),
+                              ('john@EXAMPLE.LOCAL', 1)]),
+                         set(sessions))
+
+    def test_as_int(self):
+        with self.env.db_transaction as db:
+            db("INSERT INTO session VALUES ('123456', 1, 0)")
+            db.executemany("""
+                INSERT INTO session_attribute VALUES (%s,%s,%s,%s)
+                """, (('123456', 1, 'foo', 'bar'), ('123456', 1, 'baz', 3)))
+
+        req = MockRequest(self.env, authname='123456')
+        session = Session(self.env, req)
+
+        self.assertEqual('bar', session.get('foo'))
+        self.assertEqual(2, session.as_int('foo', 2))
+        self.assertEqual(3, session.as_int('baz', 1))
+        self.assertEqual(2, session.as_int('baz', 1, max=2))
+        self.assertEqual(4, session.as_int('baz', 1, min=4))
+        self.assertIsNone(session.as_int('bat'))
+
+    def test_as_bool(self):
+        with self.env.db_transaction as db:
+            db("INSERT INTO session VALUES ('123456', 1, 0)")
+            db.executemany("""
+                INSERT INTO session_attribute VALUES (%s,%s,%s,%s)
+                """, (('123456', 1, 'foo', 'bar'), ('123456', 1, 'baz', 1)))
+
+        req = MockRequest(self.env, authname='123456')
+        session = Session(self.env, req)
+
+        self.assertEqual('bar', session.get('foo'))
+        self.assertIsNone(session.as_bool('foo', None))
+        self.assertTrue(session.as_int('baz'))
+        self.assertTrue(session.as_int('baz', False))
+        self.assertIsNone(session.as_bool('bat'))
 
     def test_add_anonymous_session_var(self):
         """
         Verify that new variables are inserted into the 'session' table in the
         database for an anonymous session.
         """
-        incookie = Cookie()
-        incookie['trac_session'] = '123456'
-        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                   outcookie=Cookie())
+        req = MockRequest(self.env, authname='anonymous')
+        req.incookie['trac_session'] = '123456'
         session = Session(self.env, req)
         session['foo'] = 'bar'
         session.save()
@@ -172,10 +243,8 @@ class SessionTestCase(unittest.TestCase):
                 INSERT INTO session_attribute VALUES
                 ('123456', 0, 'foo', 'bar')
                 """)
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                       outcookie=Cookie())
+            req = MockRequest(self.env, authname='anonymous')
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
             self.assertEqual('bar', session['foo'])
             session['foo'] = 'baz'
@@ -195,10 +264,8 @@ class SessionTestCase(unittest.TestCase):
                 INSERT INTO session_attribute VALUES
                 ('123456', 0, 'foo', 'bar')
                 """)
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                       outcookie=Cookie())
+            req = MockRequest(self.env, authname='anonymous')
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
             self.assertEqual('bar', session['foo'])
             del session['foo']
@@ -216,17 +283,15 @@ class SessionTestCase(unittest.TestCase):
         with self.env.db_transaction as db:
             db("INSERT INTO session VALUES ('123456', 0, %s)", (0,))
             db("INSERT INTO session VALUES ('987654', 0, %s)",
-               (int(time.time() - PURGE_AGE - 3600),))
+               (int(time_now() - PURGE_AGE - 3600),))
             db("""
                 INSERT INTO session_attribute
                 VALUES ('987654', 0, 'foo', 'bar')
                 """)
 
             # We need to modify a different session to trigger the purging
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                       outcookie=Cookie())
+            req = MockRequest(self.env, authname='anonymous')
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
             session['foo'] = 'bar'
             session.save()
@@ -240,7 +305,7 @@ class SessionTestCase(unittest.TestCase):
         Verify that a session gets deleted when it doesn't have any data except
         for the 'last_visit' timestamp.
         """
-        now = time.time()
+        now = time_now()
 
         # Make sure the session has data so that it doesn't get dropped
         with self.env.db_transaction as db:
@@ -251,10 +316,8 @@ class SessionTestCase(unittest.TestCase):
                 VALUES ('123456', 0, 'foo', 'bar')
                 """)
 
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                       outcookie=Cookie())
+            req = MockRequest(self.env, authname='anonymous')
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
             del session['foo']
             session.save()
@@ -277,10 +340,8 @@ class SessionTestCase(unittest.TestCase):
                 VALUES ('123456', 0, 'foo', 'bar')
                 """)
 
-        incookie = Cookie()
-        incookie['trac_session'] = '123456'
-        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                   outcookie=Cookie())
+        req = MockRequest(self.env, authname='anonymous')
+        req.incookie['trac_session'] = '123456'
         session = Session(self.env, req)
         self.assertEqual({'foo': 'bar'}, session)
 
@@ -304,7 +365,7 @@ class SessionTestCase(unittest.TestCase):
         Verify that new variables are inserted into the 'session' table in the
         database for an authenticated session.
         """
-        req = Mock(authname='john', base_path='/', incookie=Cookie())
+        req = MockRequest(self.env, authname='john')
         session = Session(self.env, req)
         session['foo'] = 'bar'
         session.save()
@@ -322,7 +383,7 @@ class SessionTestCase(unittest.TestCase):
             db("INSERT INTO session VALUES ('john', 1, 0)")
             db("INSERT INTO session_attribute VALUES ('john',1,'foo','bar')")
 
-            req = Mock(authname='john', base_path='/', incookie=Cookie())
+            req = MockRequest(self.env, authname='john')
             session = Session(self.env, req)
             self.assertEqual('bar', session['foo'])
             session['foo'] = 'baz'
@@ -346,12 +407,10 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='john' AND authenticated=1 AND name='foo'
             """)[0][0])
 
-        incookie = Cookie()
-        incookie['trac_session'] = 'john'
-        req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                   outcookie=Cookie())
+        req = MockRequest(self.env, authname='anonymous')
+        req.incookie['trac_session'] = 'john'
         session = Session(self.env, req)
-        self.assert_('foo' not in session)
+        self.assertTrue('foo' not in session)
         session['foo'] = 'baz'
         session.save()
 
@@ -377,7 +436,7 @@ class SessionTestCase(unittest.TestCase):
             db("INSERT INTO session VALUES ('john', 1, 0)")
             db("INSERT INTO session_attribute VALUES ('john', 1, 'foo', 'bar')")
 
-            req = Mock(authname='john', base_path='/', incookie=Cookie())
+            req = MockRequest(self.env, authname='john')
             session = Session(self.env, req)
             self.assertEqual('bar', session['foo'])
             del session['foo']
@@ -393,7 +452,7 @@ class SessionTestCase(unittest.TestCase):
         Verify that accessing a session after one day updates the sessions
         'last_visit' variable so that the session doesn't get purged.
         """
-        now = time.time()
+        now = time_now()
 
         # Make sure the session has data so that it doesn't get dropped
         with self.env.db_transaction as db:
@@ -403,16 +462,13 @@ class SessionTestCase(unittest.TestCase):
                 VALUES ('123456', 0, 'foo', 'bar')
                 """)
 
-            incookie = Cookie()
-            incookie['trac_session'] = '123456'
-            outcookie = Cookie()
-            req = Mock(authname='anonymous', base_path='/', incookie=incookie,
-                       outcookie=outcookie)
+            req = MockRequest(self.env, authname='anonymous')
+            req.incookie['trac_session'] = '123456'
             session = Session(self.env, req)
             session['modified'] = True
             session.save() # updating does require modifications
 
-            self.assertEqual(PURGE_AGE, outcookie['trac_session']['expires'])
+            self.assertEqual(PURGE_AGE, req.outcookie['trac_session']['expires'])
 
         self.assertAlmostEqual(now, int(self.env.db_query("""
             SELECT last_visit FROM session
@@ -485,6 +541,32 @@ class SessionTestCase(unittest.TestCase):
             WHERE sid='john' AND name='foo'
             """)[0][0])
 
+    def test_session_set_email(self):
+        """Setting session email invalidates known_users cache."""
+        session = DetachedSession(self.env, 'user')
+        session['email'] = 'user@domain.org'
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        session.save()
+        email = None
+        for sid, name, email in self.env.get_known_users():
+            if sid == 'user':
+                break
+        self.assertEqual(session['email'], email)
+
+    def test_session_set_name(self):
+        """Setting session name invalidates known_users cache."""
+        session = DetachedSession(self.env, 'user')
+        session['name'] = 'The User'
+
+        self.assertEqual([], list(self.env.get_known_users()))
+        session.save()
+        name = None
+        for sid, name, email in self.env.get_known_users():
+            if sid == 'user':
+                break
+        self.assertEqual(session['name'], name)
+
     def test_session_admin_list(self):
         auth_list, anon_list, all_list = _prep_session_table(self.env)
         sess_admin = SessionAdmin(self.env)
@@ -506,40 +588,73 @@ class SessionTestCase(unittest.TestCase):
                          all_list[:3])
 
     def test_session_admin_add(self):
-        auth_list, anon_list, all_list = _prep_session_table(self.env)
+        _prep_session_table(self.env)
         sess_admin = SessionAdmin(self.env)
+
         self.assertRaises(Exception, sess_admin._do_add, 'name00')
+
         sess_admin._do_add('john')
         result = get_session_info(self.env, 'john')
-        self.assertEqual(result, ('john', None, None))
+        self.assertEqual(result, ('john', None, None, None))
+        self.assertIn(('john', None, None),
+                      list(self.env.get_known_users()))
+
         sess_admin._do_add('john1', 'John1')
         result = get_session_info(self.env, 'john1')
-        self.assertEqual(result, ('john1', 'John1', None))
+        self.assertEqual(result, ('john1', 'John1', None, None))
+        self.assertIn(('john1', 'John1', None),
+                      list(self.env.get_known_users()))
+
         sess_admin._do_add('john2', 'John2', 'john2@example.org')
         result = get_session_info(self.env, 'john2')
-        self.assertEqual(result, ('john2', 'John2', 'john2@example.org'))
+        self.assertEqual(result,
+                         ('john2', 'John2', 'john2@example.org', None))
+        self.assertIn(('john2', 'John2', 'john2@example.org'),
+                      list(self.env.get_known_users()))
 
     def test_session_admin_set(self):
-        auth_list, anon_list, all_list = _prep_session_table(self.env)
+        _prep_session_table(self.env)
         sess_admin = SessionAdmin(self.env)
+
         self.assertRaises(TracError, sess_admin._do_set, 'name', 'nothere',
                           'foo')
+
+        self.env.get_known_users()  # Prep the cache
+        self.assertIn(('name00', 'val00', 'val00'),
+                      list(self.env.get_known_users()))
         sess_admin._do_set('name', 'name00', 'john')
         result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, ('name00', 'john', 'val00'))
+        self.assertEqual(result, ('name00', 'john', 'val00', None))
+        self.assertIn(('name00', 'john', 'val00'),
+                      list(self.env.get_known_users()))
+
         sess_admin._do_set('email', 'name00', 'john@example.org')
         result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, ('name00', 'john', 'john@example.org'))
+        self.assertEqual(result, ('name00', 'john', 'john@example.org', None))
+        self.assertIn(('name00', 'john', 'john@example.org'),
+                      list(self.env.get_known_users()))
+
+        sess_admin._do_set('default_handler', 'name00', 'SearchModule')
+        result = get_session_info(self.env, 'name00')
+        self.assertEqual(result, ('name00', 'john', 'john@example.org',
+                                  'SearchModule'))
 
     def test_session_admin_delete(self):
-        auth_list, anon_list, all_list = _prep_session_table(self.env)
+        _prep_session_table(self.env)
         sess_admin = SessionAdmin(self.env)
+
+        self.assertIn(('name00', 'val00', 'val00'),
+                      list(self.env.get_known_users()))
         sess_admin._do_delete('name00')
         result = get_session_info(self.env, 'name00')
-        self.assertEqual(result, (None, None, None))
+        self.assertEqual(result, (None, None, None, None))
+        self.assertNotIn(('name00', 'val00', 'val00'),
+                         list(self.env.get_known_users()))
+
         sess_admin._do_delete('nothere')
         result = get_session_info(self.env, 'nothere')
-        self.assertEqual(result, (None, None, None))
+        self.assertEqual(result, (None, None, None, None))
+
         auth_list, anon_list, all_list = _prep_session_table(self.env)
         sess_admin._do_delete('anonymous')
         result = [i for i in sess_admin._get_list(['*'])]
@@ -554,9 +669,9 @@ class SessionTestCase(unittest.TestCase):
         result = [i for i in sess_admin._get_list(['*'])]
         self.assertEqual(result, auth_list + anon_list)
         result = get_session_info(self.env, anon_list[0][0])
-        self.assertEqual(result, ('name10', 'val10', 'val10'))
+        self.assertEqual(result, ('name10', 'val10', 'val10', None))
         result = get_session_info(self.env, anon_list[1][0])
-        self.assertEqual(result, ('name11', 'val11', 'val11'))
+        self.assertEqual(result, ('name11', 'val11', 'val11', None))
 
         auth_list, anon_list, all_list = \
             _prep_session_table(self.env, spread_visits=True)
@@ -568,12 +683,44 @@ class SessionTestCase(unittest.TestCase):
             """, (anon_list[0][0],))
         self.assertEqual([], rows)
         result = get_session_info(self.env, anon_list[1][0])
-        self.assertEqual(result, ('name11', 'val11', 'val11'))
+        self.assertEqual(result, ('name11', 'val11', 'val11', None))
+
+    def test_session_get_session_with_invalid_sid(self):
+        req = MockRequest(self.env, authname='anonymous')
+        session = Session(self.env, req)
+        session.get_session('0123456789')
+        self.assertEqual('0123456789', session.sid)
+        session.get_session('abcxyz')
+        self.assertEqual('abcxyz', session.sid)
+        session.get_session('abc123xyz')
+        self.assertEqual('abc123xyz', session.sid)
+        self.assertRaises(TracError, session.get_session, 'abc 123 xyz')
+        self.assertRaises(TracError, session.get_session, 'abc-123-xyz')
+        self.assertRaises(TracError, session.get_session, 'abc<i>123</i>xyz')
+        self.assertRaises(TracError, session.get_session, u'abc123xÿz')
+        self.assertRaises(TracError, session.get_session,
+                          u'abc¹₂³xyz')  # Unicode digits
+
+    def test_session_change_id_with_invalid_sid(self):
+        req = MockRequest(self.env, authname='anonymous', base_path='/')
+        session = Session(self.env, req)
+        session.change_sid('0123456789')
+        self.assertEqual('0123456789', session.sid)
+        session.change_sid('abcxyz')
+        self.assertEqual('abcxyz', session.sid)
+        session.change_sid('abc123xyz')
+        self.assertEqual('abc123xyz', session.sid)
+        self.assertRaises(TracError, session.change_sid, 'abc 123 xyz')
+        self.assertRaises(TracError, session.change_sid, 'abc-123-xyz')
+        self.assertRaises(TracError, session.change_sid, 'abc<i>123</i>xyz')
+        self.assertRaises(TracError, session.change_sid, u'abc123xÿz')
+        self.assertRaises(TracError, session.change_sid,
+                          u'abc¹₂³xyz')  # Unicode digits
 
 
-def suite():
-    return unittest.makeSuite(SessionTestCase, 'test')
+def test_suite():
+    return unittest.makeSuite(SessionTestCase)
 
 
 if __name__ == '__main__':
-    unittest.main()
+    unittest.main(defaultTest='test_suite')
