@@ -14,6 +14,7 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
+from abc import ABCMeta, abstractmethod
 import errno
 import socket
 import sys
@@ -22,6 +23,17 @@ from SocketServer import ForkingMixIn, ThreadingMixIn
 import urllib
 
 
+# winsock errors
+_WSAECONNABORTED = 10053
+_WSAECONNRESET = 10054
+
+
+def is_client_disconnect_exception(e):
+    return isinstance(e, (IOError, socket.error)) and \
+           e.args[0] in (errno.EPIPE, errno.ECONNRESET, _WSAECONNABORTED,
+                         _WSAECONNRESET)
+
+
 class _ErrorsWrapper(object):
 
     def __init__(self, logfunc):
@@ -60,6 +72,8 @@ class _FileWrapper(object):
 class WSGIGateway(object):
     """Abstract base class for WSGI servers or gateways."""
 
+    __metaclass__ = ABCMeta
+
     wsgi_version = (1, 0)
     wsgi_multithread = True
     wsgi_multiprocess = True
@@ -85,6 +99,7 @@ class WSGIGateway(object):
 
         self.headers_set = []
         self.headers_sent = []
+        self.use_chunked = False
 
     def run(self, application):
         """Start the gateway with the given WSGI application."""
@@ -98,8 +113,8 @@ class WSGIGateway(object):
                 for chunk in response:
                     if chunk:
                         self._write(chunk)
-                if not self.headers_sent:
-                    self._write('')
+                if not self.headers_sent or self.use_chunked:
+                    self._write('') # last chunk '\r\n0\r\n' if use_chunked
         finally:
             if hasattr(response, 'close'):
                 response.close()
@@ -118,11 +133,12 @@ class WSGIGateway(object):
         self.headers_set = [status, headers]
         return self._write
 
+    @abstractmethod
     def _write(self, data):
         """Callback for writing data to the response.
 
         Concrete subclasses must implement this method."""
-        raise NotImplementedError
+        pass
 
 
 class WSGIRequestHandler(BaseHTTPRequestHandler):
@@ -179,10 +195,9 @@ class WSGIRequestHandler(BaseHTTPRequestHandler):
     def handle_one_request(self):
         try:
             environ = self.setup_environ()
-        except (IOError, socket.error), e:
+        except (IOError, socket.error) as e:
             environ = None
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
-                # client disconnect
+            if is_client_disconnect_exception(e):
                 self.close_connection = 1
             else:
                 raise
@@ -193,9 +208,15 @@ class WSGIRequestHandler(BaseHTTPRequestHandler):
 
     def finish(self):
         """We need to help the garbage collector a little."""
-        BaseHTTPRequestHandler.finish(self)
-        self.wfile = None
-        self.rfile = None
+        try:
+            BaseHTTPRequestHandler.finish(self)
+        except (IOError, socket.error) as e:
+            # ignore an exception if client disconnects
+            if not is_client_disconnect_exception(e):
+                raise
+        finally:
+            self.wfile = None
+            self.rfile = None
 
 
 class WSGIServerGateway(WSGIGateway):
@@ -212,15 +233,29 @@ class WSGIServerGateway(WSGIGateway):
 
         try:
             if not self.headers_sent:
+                # Worry at the last minute about Content-Length. If not
+                # yet set, use either chunked encoding or close connection
                 status, headers = self.headers_sent = self.headers_set
+                if any(n.lower() == 'content-length' for n, v in headers):
+                    self.use_chunked = False
+                else:
+                    self.use_chunked = (
+                        self.environ['SERVER_PROTOCOL'] >= 'HTTP/1.1' and
+                        self.handler.protocol_version >= 'HTTP/1.1')
+                    if self.use_chunked:
+                        headers.append(('Transfer-Encoding', 'chunked'))
+                    else:
+                        headers.append(('Connection', 'close'))
                 self.handler.send_response(int(status[:3]))
                 for name, value in headers:
                     self.handler.send_header(name, value)
                 self.handler.end_headers()
-            self.handler.wfile.write(data)
-        except (IOError, socket.error), e:
-            if e.args[0] in (errno.EPIPE, errno.ECONNRESET, 10053, 10054):
-                # client disconnect
+            if self.use_chunked:
+                self.handler.wfile.write('%x\r\n%s\r\n' % (len(data), data))
+            else:
+                self.handler.wfile.write(data)
+        except (IOError, socket.error) as e:
+            if is_client_disconnect_exception(e):
                 self.handler.close_connection = 1
             else:
                 raise
