@@ -14,8 +14,6 @@
 #
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
-from __future__ import with_statement
-
 from fnmatch import fnmatchcase
 from itertools import groupby
 import inspect
@@ -43,8 +41,10 @@ from trac.wiki.formatter import (
     MacroError, OutlineFormatter, extract_link, format_to_html,
     format_to_oneliner, system_message
 )
+from trac.wiki.interwiki import InterWikiMap
 
 
+# TODO: should be moved in .api
 class WikiMacroBase(Component):
     """Abstract base class for wiki macros."""
 
@@ -89,11 +89,16 @@ class TitleIndexMacro(WikiMacroBase):
 
     Accepts a prefix string as parameter: if provided, only pages with names
     that start with the prefix are included in the resulting list. If this
-    parameter is omitted, all pages are listed.
-    If the prefix is specified, a second argument of value `hideprefix`
-    can be given as well, in order to remove that prefix from the output.
+    parameter is omitted, all pages are listed. If the prefix is specified,
+    a second argument of value `hideprefix` can be given as well, in order
+    to remove that prefix from the output.
 
-    Alternate `format` and `depth` named parameters can be specified:
+    The prefix string supports the standard relative-path notation ''when
+    using the macro in a wiki page''. A prefix string starting with `./`
+    will be relative to the current page, and parent pages can be
+    specified using `../`.
+
+    Several named parameters can be specified:
      - `format=compact`: The pages are displayed as comma-separated links.
      - `format=group`: The list of pages will be structured in groups
        according to common prefix. This format also supports a `min=n`
@@ -124,7 +129,6 @@ class TitleIndexMacro(WikiMacroBase):
         minsize = _arg_as_int(kw.get('min', 1), 'min', min=1)
         minsize_group = max(minsize, 2)
         depth = _arg_as_int(kw.get('depth', -1), 'depth', min=-1)
-        start = prefix.count('/') if prefix else 0
         format = kw.get('format', '')
 
         def parse_list(name):
@@ -134,13 +138,18 @@ class TitleIndexMacro(WikiMacroBase):
         includes = parse_list('include') or ['*']
         excludes = parse_list('exclude')
 
+        wiki = formatter.wiki
+        resource = formatter.resource
+        if prefix and resource and resource.realm == 'wiki':
+            prefix = wiki.resolve_relative_name(prefix, resource.id)
+
+        start = prefix.count('/') if prefix else 0
+
         if hideprefix:
             omitprefix = lambda page: page[len(prefix):]
         else:
             omitprefix = lambda page: page
 
-        wiki = formatter.wiki
-
         pages = sorted(page for page in wiki.get_pages(prefix)
                        if (depth < 0 or depth >= page.count('/') - start)
                        and 'WIKI_VIEW' in formatter.perm('wiki', page)
@@ -282,7 +291,7 @@ class TitleIndexMacro(WikiMacroBase):
             'group': lambda p: render_group(tree_group(split_pages_group(p))),
             'hierarchy': lambda p: render_hierarchy(
                                     tree_hierarchy(split_pages_hierarchy(p))),
-            }.get(format, None)
+            }.get(format)
 
         if transform:
             titleindex = transform(pages)
@@ -457,14 +466,15 @@ class ImageMacro(WikiMacroBase):
      * `file` to refer to a local attachment named 'file'. This only works from
        within that wiki page or a ticket.
 
-    Also, the file specification may refer to repository files, using the
-    `source:file` syntax (`source:file@rev` works also).
-
-    Files can also be accessed with a direct URLs; `/file` for a
-    project-relative, `//file` for a server-relative, or `http://server/file`
-    for absolute location of the file. The
-    [http://tools.ietf.org/html/rfc2397 rfc2397] `data` URL scheme is also
-    supported if the URL is enclosed in quotes.
+    The file specification may also refer to:
+     * repository files, using the `source:file` syntax
+       (`source:file@rev` works also).
+     * files, using direct URLs: `/file` for a project-relative,
+       `//file` for a server-relative, or `http://server/file` for
+       absolute location. An InterWiki prefix may be used.
+     * embedded data using the
+       [http://tools.ietf.org/html/rfc2397 rfc2397] `data` URL scheme,
+       provided the URL is enclosed in quotes.
 
     The remaining arguments are optional and allow configuring the attributes
     and style of the rendered `<img>` element:
@@ -601,11 +611,16 @@ class ImageMacro(WikiMacroBase):
         # parse filespec argument to get realm and id if contained.
         parts = [i.strip('''['"]''')
                  for i in self._split_filespec_re.split(filespec)[1::2]]
+        realm = parts[0] if parts else None
         url = raw_url = desc = None
         attachment = None
-        if parts and parts[0] in ('http', 'https', 'ftp', 'data'):  # absolute
+        interwikimap = InterWikiMap(self.env)
+        if realm in ('http', 'https', 'ftp', 'data'):  # absolute
             raw_url = url = filespec
             desc = url.rsplit('?')[0]
+        elif realm in interwikimap:
+            url, desc = interwikimap.url(realm, ':'.join(parts[1:]))
+            raw_url = url
         elif filespec.startswith('//'):       # server-relative
             raw_url = url = filespec[1:]
             desc = url.rsplit('?')[0]
@@ -719,7 +734,7 @@ class MacroListMacro(WikiMacroBase):
                     name_descriptions = [
                         (name, macro_provider.get_macro_description(name))
                         for name in names]
-                except Exception, e:
+                except Exception as e:
                     yield system_message(
                         _("Error: Can't get description for macro %(name)s",
                           name=names[0]), e), names
@@ -772,14 +787,8 @@ class TracIniMacro(WikiMacroBase):
         if args:
             key_filter = args.pop(0).strip()
 
-        def getdoc(option_or_section):
-            doc = to_unicode(option_or_section.__doc__)
-            if doc:
-                doc = dgettext(option_or_section.doc_domain, doc)
-            return doc
-
         registry = ConfigSection.get_registry(self.compmgr)
-        sections = dict((name, getdoc(section))
+        sections = dict((name, section.doc)
                         for name, section in registry.iteritems()
                         if name.startswith(section_filter))
 
@@ -802,9 +811,9 @@ class TracIniMacro(WikiMacroBase):
             (tag.h3(tag.code('[%s]' % section), id='%s-section' % section),
              format_to_html(self.env, formatter.context, section_doc),
              tag.table(class_='wiki')(tag.tbody(
-                 tag.tr(tag.td(tag.tt(option.name)),
+                 tag.tr(tag.td(tag.code(option.name)),
                         tag.td(format_to_html(self.env, formatter.context,
-                                              getdoc(option))),
+                                              option.doc)),
                         default_cell(option),
                         class_='odd' if idx % 2 else 'even')
                  for idx, option in
@@ -844,7 +853,7 @@ class KnownMimeTypesMacro(WikiMacroBase):
                                  href=formatter.context.href.wiki(
                                      'WikiProcessors'))))),
                 tag.tbody(
-                    tag.tr(tag.th(tag.tt(mime_type),
+                    tag.tr(tag.th(tag.code(mime_type),
                                   style="text-align: left"),
                            tag.td(tag.code(
                                ' '.join(sorted(mime_types[mime_type])))))
@@ -913,6 +922,6 @@ def _arg_as_int(val, key=None, min=None, max=None):
     int_val = as_int(val, None, min=min, max=max)
     if int_val is None:
         raise MacroError(tag_("Invalid macro argument %(expr)s",
-                              expr=tag.tt("%s=%s" % (key, val))
-                                   if key else tag.tt(val)))
+                              expr=tag.code("%s=%s" % (key, val))
+                                   if key else tag.code(val)))
     return int_val
