@@ -47,6 +47,9 @@ def image_setup(tc):
     tc.env.path = tempfile.mkdtemp(prefix='trac-tempenv-')
     add_attachment(tc, 'wiki', 'page:fr', 'img.png')
     add_attachment(tc, 'wiki', 'page', 'img.png')
+    tc.env.config.set('interwiki', 'shields', 'https://img.shields.io/')
+    tc.env.config.set('interwiki', 'travis',
+                      'https://travis-ci.org/$1?branch=$2')
     htdocs_location = 'http://assets.example.org/common'
     tc.context.req.chrome['htdocs_location'] = htdocs_location
     tc.env.config.set('trac', 'htdocs_location', htdocs_location)
@@ -167,6 +170,20 @@ IMAGE_MACRO_TEST_CASES = u"""
 <p>
 <img src="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" alt="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" title="data:image/png;base64,iVBORw0KGgoAAAANSUhEUgAAACgAAAAoAQMAAAC2MCouAAAAA1BMVEXLQ0MOAUiXAAAAC0lEQVQIHWMYYQAAAPAAASEIRrcAAAAASUVORK5CYII=" />
 </p>
+============================== InterWiki
+[[Image(shields:travis/edgewall/trac.svg, link=trac:source:/trunk)]]
+[[Image(travis:edgewall/trac.svg:1.0-stable, link=trac:source:/branches/1.0-stable)]]
+------------------------------
+<p>
+<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/trunk"><img src="https://img.shields.io/travis/edgewall/trac.svg" alt="travis/edgewall/trac.svg in shields" title="travis/edgewall/trac.svg in shields" /></a>
+<a style="padding:0; border:none" href="http://trac.edgewall.org/intertrac/source%3A/branches/1.0-stable"><img src="https://travis-ci.org/edgewall/trac.svg?branch=1.0-stable" alt="edgewall/trac.svg:1.0-stable in travis" title="edgewall/trac.svg:1.0-stable in travis" /></a>
+</p>
+============================== InterWiki, nolink
+[[Image(shields:pypi/dm/trac.svg, nolink)]]
+------------------------------
+<p>
+<img src="https://img.shields.io/pypi/dm/trac.svg" alt="pypi/dm/trac.svg in shields" title="pypi/dm/trac.svg in shields" />
+</p>
 ==============================
 [[Image(notfound.png, nolink)]]
 ------------------------------
@@ -183,37 +200,37 @@ IMAGE_MACRO_TEST_CASES = u"""
 [[Image(img.png, margin-bottom=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>margin-bottom=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>margin-bottom=--</code></strong></div>
 </p>
 ==============================
 [[Image(img.png, margin-top=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>margin-top=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>margin-top=--</code></strong></div>
 </p>
 ==============================
 [[Image(img.png, margin=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>margin=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>margin=--</code></strong></div>
 </p>
 ==============================
 [[Image(img.png, margin-left=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>margin-left=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>margin-left=--</code></strong></div>
 </p>
 ==============================
 [[Image(img.png, margin-right=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>margin-right=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>margin-right=--</code></strong></div>
 </p>
 ==============================
 [[Image(img.png, border=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>border=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>border=--</code></strong></div>
 </p>
 ==============================  # Regression test for #12333
 = [[Image]]
@@ -268,13 +285,13 @@ TITLEINDEX1_MACRO_TEST_CASES = u"""
 [[TitleIndex(min=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>min=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>min=--</code></strong></div>
 </p>
 ==============================
 [[TitleIndex(depth=--)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>depth=--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>depth=--</code></strong></div>
 </p>
 ------------------------------
 """
@@ -363,6 +380,55 @@ TITLEINDEX3_MACRO_TEST_CASES = u"""
 </p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/First">First</a></li><li><a href="/wiki/WikiStart/Second">Second</a></li><li><a href="/wiki/WikiStart/Third">Third</a></li></ul></div><p>
 </p>
 ------------------------------
+============================== TitleIndex, relative prefix
+[[TitleIndex(../../WikiStart)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart">WikiStart</a></li><li><a href="/wiki/WikiStart/First">WikiStart/First</a></li><li><a href="/wiki/WikiStart/Second">WikiStart/Second</a></li><li><a href="/wiki/WikiStart/Third">WikiStart/Third</a></li></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative prefix with trailing slash
+[[TitleIndex(../../WikiStart/)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/First">WikiStart/First</a></li><li><a href="/wiki/WikiStart/Second">WikiStart/Second</a></li><li><a href="/wiki/WikiStart/Third">WikiStart/Third</a></li></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative prefix ..
+[[TitleIndex(..)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart">WikiStart</a></li><li><a href="/wiki/WikiStart/First">WikiStart/First</a></li><li><a href="/wiki/WikiStart/Second">WikiStart/Second</a></li><li><a href="/wiki/WikiStart/Third">WikiStart/Third</a></li></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative prefix ../
+[[TitleIndex(../)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/First">WikiStart/First</a></li><li><a href="/wiki/WikiStart/Second">WikiStart/Second</a></li><li><a href="/wiki/WikiStart/Third">WikiStart/Third</a></li></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative prefix .
+[[TitleIndex(.)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/Second">WikiStart/Second</a></li></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative prefix ./
+[[TitleIndex(./)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul></ul></div><p>
+</p>
+------------------------------
+============================== TitleIndex, relative hidden prefix ../
+[[TitleIndex(../,hideprefix)]]
+------------------------------
+<p>
+</p><div class="titleindex"><ul><li><a href="/wiki/WikiStart/First">First</a></li><li><a href="/wiki/WikiStart/Second">Second</a></li><li><a href="/wiki/WikiStart/Third">Third</a></li></ul></div><p>
+</p>
+------------------------------
 ============================== TitleIndex, top-level pages only
 [[TitleIndex(depth=0)]]
 ------------------------------
@@ -509,7 +575,7 @@ RECENTCHANGES_MACRO_TEST_CASES = u""""
 [[RecentChanges(Trac, --)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>--</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>--</code></strong></div>
 </p>
 ------------------------------
 """
@@ -538,13 +604,13 @@ PAGEOUTLINE_MACRO_TEST_CASES = u""""\
 [[PageOutline(a)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>a</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>a</code></strong></div>
 </p>
 ==============================
 [[PageOutline(a-b)]]
 ------------------------------
 <p>
-<div class="system-message"><strong>Invalid macro argument <tt>a</tt></strong></div>
+<div class="system-message"><strong>Invalid macro argument <code>a</code></strong></div>
 </p>
 ==============================
 [[PageOutline(0)]]
@@ -637,10 +703,13 @@ TRACINI_MACRO_TEST_CASES = u"""\
 <p>
 </p><div class="tracini"><h3 id="section-42-section"><code>[section-42]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><tt>option1</tt></td><td></td><td class="default"><code>value</code></td></tr>\
-<tr class="odd"><td><tt>option2</tt></td><td><p>
+<tr class="even"><td><code>option1</code></td><td></td><td class="default"><code>value</code></td></tr>\
+<tr class="odd"><td><code>option2</code></td><td><p>
 blah
 </p>
+</td><td class="default"><code>value</code></td></tr><tr class="even"><td><code>option3</code></td><td><p>
+Doc for option3
+</p>
 </td><td class="default"><code>value</code></td></tr></tbody></table></div><p>
 </p>
 ------------------------------
@@ -651,7 +720,7 @@ blah
 </p><div class="tracini">\
 <h3 id="section-list-section"><code>[section-list]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><tt>option1</tt></td><td></td><td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
+<tr class="even"><td><code>option1</code></td><td></td><td class="default"><code>4.2|42|42||0|enabled</code></td></tr>\
 </tbody></table>\
 </div><p>
 </p>
@@ -663,11 +732,11 @@ blah
 </p><div class="tracini">\
 <h3 id="section-def-section"><code>[section-def]</code></h3>\
 <table class="wiki"><tbody>\
-<tr class="even"><td><tt>option1</tt></td><td></td><td class="nodefault">(no default)</td></tr>\
-<tr class="odd"><td><tt>option2</tt></td><td></td><td class="nodefault">(no default)</td></tr>\
-<tr class="even"><td><tt>option3</tt></td><td></td><td class="default"><code>0</code></td></tr>\
-<tr class="odd"><td><tt>option4</tt></td><td></td><td class="default"><code>disabled</code></td></tr>\
-<tr class="even"><td><tt>option5</tt></td><td></td><td class="default"><code></code></td></tr>\
+<tr class="even"><td><code>option1</code></td><td></td><td class="nodefault">(no default)</td></tr>\
+<tr class="odd"><td><code>option2</code></td><td></td><td class="nodefault">(no default)</td></tr>\
+<tr class="even"><td><code>option3</code></td><td></td><td class="default"><code>0</code></td></tr>\
+<tr class="odd"><td><code>option4</code></td><td></td><td class="default"><code>disabled</code></td></tr>\
+<tr class="even"><td><code>option5</code></td><td></td><td class="nodefault">(no default)</td></tr>\
 </tbody></table>\
 </div><p>
 </p>
@@ -679,8 +748,12 @@ def tracini_setup(tc):
     class Foo(object):
         option_a1 = (Option)('section-42', 'option1', 'value', doc='')
         option_a2 = (Option)('section-42', 'option2', 'value', doc='blah')
+        option_a3 = (Option)('section-42', 'option3', 'value',
+                             doc='Doc for %(name)s',
+                             doc_args={'name': 'option3'})
         option_l1 = (ListOption)('section-list', 'option1',
-                                 [4.2, '42', 42, None, 0, True], sep='|')
+                                 [4.2, '42', 42, None, 0, True], sep='|',
+                                 keep_empty=True)
         option_d1 = (Option)('section-def', 'option1', None)
         option_d2 = (Option)('section-def', 'option2', '')
         option_d3 = (IntOption)('section-def', 'option3', 0)
@@ -691,33 +764,43 @@ def tracini_teardown(tc):
     Option.registry = tc._orig_registry
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(formatter.suite(IMAGE_MACRO_TEST_CASES, file=__file__,
-                                  setup=image_setup,
-                                  teardown=image_teardown))
-    suite.addTest(formatter.suite(TITLEINDEX1_MACRO_TEST_CASES, file=__file__))
-    suite.addTest(formatter.suite(TITLEINDEX2_MACRO_TEST_CASES, file=__file__,
-                                  setup=titleindex2_setup,
-                                  teardown=titleindex_teardown))
-    suite.addTest(formatter.suite(TITLEINDEX3_MACRO_TEST_CASES, file=__file__,
-                                  setup=titleindex3_setup,
-                                  teardown=titleindex_teardown))
-    suite.addTest(formatter.suite(TITLEINDEX4_MACRO_TEST_CASES, file=__file__,
-                                  setup=titleindex4_setup,
-                                  teardown=titleindex_teardown))
-    suite.addTest(formatter.suite(TITLEINDEX5_MACRO_TEST_CASES, file=__file__,
-                                  setup=titleindex5_setup,
-                                  teardown=titleindex_teardown))
-    suite.addTest(formatter.suite(RECENTCHANGES_MACRO_TEST_CASES, file=__file__,
-                                  setup=recentchanges_setup,
-                                  teardown=recentchanges_teardown))
-    suite.addTest(formatter.suite(PAGEOUTLINE_MACRO_TEST_CASES, file=__file__))
-    suite.addTest(formatter.suite(TRACINI_MACRO_TEST_CASES, file=__file__,
-                                  setup=tracini_setup,
-                                  teardown=tracini_teardown))
+    suite.addTest(formatter.test_suite(IMAGE_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=image_setup,
+                                       teardown=image_teardown))
+    suite.addTest(formatter.test_suite(TITLEINDEX1_MACRO_TEST_CASES,
+                                       file=__file__))
+    suite.addTest(formatter.test_suite(TITLEINDEX2_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=titleindex2_setup,
+                                       teardown=titleindex_teardown))
+    suite.addTest(formatter.test_suite(TITLEINDEX3_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=titleindex3_setup,
+                                       teardown=titleindex_teardown,
+                                       context=('wiki', 'WikiStart/Second')))
+    suite.addTest(formatter.test_suite(TITLEINDEX4_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=titleindex4_setup,
+                                       teardown=titleindex_teardown))
+    suite.addTest(formatter.test_suite(TITLEINDEX5_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=titleindex5_setup,
+                                       teardown=titleindex_teardown))
+    suite.addTest(formatter.test_suite(RECENTCHANGES_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=recentchanges_setup,
+                                       teardown=recentchanges_teardown))
+    suite.addTest(formatter.test_suite(PAGEOUTLINE_MACRO_TEST_CASES,
+                                       file=__file__))
+    suite.addTest(formatter.test_suite(TRACINI_MACRO_TEST_CASES,
+                                       file=__file__,
+                                       setup=tracini_setup,
+                                       teardown=tracini_teardown))
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
