@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright (C) 2012 Edgewall Software
+# Copyright (C) 2012-2013 Edgewall Software
 # All rights reserved.
 #
 # This software is licensed as described in the file COPYING, which
@@ -16,11 +16,11 @@ import unittest
 from tracopt.perm.tests import authz_policy
 
 
-def suite():
+def test_suite():
     suite = unittest.TestSuite()
-    suite.addTest(authz_policy.suite())
+    suite.addTest(authz_policy.test_suite())
     return suite
 
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
