@@ -12,8 +12,6 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
 import os
 import codecs
 from collections import deque
@@ -23,17 +21,18 @@ from functools import partial
 import re
 from subprocess import Popen, PIPE
 from threading import Lock
-import time
 import weakref
 
+from trac.core import TracBaseError
 from trac.util import terminate
 from trac.util.compat import close_fds
+from trac.util.datefmt import time_now
 from trac.util.text import to_unicode
 
 __all__ = ['GitError', 'GitErrorSha', 'Storage', 'StorageFactory']
 
 
-class GitError(Exception):
+class GitError(TracBaseError):
     pass
 
 class GitErrorSha(GitError):
@@ -86,6 +85,13 @@ def _unquote(path):
     return path
 
 
+def _close_proc_pipes(proc):
+    if proc:
+        for f in (proc.stdin, proc.stdout, proc.stderr):
+            if f:
+                f.close()
+
+
 class GitCore(object):
     """Low-level wrapper around git executable"""
 
@@ -126,16 +132,20 @@ class GitCore(object):
         return cmd
 
     def __pipe(self, git_cmd, *cmd_args, **kw):
+        kw.setdefault('stdin', PIPE)
+        kw.setdefault('stdout', PIPE)
+        kw.setdefault('stderr', PIPE)
         return Popen(self.__build_git_cmd(git_cmd, *cmd_args),
                      close_fds=close_fds, **kw)
 
     def __execute(self, git_cmd, *cmd_args):
         """execute git command and return file-like object of stdout"""
 
-        #print >>sys.stderr, "DEBUG:", git_cmd, cmd_args
+        #print("DEBUG:", git_cmd, cmd_args, file=sys.stderr)
 
         p = self.__pipe(git_cmd, stdout=PIPE, stderr=PIPE, *cmd_args)
         stdout_data, stderr_data = p.communicate()
+        _close_proc_pipes(p)
         if self.__log and (p.returncode != 0 or stderr_data):
             self.__log.debug('%s exits with %d, dir: %r, args: %s %r, '
                              'stderr: %r', self.__git_bin, p.returncode,
@@ -195,7 +205,7 @@ class SizedDict(dict):
             return rc
 
     def setdefault(self, *_):
-        raise NotImplemented("SizedDict has no setdefault() method")
+        raise NotImplementedError("SizedDict has no setdefault() method")
 
 
 class StorageFactory(object):
@@ -333,7 +343,7 @@ class Storage(object):
             result['v_compatible'] = split_version >= GIT_VERSION_MIN_REQUIRED
             return result
 
-        except Exception, e:
+        except Exception as e:
             raise GitError("Could not retrieve GIT version (tried to "
                            "execute/parse '%s --version' but got %s)"
                            % (git_bin, repr(e)))
@@ -377,7 +387,8 @@ class Storage(object):
             codecs.lookup(git_fs_encoding)
 
             # setup conversion functions
-            self._fs_to_unicode = lambda s: s.decode(git_fs_encoding)
+            self._fs_to_unicode = lambda s: s.decode(git_fs_encoding,
+                                                     'replace')
             self._fs_from_unicode = lambda s: s.encode(git_fs_encoding)
         else:
             # pass bytestrings as-is w/o any conversion
@@ -403,7 +414,7 @@ class Storage(object):
         try:
             with open(head_file, 'rb'):
                 pass
-        except IOError, e:
+        except IOError as e:
             raise GitError("Make sure the Git repository '%s' is readable: %s"
                            % (git_dir, to_unicode(e)))
 
@@ -413,12 +424,15 @@ class Storage(object):
         self.logger.debug("PyGIT.Storage instance for '%s' is constructed",
                           git_dir)
 
+    def _cleanup_proc(self, proc):
+        if proc:
+            _close_proc_pipes(proc)
+            terminate(proc)
+            proc.wait()
+
     def __del__(self):
         with self.__cat_file_pipe_lock:
-            if self.__cat_file_pipe is not None:
-                self.__cat_file_pipe.stdin.close()
-                terminate(self.__cat_file_pipe)
-                self.__cat_file_pipe.wait()
+            self._cleanup_proc(self.__cat_file_pipe)
 
     #
     # cache handling
@@ -460,40 +474,46 @@ class Storage(object):
     def _build_rev_cache(self, refs):
         self.logger.debug("triggered rebuild of commit tree db for '%s'",
                           self.repo_path)
-        ts0 = time.time()
+        ts0 = time_now()
 
-        youngest = None
-        oldest = None
         new_db = {} # db
         new_sdb = {} # short_rev db
 
         # helper for reusing strings
         revs_seen = {}
-        def __rev_reuse(rev):
+        def _rev_reuse(rev):
             return revs_seen.setdefault(rev, rev)
 
-        refs = dict((refname, __rev_reuse(rev))
+        refs = dict((refname, _rev_reuse(rev))
                     for refname, rev in refs.iteritems())
         head_revs = set(rev for refname, rev in refs.iteritems()
                             if refname.startswith('refs/heads/'))
+        rev_list = [map(_rev_reuse, line.split())
+                    for line in self.repo.rev_list('--parents', '--topo-order',
+                                                   '--all').splitlines()]
+        revs_seen = None
+
+        if rev_list:
+            # first rev seen is assumed to be the youngest one
+            youngest = rev_list[0][0]
+            # last rev seen is assumed to be the oldest one
+            oldest = rev_list[-1][0]
+        else:
+            youngest = oldest = None
+
+        rheads_seen = {}
+        def _rheads_reuse(rheads):
+            rheads = frozenset(rheads)
+            return rheads_seen.setdefault(rheads, rheads)
 
         __rev_key = self.__rev_key
-        rev = ord_rev = None
-        rev_list = self.repo.rev_list('--parents', '--topo-order', '--all')
-        for ord_rev, revs in enumerate(rev_list.splitlines()):
-            revs = map(__rev_reuse, revs.split())
+        for ord_rev, revs in enumerate(rev_list):
             rev = revs[0]
-
-            # first rev seen is assumed to be the youngest one
-            if not ord_rev:
-                youngest = rev
+            parents = revs[1:]
 
             # shortrev "hash" map
             new_sdb.setdefault(__rev_key(rev), []).append(rev)
 
-            # parents
-            parents = revs[1:]
-
             # new_db[rev] = (children(rev), parents(rev),
             #                ordinal_id(rev), rheads(rev))
             if rev in new_db:
@@ -509,13 +529,13 @@ class Storage(object):
                 _rheads.add(rev)
 
             # create/update entry
-            # transform lists into tuples since entry will be final
-            new_db[rev] = _children, parents, ord_rev + 1, _rheads
+            # transform into frozenset and tuple since entry will be final
+            new_db[rev] = (frozenset(_children), tuple(parents), ord_rev + 1,
+                           _rheads_reuse(_rheads))
 
             # update parents(rev)s
             for parent in parents:
-                # by default, a dummy ordinal_id is used
-                # for the mean-time
+                # by default, a dummy ordinal_id is used for the mean-time
                 _children, _parents, _ord_rev, _rheads2 = \
                     new_db.setdefault(parent, (set(), [], 0, set()))
 
@@ -525,9 +545,7 @@ class Storage(object):
                 # update parent(rev)'s rheads
                 _rheads2.update(_rheads)
 
-        # last rev seen is assumed to be the oldest one (with highest ord_rev)
-        oldest = rev
-        revs_seen = None
+        rheads_seen = None
 
         # convert sdb either to dict or array depending on size
         tmp = [()] * (max(new_sdb.keys()) + 1) if len(new_sdb) > 5000 else {}
@@ -543,7 +561,7 @@ class Storage(object):
         rev_cache = self.RevCache(youngest, oldest, new_db, refs, new_sdb)
         self.logger.debug("rebuilt commit tree db for '%s' with %d entries "
                           "(took %.1f ms)", self.repo_path, len(new_db),
-                          1000 * (time.time() - ts0))
+                          1000 * (time_now() - ts0))
         return rev_cache
 
     def _get_refs(self):
@@ -675,9 +693,7 @@ class Storage(object):
                 # consistent state (Otherwise it happens that next time we
                 # call cat_file we get payload from previous call)
                 self.logger.debug("closing cat_file pipe")
-                self.__cat_file_pipe.stdin.close()
-                terminate(self.__cat_file_pipe)
-                self.__cat_file_pipe.wait()
+                self._cleanup_proc(self.__cat_file_pipe)
                 self.__cat_file_pipe = None
 
     def verifyrev(self, rev):
@@ -915,9 +931,8 @@ class Storage(object):
                             path, _ = path.rsplit('/', 1)
                         except ValueError:
                             break
-            f.close()
-            terminate(p[0])
-            p[0].wait()
+            if p:
+                self._cleanup_proc(p[0])
             p[:] = []
             while True:
                 yield None
@@ -935,9 +950,7 @@ class Storage(object):
             yield historian
         finally:
             if p:
-                p[0].stdout.close()
-                terminate(p[0])
-                p[0].wait()
+                self._cleanup_proc(p[0])
 
     def last_change(self, sha, path, historian=None):
         if historian is not None:
