@@ -12,8 +12,6 @@
 # individuals. For the exact contribution history, see the revision
 # history and logs, available at http://trac.edgewall.org/log/.
 
-from __future__ import with_statement
-
 from datetime import datetime
 import itertools
 import os
@@ -22,8 +20,9 @@ from genshi.builder import tag
 from genshi.core import Markup
 
 from trac.cache import cached
-from trac.config import BoolOption, IntOption, PathOption, Option
+from trac.config import BoolOption, IntOption, ListOption, PathOption, Option
 from trac.core import *
+from trac.env import ISystemInfoProvider
 from trac.util import TracError, shorten_line
 from trac.util.datefmt import FixedOffset, to_timestamp, format_datetime
 from trac.util.text import to_unicode, exception_to_unicode
@@ -136,15 +135,12 @@ class GitCachedRepository(CachedRepository):
                     rev = revs.pop()
                     self.log.info("Trying to sync revision [%s]", rev)
                     cset = repos.get_changeset(rev)
-                    with self.env.db_transaction as db:
-                        try:
-                            self._insert_changeset(db, rev, cset)
-                            updated = True
-                        except self.env.db_exc.IntegrityError, e:
-                            self.log.info('Revision %s already cached: %r',
-                                          rev, e)
-                            db.rollback()
-                            continue
+                    try:
+                        self.insert_changeset(rev, cset)
+                        updated = True
+                    except self.env.db_exc.IntegrityError as e:
+                        self.log.info('Revision %s already cached: %r', rev, e)
+                        continue
                     if feedback:
                         feedback(rev)
 
@@ -207,24 +203,29 @@ def _parse_user_time(s):
 
 class GitConnector(Component):
 
-    implements(IRepositoryConnector, IWikiSyntaxProvider)
+    implements(IRepositoryConnector, ISystemInfoProvider, IWikiSyntaxProvider)
 
     def __init__(self):
         self._version = None
 
         try:
             self._version = PyGIT.Storage.git_version(git_bin=self.git_bin)
-        except PyGIT.GitError, e:
+        except PyGIT.GitError as e:
             self.log.error("GitError: %s", e)
 
         if self._version:
             self.log.info("detected GIT version %s", self._version['v_str'])
-            self.env.systeminfo.append(('GIT', self._version['v_str']))
             if not self._version['v_compatible']:
                 self.log.error("GIT version %s installed not compatible"
                                "(need >= %s)", self._version['v_str'],
                                self._version['v_min_str'])
 
+    # ISystemInfoProvider methods
+
+    def get_system_info(self):
+        if self._version:
+            yield 'GIT', self._version['v_str']
+
     # IWikiSyntaxProvider methods
 
     def _format_sha_link(self, formatter, sha, label):
@@ -250,7 +251,7 @@ class GitConnector(Component):
             return tag.a(label, class_='changeset',
                          title=shorten_line(changeset.message),
                          href=formatter.href.changeset(sha, repos.reponame))
-        except Exception, e:
+        except Exception as e:
             return tag.a(label, class_='missing changeset',
                          title=to_unicode(e), rel='nofollow')
 
@@ -405,7 +406,7 @@ class CsetPropertyRenderer(Component):
                              title=shorten_line(cset.message),
                              href=context.href.changeset(sha, repos.reponame))
 
-            except Exception, e:
+            except Exception as e:
                 return tag.a(sha, class_='missing changeset',
                              title=to_unicode(e), rel='nofollow')
 
@@ -444,9 +445,10 @@ class CsetPropertyRenderer(Component):
                                              "itself.")),
                                     class_='hint'),
                            tag.br(),
-                           tag.span(Markup(_("Use the <tt>(diff)</tt> links "
-                                             "above to see all the changes "
-                                             "relative to each parent.")),
+                           tag.span(Markup(_("Use the <code>(diff)</code> "
+                                             "links above to see all the "
+                                             "changes relative to each "
+                                             "parent.")),
                                     class_='hint'))
 
             # simple non-merge commit
@@ -490,7 +492,7 @@ class GitRepository(Repository):
                                            git_bin=git_bin,
                                            git_fs_encoding=git_fs_encoding)
             self._git = factory.getInstance()
-        except PyGIT.GitError, e:
+        except PyGIT.GitError as e:
             log.error(exception_to_unicode(e))
             raise InvalidRepository(
                 _("%(path)s does not appear to be a Git repository.",
@@ -517,6 +519,9 @@ class GitRepository(Repository):
     def get_youngest_rev(self):
         return self.git.youngest_rev()
 
+    def get_path_history(self, path, rev=None, limit=None):
+        raise TracError(_("Unsupported \"Show only adds and deletes\""))
+
     def get_oldest_rev(self):
         return self.git.oldest_rev()
 
@@ -879,17 +884,22 @@ class GitwebProjectsRepositoryProvider(Component):
 
     implements(IRepositoryProvider)
 
-    projects_list = PathOption('git', 'projects_list', doc=
+    projects_list = PathOption('gitweb-repositories', 'projects_list', doc=
         """Path to a gitweb-formatted projects.list""")
 
-    projects_base = PathOption('git', 'projects_base', doc=
+    projects_base = PathOption('gitweb-repositories', 'projects_base', doc=
         """Path to the base of your git projects""")
 
-    projects_url = Option('git', 'projects_url', doc=
-        """Template for project URLs. %s will be replaced with the repo
+    projects_url = Option('gitweb-repositories', 'projects_url', doc=
+        """Template for project URLs. `%s` will be replaced with the repo
         name""")
 
+    sync_per_request = ListOption('gitweb-repositories',
+        'sync_per_request', '', doc="""Repositories to sync on every request
+        (not recommended).""")
+
     def get_repositories(self):
+        """Retrieve repositories specified in a `projects_list` file."""
         if not self.projects_list:
             return
 
@@ -902,17 +912,18 @@ class GitwebProjectsRepositoryProvider(Component):
             for line in fp:
                 entries = line.strip().split()
                 if entries:
-                    reponame = entries[0]
-                    repo = {
-                        'dir': os.path.join(self.projects_base, reponame),
+                    name = entries[0]
+                    reponame = name.rstrip('.git')
+                    info = {
+                        'dir': os.path.join(self.projects_base, name),
+                        'sync_per_request': reponame in self.sync_per_request,
                         'type': 'git',
                     }
                     description_path = \
-                        os.path.join(repo['dir'], 'description')
+                        os.path.join(info['dir'], 'description')
                     if os.path.exists(description_path):
                         with open(description_path, 'r') as fd:
-                            repo['description'] = fd.read().strip()
-                    name = reponame.rstrip('.git')
+                            info['description'] = fd.read().strip()
                     if self.projects_url:
-                        repo['url'] = self.projects_url % name
-                    yield name, repo
+                        info['url'] = self.projects_url % reponame
+                    yield reponame, info
