@@ -47,8 +47,6 @@ Warning:
   those properties...
 """
 
-from __future__ import with_statement
-
 import os.path
 import re
 import weakref
@@ -286,9 +284,9 @@ class SubversionConnector(Component):
         """End-of-Line character sequences when `svn:eol-style` property is
         `native`.
 
-        If `native` (the default), substitute with the native EOL marker on
-        the server. Otherwise, if `LF`, `CRLF` or `CR`, substitute with the
-        specified EOL marker.
+        If `native`, substitute with the native EOL marker on the server.
+        Otherwise, if `LF`, `CRLF` or `CR`, substitute with the specified
+        EOL marker.
 
         (''since 1.0.2'')""")
 
@@ -298,11 +296,11 @@ class SubversionConnector(Component):
         self._version = None
         try:
             _import_svn()
-            self.log.debug("Subversion bindings imported")
-        except ImportError, e:
+        except ImportError as e:
             self.error = e
             self.log.info('Failed to load Subversion bindings', exc_info=True)
         else:
+            self.log.debug("Subversion bindings imported")
             version = (core.SVN_VER_MAJOR, core.SVN_VER_MINOR,
                        core.SVN_VER_MICRO)
             self._version = '%d.%d.%d' % version + core.SVN_VER_TAG
@@ -314,7 +312,7 @@ class SubversionConnector(Component):
     # ISystemInfoProvider methods
 
     def get_system_info(self):
-        if self._version is not None:
+        if self._version:
             yield 'Subversion', self._version
 
     # IRepositoryConnector methods
@@ -368,7 +366,7 @@ class SubversionRepository(Repository):
 
         try:
             self.repos = repos.svn_repos_open(root_path_utf8, self.pool())
-        except core.SubversionException, e:
+        except core.SubversionException as e:
             raise InvalidRepository(
                 _("Couldn't open Subversion repository %(path)s: "
                   "%(svn_error)s", path=to_unicode(path_utf8),
@@ -770,7 +768,7 @@ class SubversionNode(Node):
         else:
             try:
                 self.root = fs.revision_root(self.fs_ptr, rev, pool)
-            except core.SubversionException, e:
+            except core.SubversionException as e:
                 raise NoSuchNode(path, rev, exception_to_unicode(e))
         node_type = fs.check_path(self.root, self._scoped_path_utf8, pool)
         if not node_type in _kindmap:
@@ -866,7 +864,7 @@ class SubversionNode(Node):
                 from svn import client
                 client.blame2(file_url_utf8, rev, start, rev, blame_receiver,
                               client.create_context(), self.pool())
-            except (core.SubversionException, AttributeError), e:
+            except (core.SubversionException, AttributeError) as e:
                 # svn thinks file is a binary or blame not supported
                 raise TracError(_('svn blame failed on %(path)s: %(error)s',
                                   path=self.path, error=to_unicode(e)))
