@@ -15,6 +15,7 @@
 # Author: Christopher Lenz <cmlenz@gmx.de>
 
 from datetime import datetime
+import locale
 import new
 import os.path
 import tempfile
@@ -31,7 +32,7 @@ except ImportError:
 from genshi.core import Stream
 
 import trac.tests.compat
-from trac.test import EnvironmentStub, Mock, MockPerm, TestSetup
+from trac.test import EnvironmentStub, MockRequest, TestSetup
 from trac.core import TracError
 from trac.mimeview.api import RenderingContext
 from trac.resource import Resource, resource_exists
@@ -41,7 +42,6 @@ from trac.versioncontrol.api import DbRepositoryProvider, Changeset, \
                                     InvalidRepository, Node, \
                                     NoSuchChangeset, RepositoryManager
 from trac.web.chrome import web_context
-from trac.web.href import Href
 from tracopt.versioncontrol.svn import svn_fs, svn_prop
 
 
@@ -55,13 +55,55 @@ TETE = 26
 NATIVE_EOL = '\r\n' if os.name == 'nt' else '\n'
 
 
-def _create_context():
-    req = Mock(base_path='', chrome={}, args={}, session={},
-               abs_href=Href('/'), href=Href('/'), locale=None,
-               perm=MockPerm(), authname='anonymous', tz=utc)
+def _create_context(env):
+    req = MockRequest(env)
     return web_context(req)
 
 
+def setlocale(*locales):
+    """Decorator to call test method with each locale.
+    """
+    if os.name != 'nt':
+        locales_map = {'de': 'de_DE.UTF8', 'fr': 'fr_FR.UTF8',
+                       'pl': 'pl_PL.UTF8', 'ja': 'ja_JP.UTF8',
+                       'zh_CN': 'zh_CN.UTF8'}
+    else:
+        locales_map = {'de': 'German_Germany', 'fr': 'French_France',
+                       'pl': 'Polish_Poland', 'ja': 'Japanese_Japan',
+                       'zh_CN': "Chinese_People's Republic of China"}
+    locales_map['C'] = 'C'
+
+    def getlocale():
+        rv = locale.getlocale(locale.LC_ALL)
+        return rv if rv[0] else 'C'
+
+    def setlocale(locale_id):
+        locale_id = locales_map[locale_id]
+        try:
+            locale.setlocale(locale.LC_ALL, locale_id)
+            return True
+        except locale.Error:
+            return False
+
+    def wrap(fn, locales):
+        def wrapped(*args, **kwargs):
+            saved_locale = getlocale()
+            try:
+                for locale_id in locales:
+                    if setlocale(locale_id):
+                        fn(*args, **kwargs)
+            finally:
+                locale.setlocale(locale.LC_ALL, saved_locale)
+        return wrapped
+
+    if len(locales) == 1 and hasattr(locales[0], '__call__'):
+        return wrap(locales[0], sorted(locales_map))
+    else:
+        def decorator(fn):
+            return wrap(fn, locales)
+        return decorator
+
+
 class SubversionRepositoryTestSetup(TestSetup):
 
     def setUp(self):
@@ -133,6 +175,26 @@ class NormalTests(object):
         self.assertRaises(NoSuchChangeset, self.repos.normalize_rev, -1)
         self.assertRaises(NoSuchChangeset, self.repos.normalize_rev, -42)
 
+    def test_repos_display_rev(self):
+        self.assertEqual(str(HEAD), self.repos.display_rev('latest'))
+        self.assertEqual(str(HEAD), self.repos.display_rev('head'))
+        self.assertEqual(str(HEAD), self.repos.display_rev(''))
+        self.assertRaises(NoSuchChangeset,
+                          self.repos.display_rev, 'something else')
+        self.assertEqual(str(HEAD), self.repos.display_rev(None))
+        self.assertEqual('11', self.repos.display_rev('11'))
+        self.assertEqual('11', self.repos.display_rev(11))
+
+    def test_repos_short_rev(self):
+        self.assertEqual(str(HEAD), self.repos.short_rev('latest'))
+        self.assertEqual(str(HEAD), self.repos.short_rev('head'))
+        self.assertEqual(str(HEAD), self.repos.short_rev(''))
+        self.assertRaises(NoSuchChangeset,
+                          self.repos.short_rev, 'something else')
+        self.assertEqual(str(HEAD), self.repos.short_rev(None))
+        self.assertEqual('11', self.repos.short_rev('11'))
+        self.assertEqual('11', self.repos.short_rev(11))
+
     def test_rev_navigation(self):
         self.assertEqual(1, self.repos.oldest_rev)
         self.assertIsNone(self.repos.previous_rev(0))
@@ -332,6 +394,7 @@ Now with fixed width fields:
 En r\xe9sum\xe9 ... \xe7a marche.
 '''.splitlines(), f.get_processed_content().read().splitlines())
 
+    @setlocale
     def test_get_file_content_with_keyword_substitution_25(self):
         f = self.repos.get_node(u'/tête/Résumé.txt', 25)
         props = f.get_properties()
@@ -353,6 +416,7 @@ Now with fixed width fields:
 En r\xe9sum\xe9 ... \xe7a marche.
 '''.splitlines(), f.get_processed_content().read().splitlines())
 
+    @setlocale
     def test_get_file_content_with_keyword_substitution_30(self):
         self.maxDiff = None
         f = self.repos.get_node(u'/branches/v4/Résumé.txt', 30)
@@ -828,7 +892,7 @@ En r\xe9sum\xe9 ... \xe7a marche.
         del test_canonical_repos_path
 
     def test_merge_prop_renderer_without_deleted_branches(self):
-        context = _create_context()
+        context = _create_context(self.env)
         context = context(self.repos.get_node('branches/v1x', HEAD).resource)
         renderer = svn_prop.SubversionMergePropertyRenderer(self.env)
         props = {'svn:mergeinfo': u"""\
@@ -840,7 +904,8 @@ En r\xe9sum\xe9 ... \xe7a marche.
                                                  context, props))
 
         node = unicode(result.select('//tr[1]//td[1]'))
-        self.assertIn(' href="/browser/repo/branches/v2?rev=%d"' % HEAD, node)
+        self.assertIn(' href="/trac.cgi/browser/repo/branches/v2?rev=%d"'
+                      % HEAD, node)
         self.assertIn('>/branches/v2</a>', node)
         node = unicode(result.select('//tr[1]//td[2]'))
         self.assertIn(' title="16"', node)
@@ -850,23 +915,24 @@ En r\xe9sum\xe9 ... \xe7a marche.
         self.assertIn('>eligible</span>', node)
 
         node = unicode(result.select('//tr[3]//td[1]'))
-        self.assertIn(' href="/browser/repo/%s?rev=%d"' % ('t%C3%AAte', HEAD),
-                      node)
+        self.assertIn(' href="/trac.cgi/browser/repo/%s?rev=%d"'
+                      % ('t%C3%AAte', HEAD), node)
         self.assertIn(u'>/tête</a>', node)
         node = unicode(result.select('//tr[3]//td[2]'))
         self.assertIn(' title="1-20, 23-26"', node)
-        self.assertIn(' href="/log/repo/t%C3%AAte?revs=1-20%2C23-26"', node)
+        self.assertIn(' href="/trac.cgi/log/repo/t%C3%AAte?revs=1-20%2C23-26"',
+                      node)
         self.assertIn('>merged</a>', node)
         node = unicode(result.select('//tr[3]//td[3]'))
         self.assertIn(' title="21"', node)
-        self.assertIn(' href="/changeset/21/repo/t%C3%AAte"', node)
+        self.assertIn(' href="/trac.cgi/changeset/21/repo/t%C3%AAte"', node)
         self.assertIn('>eligible</a>', node)
 
         self.assertNotIn('(toggle deleted branches)', unicode(result))
         self.assertNotIn('False', unicode(result))  # See #12125
 
     def test_merge_prop_renderer_with_deleted_branches(self):
-        context = _create_context()
+        context = _create_context(self.env)
         context = context(self.repos.get_node('branches/v1x', HEAD).resource)
         renderer = svn_prop.SubversionMergePropertyRenderer(self.env)
         props = {'svn:mergeinfo': u"""\
@@ -878,7 +944,8 @@ En r\xe9sum\xe9 ... \xe7a marche.
                                                  context, props))
 
         node = unicode(result.select('//tr[1]//td[1]'))
-        self.assertIn(' href="/browser/repo/branches/v3?rev=%d"' % HEAD, node)
+        self.assertIn(' href="/trac.cgi/browser/repo/branches/v3?rev=%d"'
+                      % HEAD, node)
         self.assertIn('>/branches/v3</a>', node)
         node = unicode(result.select('//tr[1]//td[2]'))
         self.assertIn(' title="22"', node)
@@ -888,16 +955,16 @@ En r\xe9sum\xe9 ... \xe7a marche.
         self.assertIn('>eligible</span>', node)
 
         node = unicode(result.select('//tr[2]//td[1]'))
-        self.assertIn(' href="/browser/repo/%s?rev=%d"' % ('t%C3%AAte', HEAD),
-                      node)
+        self.assertIn(' href="/trac.cgi/browser/repo/%s?rev=%d"'
+                      % ('t%C3%AAte', HEAD), node)
         self.assertIn(u'>/tête</a>', node)
         node = unicode(result.select('//tr[2]//td[2]'))
         self.assertIn(' title="19"', node)
-        self.assertIn(' href="/changeset/19/repo/t%C3%AAte"', node)
+        self.assertIn(' href="/trac.cgi/changeset/19/repo/t%C3%AAte"', node)
         self.assertIn('>merged</a>', node)
         node = unicode(result.select('//tr[2]//td[3]'))
         self.assertIn(' title="13-14, 17-18, 20-21, 23-26"', node)
-        self.assertIn(' href="/log/repo/t%C3%AAte?revs='
+        self.assertIn(' href="/trac.cgi/log/repo/t%C3%AAte?revs='
                       '13-14%2C17-18%2C20-21%2C23-26"', node)
         self.assertIn('>eligible</a>', node)
 
@@ -908,7 +975,7 @@ En r\xe9sum\xe9 ... \xe7a marche.
                       unicode(result.select('//tr[3]//td[2]')))
 
     def test_merge_prop_diff_renderer_added(self):
-        context = _create_context()
+        context = _create_context(self.env)
         old_context = context(self.repos.get_node(u'tête', 20).resource)
         old_props = {'svn:mergeinfo': u"""\
 /branches/v2:1,8-9,12-15
@@ -928,15 +995,15 @@ En r\xe9sum\xe9 ... \xe7a marche.
                 new_props, options))
 
         node = unicode(result.select('//tr[1]//td[1]'))
-        self.assertIn(' href="/browser/repo/branches/v2?rev=21"', node)
+        self.assertIn(' href="/trac.cgi/browser/repo/branches/v2?rev=21"', node)
         self.assertIn('>/branches/v2</a>', node)
         node = unicode(result.select('//tr[1]//td[2]'))
         self.assertIn(' title="16"', node)
-        self.assertIn(' href="/changeset/16/repo/branches/v2"', node)
+        self.assertIn(' href="/trac.cgi/changeset/16/repo/branches/v2"', node)
 
     def test_merge_prop_diff_renderer_wrong_mergeinfo(self):
         rev = HEAD
-        context = _create_context()
+        context = _create_context(self.env)
         old_context = context(self.repos.get_node(u'tête', rev - 1).resource)
         old_mergeinfo = '/missing:12\n'
         new_context = context(self.repos.get_node(u'tête', rev).resource)
@@ -947,16 +1014,18 @@ En r\xe9sum\xe9 ... \xe7a marche.
             new_context, {'svn:mergeinfo': new_mergeinfo}, {}))
 
         node = unicode(result.select('//tr[1]//td[1]'))
-        self.assertIn(' href="/browser/repo/missing?rev=%d"' % rev, node)
+        self.assertIn(' href="/trac.cgi/browser/repo/missing?rev=%d"'
+                      % rev, node)
         self.assertIn('>/missing</a>', node)
         node = unicode(result.select('//tr[1]//td[2]'))
         self.assertIn(' title="13-15"', node)
-        self.assertIn(' href="/log/repo/missing?revs=13-15"', node)
+        self.assertIn(' href="/trac.cgi/log/repo/missing?revs=13-15"', node)
 
     def test_merge_prop_diff_renderer_added_svnmerge_integrated(self):
-        # Property diff of svnmerge-integrated property (from svnmerge.py,
-        # used prior to svn 1.5) is rendered correctly.
-        context = _create_context()
+        """Property diff of svnmerge-integrated property (from
+        svnmerge.py, used prior to svn 1.5) is rendered correctly.
+        """
+        context = _create_context(self.env)
         old_context = context(self.repos.get_node(u'tête', 20).resource)
         old_props = {'svnmerge-integrated': u"""\
         /branches/v2:1,8-9,12-15 /branches/v1x:12 /branches/deleted:1,3-5,22
@@ -972,16 +1041,18 @@ En r\xe9sum\xe9 ... \xe7a marche.
             new_props, {}))
 
         node = unicode(result.select('//tr[1]//td[1]'))
-        self.assertIn(' href="/browser/repo/branches/v2?rev=21"', node)
+        self.assertIn(' href="/trac.cgi/browser/repo/branches/v2?rev=21"',
+                      node)
         self.assertIn('>/branches/v2</a>', node)
         node = unicode(result.select('//tr[1]//td[2]'))
         self.assertIn(' title="16"', node)
-        self.assertIn(' href="/changeset/16/repo/branches/v2"', node)
+        self.assertIn(' href="/trac.cgi/changeset/16/repo/branches/v2"',
+                      node)
 
     def test_render_needslock(self):
         htdocs_location = 'http://assets.example.org/common'
         self.env.config.set('trac', 'htdocs_location', htdocs_location)
-        context = _create_context()
+        context = _create_context(self.env)
         context.req.chrome['htdocs_location'] = htdocs_location
         context = context(self.repos.get_node(u'tête', HEAD).resource)
         renderer = svn_prop.SubversionPropertyRenderer(self.env)
@@ -1441,7 +1512,34 @@ class SvnCachedRepositoryTestCase(unittest.TestCase):
         RepositoryManager(self.env).reload_repositories()
 
 
-def suite():
+class SubversionConnectorTestCase(unittest.TestCase):
+
+    def setUp(self):
+        self.env = EnvironmentStub()
+        self.repos_path = tempfile.mkdtemp(prefix='trac-svnrepos-')
+        self.dbprovider = DbRepositoryProvider(self.env)
+        pool = core.svn_pool_create(None)
+        repos.svn_repos_create(self.repos_path, '', '', None, None, pool)
+        self.dbprovider.add_repository(REPOS_NAME, self.repos_path, 'svn')
+
+    def tearDown(self):
+        self.env.reset_db()
+        # clear cached repositories to avoid TypeError on termination (#11505)
+        RepositoryManager(self.env).reload_repositories()
+        repos.svn_repos_delete(self.repos_path)
+
+    def _svn_version_from_system_info(self):
+        svn_version = None
+        for name, version in self.env.get_systeminfo():
+            if name == 'Subversion':
+                svn_version = version
+        return svn_version
+
+    def test_get_system_info(self):
+        self.assertIsNotNone(self._svn_version_from_system_info())
+
+
+def test_suite():
     global REPOS_PATH
     suite = unittest.TestSuite()
     if has_svn:
@@ -1476,10 +1574,11 @@ def suite():
                 setattr(tc, skip, lambda self: None) # no skip, so we cheat...
             suite.addTest(unittest.makeSuite(
                 tc, suiteClass=SubversionRepositoryTestSetup))
+        suite.addTest(unittest.makeSuite(SubversionConnectorTestCase))
     else:
         print("SKIP: tracopt/versioncontrol/svn/tests/svn_fs.py (no svn "
               "bindings)")
     return suite
 
 if __name__ == '__main__':
-    unittest.main(defaultTest='suite')
+    unittest.main(defaultTest='test_suite')
